 /* AuxCycle.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  Sergio Atzori, 2010
 *
 *
 */ 
 
#include <QtGui>

#include "AuxCycle.h"
#include "WinCycle.h"

AuxCycle::AuxCycle(WinCycle *parent) {
  appName = "Cyclone";
  if (SUB_VERS > 0)
    appVersion = QString("%1.%2.%3") .arg(VERS_MAJ) .arg(VERS_MIN) .arg(SUB_VERS);
  else
    appVersion = QString("%1.%2") .arg(VERS_MAJ) .arg(VERS_MIN);
  appOrg = "rEvol";
  appProj = "Cyclone_Project";
  appAuthor = "Sergio Atzori";
  main = parent;
  reset();
  pasteL = false;
  pasteS = false;
}

void AuxCycle::reset() {
  rowHeight = 26;
  graphicsfx = 1;
  graphicsfxnext = 5;
  grsys = "raster";
  restoreLastSession = 0;
  standardDebug = 0;
  standardSL = Record;
  standardS = EndLoop;
  standardV = 100;
  standardBuffer = 512;
  standardQuickfade = 50;
  sooloovol = 90;
  colid = 30;
  colname = 220;
  colsl = 100;
  cols = 60;
  colvol = 125;
  colbtn = 36;
  colsel = 48;  
}

void AuxCycle::writeSettings() {
  QSettings settings(appOrg, appName);

  settings.setValue("geometry", main->saveGeometry());
  settings.beginGroup("Column Width");
  settings.setValue("colid", colid);
  settings.setValue("colname", colname);
  settings.setValue("colsl", colsl);
  settings.setValue("cols", cols);
  settings.setValue("colvol", colvol);
  settings.setValue("colbtn", colbtn);
  settings.setValue("colsel", colsel);
  settings.endGroup();
  settings.setValue("graphic_system", grsys);
  settings.setValue("standard_location", standardLocation);
  settings.setValue("standard_location_project", standardLocationProject);
  settings.setValue("project_name", projectName);
  settings.setValue("standard_SLmode", slmodeToIndex(standardSL));
  settings.setValue("standard_Smode", smodeToIndex(standardS));
  settings.setValue("standard_LoopVolume", standardV);
  settings.setValue("standard_Buffer", standardBuffer);
  settings.setValue("standard_Quickfade_time", standardQuickfade);
  settings.setValue("standard_Debug", standardDebug);
  settings.setValue("restore_last_session", restoreLastSession);
  settings.setValue("sooperlooper_volume", sooloovol);
  if (graphicsfxnext < 5)
    settings.setValue("graphics_fx", graphicsfxnext);  
  else
    settings.setValue("graphics_fx", graphicsfx);
  if (!currentFile.isEmpty())
    settings.setValue("restore_last_session_path", currentFile);
}

int AuxCycle::slmodeToIndex(slmode n) {
  int index;
  switch (n) { /*@@@ Modify when adding new sl functions @@@*/
    case Record:
      index = 2;
      break;
    case Multiply:
      index = 1;
      break;
    case RecOverdub:
      index = 3;
      break;
    case MultiOverdub:
      index = 4;
      break;
    case None:
      index = 0;
      break;
  }
  return index;
}

int AuxCycle::smodeToIndex(smode n) {
  int index;
  switch (n) { /*@@@ Modify when adding new s functions @@@*/
    case Instantaneous:
      index = 0;
      break;
    case EndLoop:
      index = 1;
      break;
    case Once:
      index = 2;
      break;
    case Asynchronous:
      index = 3;
  }
  return index;  
}

slmode AuxCycle::indexToSlmode(int n) {
  slmode index;
  switch (n) { /*@@@ Modify when adding new sl functions @@@*/
    case 2:
      index = Record;
      break;
    case 1:
      index = Multiply;
      break;
    case 3:
      index = RecOverdub;
      break;
    case 4:
      index = MultiOverdub;
      break;
    case 0:
      index = None;
      break;
  }
  return index;  
}

smode AuxCycle::indexToSmode(int n) {
  smode index;
  switch (n) { /*@@@ Modify when adding new s functions @@@*/
    case 0:
      index = Instantaneous;
      break;
    case 1:
      index = EndLoop;
      break;
    case 2:
      index = Once;
      break;
    case 3:
      index = Asynchronous;
      break;
  }
  return index;  
}

void AuxCycle::readSettings() {
  proj = new QDomDocument(appName);
  
  QSettings settings(appOrg, appName);

  main->restoreGeometry(settings.value("geometry").toByteArray());
  colid = settings.value("Column Width/colid", false).toInt();
  colname = settings.value("Column Width/colname", false).toInt();
  colsl = settings.value("Column Width/colsl", false).toInt();
  cols = settings.value("Column Width/cols", false).toInt();
  colvol = settings.value("Column Width/colvol", false).toInt();
  colbtn = settings.value("Column Width/colbtn", false).toInt();
  colsel = settings.value("Column Width/colsel", false).toInt();
  if ((colid + colname + colsl + cols + colvol + colbtn + colsel) < 200)
    reset();
  grsys = settings.value("graphic_system", false).toString();
  if (grsys == "false")
    grsys = "raster";
  if (settings.value("graphics_fx", false) != "false")
    graphicsfx = settings.value("graphics_fx").toInt();
  standardLocation = settings.value("standard_location", false).toString();
  if (standardLocation == "false")
    standardLocation = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);    
  standardLocationProject = settings.value("standard_location_project", false).toString();
  if (standardLocationProject == "false")
    standardLocationProject = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
  projectName = settings.value("project_name").toString();
  if (projectName == "")
    projectName = "Untitled";
  if (settings.value("standard_SLmode", false) != "false")
    standardSL = indexToSlmode(settings.value("standard_SLmode").toInt());
  if (settings.value("standard_Smode", false) != "false")
    standardS = indexToSmode(settings.value("standard_Smode").toInt());
  if (settings.value("standard_LoopVolume", false) != "false")
    standardV = settings.value("standard_LoopVolume").toInt();
  if (settings.value("standard_Buffer", false) != "false")
    standardBuffer = settings.value("standard_Buffer").toInt();
  if (settings.value("standard_Quickfade_time", false) != "false")
    standardQuickfade = settings.value("standard_Quickfade_time").toInt();
  if (settings.value("standard_Debug", false) != "false")
    standardDebug = settings.value("standard_Debug").toInt();
  if (settings.value("restore_last_session", false) != "false")
    restoreLastSession = settings.value("restore_last_session").toInt();
  if (settings.value("sooperlooper_volume", false) != "false")
    sooloovol = settings.value("sooperlooper_volume").toInt();
  if ((settings.value("restore_last_session_path", false) != "false") && (restoreLastSession == 1)) 
    currentFile = settings.value("restore_last_session_path").toString();
  else
    currentFile = "";
}

void AuxCycle::reloadLastSession() {
  open(currentFile);
}

void AuxCycle::setupXml() {
  proj->clear();
  QDomElement rad = proj->createElement(appProj);
  rad.setAttribute("version", appVersion);
  proj->appendChild(rad);
  QDomElement settings = proj->createElement("Settings");
  rad.appendChild(settings);
  QDomElement set = GlobalToNode();
  settings.appendChild(set);
  QDomElement loops = proj->createElement(projectName);
  rad.appendChild(loops);
  QDomElement lps = CyLoopToNode();
  loops.appendChild(lps);
}

QDomElement AuxCycle::CyLoopToNode() {
  QDomElement lp = proj->createElement("Elements");
  QDomElement qde;
  QDomElement qdey;
  for (int y = 0; y < main->sessions->sessionsCount; y++) {
    if (main->sessions->loopCount(y) > 0) {
      qdey = proj->createElement("Session");
      qdey.setAttribute("name", main->sessions->sessionName(y));
      for (int x = 0; x < main->sessions->loopCount(y); x++) {
	qde = setCyLoop(main->sessions->loopz[x][y], x, y);
	qdey.appendChild(qde);
      } 
      lp.appendChild(qdey);
    }
  }
  return lp;
}

QDomElement AuxCycle::setCyLoop(CyLoop *loop, int x, int y) {
  QDomElement header = proj->createElement("Loop");
  header.setAttribute("index", x);
  header.setAttribute("session", y);
  
  QDomElement index = proj->createElement("index");
  index.setAttribute("param", loop->getIndex());
  QDomElement name = proj->createElement("name");
  name.setAttribute("param", loop->getName());
  QDomElement path = proj->createElement("path");
  path.setAttribute("param", loop->loopPath());
  QDomElement slmode = proj->createElement("sooperlooper_mode");
  switch (loop->getSLmode()) { /*@@@ Modify when adding new sl functions @@@*/
    case Record:
      slmode.setAttribute("param", "Record");
      break;
    case Multiply:
      slmode.setAttribute("param", "Multiply");
      break;
    case RecOverdub:
      slmode.setAttribute("param", "RecOverdub");
      break;
    case MultiOverdub:
      slmode.setAttribute("param", "MultiOverdub");
      break;
    case None:
      slmode.setAttribute("param", "None");
      break;
  }
  QDomElement smode = proj->createElement("end_mode");
  switch (loop->getSmode()) { /*@@@ Modify when adding new s functions @@@*/
    case EndLoop:
      smode.setAttribute("param", "EndLoop");
      break;
    case Instantaneous:
      smode.setAttribute("param", "Instantaneous");
      break;
    case Once:
      smode.setAttribute("param", "Once");
      break;
    case Asynchronous:
      smode.setAttribute("param", "Asynchronous");
  }
  QDomElement volume = proj->createElement("volume");
  volume.setAttribute("param", loop->getVolume());
  QDomElement midi = proj->createElement("midi_command");
  midiPlayFunction mpf = loop->getMidi();
  if (mpf.fn == PC)
    midi.setAttribute("fn", "PC");
  else
    midi.setAttribute("fn", "NT");
  midi.setAttribute("ch", mpf.ch);
  midi.setAttribute("val", mpf.val);
  QDomElement midiv = proj->createElement("midi_command_volume");
  midiVolumeFunction mvf = loop->getMidiV();
  midiv.setAttribute("ch", mvf.ch);
  midiv.setAttribute("val", mvf.val);
  header.appendChild(index);
  header.appendChild(name);
  header.appendChild(path);
  header.appendChild(slmode);
  header.appendChild(smode);
  header.appendChild(volume);
  header.appendChild(midi);
  header.appendChild(midiv);
  
  return header;
}

QDomElement AuxCycle::GlobalToNode() {
  QDomElement set = proj->createElement("Interface_Settings");

  set.setAttribute("GlobalVolume", main->volume->value());
  set.appendChild(midicmdToNode(main->globalmidifunctions[0], "GlobalVolumeMidiCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[4], "GlobalNextSessionMidiCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[6], "GlobalPrevSessionMidiCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[5], "GlobalStartMidiCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[3], "GlobalNextLoopCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[1], "GlobalPrevLoopCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[2], "GlobalPlayLoopCommand"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[7], "GlobalFirst"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[8], "GlobalRew"));
  //set.appendChild(midicmdToNode(main->globalmidifunctions[9], "GlobalRec"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[10], "GlobalFwd"));
  set.appendChild(midicmdToNode(main->globalmidifunctions[11], "GlobalLast"));
  //set.appendChild(midicmdToNode(main->globalmidifunctions[12], "Global?????????????"));
  
  return set;
}

QDomElement AuxCycle::midicmdToNode(midicmd mc, QString name) {
  QDomElement qmc = proj->createElement(QString("MidiCommand_%1") .arg(name));
  QDomElement type = proj->createElement("type");
  type.setAttribute("param", mc.type);
  qmc.appendChild(type);
  QDomElement num = proj->createElement("num");
  num.setAttribute("param", mc.num);
  qmc.appendChild(num);
  QDomElement ch = proj->createElement("ch");
  ch.setAttribute("param", mc.ch);
  qmc.appendChild(ch);
  QDomElement val = proj->createElement("val");
  val.setAttribute("param", mc.val);
  qmc.appendChild(val);
  
  return qmc;
}

midicmd AuxCycle::nodeToMidicmd(QDomElement *qdome) {
  midicmd mcmd;
  
  QDomNode set = qdome->toElement();
  QDomElement type = set.firstChildElement("type");
  mcmd.type = type.attribute("param", "").toInt();
  set.nextSibling();
  QDomElement num = set.firstChildElement("num");
  mcmd.num = num.attribute("param", "").toInt();
  set.nextSibling();
  QDomElement ch = set.firstChildElement("ch");
  mcmd.ch = ch.attribute("param", "").toInt();
  set.nextSibling();
  QDomElement val = set.firstChildElement("val");
  mcmd.val = val.attribute("param", "").toInt();
  
  return mcmd;
}

void AuxCycle::openFile() {
  QString projx = QFileDialog::getOpenFileName(main, tr("Select File"), standardLocationProject, tr("Cyclone Project(%1);; All Files (%2)") .arg("*.cyp") .arg("*.*"));
  if (projx == "")
    return;
  if (testOpen(projx))
    open(projx);
}

bool AuxCycle::testOpen(QString prj) {
  bool precedente = false;
  QApplication::setOverrideCursor(Qt::WaitCursor);
  QFileInfo qfi(prj);
  QString filepath = qfi.absoluteFilePath();
  if (!qfi.exists()) {
    openError(tr("Cannot find file.\n\n%1") .arg(filepath));
    return false;
  }
  
  QFile file(prj);
  if (!file.open(QIODevice::ReadOnly)) {
    openError(tr("Cannot open file.\n\n%1") .arg(filepath));
    file.close();
    return false;
  }
  
  QString err;
  int errl;
  int errc;
  if (!proj->setContent(&file, &err, &errl, &errc)) {
    openError(tr("Not valid xml encoding.\n%1 line:%2 column:%3\n\n%4") .arg(err) .arg(errl) .arg(errc) .arg(filepath));
    file.close();
    return false;
  }  
  
  QDomElement rad = proj->documentElement();
  if (rad.tagName() != appProj) {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(rad.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  QStringList vf = rad.attribute("version", "").split(QRegExp("\\."));
  QStringList vp = appVersion.split(QRegExp("\\."));
  if (vf[1].toInt() > vp[1].toInt()) {
    if (vf[0].toInt() >= vp[0].toInt()) {
      openError(tr("Cannot open project files with version superior than %1.%2") .arg(vp[0]) .arg(vp[1]));
      file.close();
      return false;
    }
  }
  QString flvers = rad.attribute("version", "");
  /* Read Global Settings */
  QDomNode setz = rad.firstChild();
  QDomElement setting = setz.toElement();
  if (setting.tagName() != "Settings") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(setting.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  QDomNode intset = setting.firstChild();
  intsetting = intset.toElement();
  if (intsetting.tagName() != "Interface_Settings") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(intsetting.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  gvmc = intset.firstChildElement("MidiCommand_GlobalVolumeMidiCommand");
  if (gvmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(gvmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  nsmc = intset.firstChildElement("MidiCommand_GlobalNextSessionMidiCommand");
  if (nsmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(nsmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  psmc = intset.firstChildElement("MidiCommand_GlobalPrevSessionMidiCommand");
  if (psmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(psmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  gpmc = intset.firstChildElement("MidiCommand_GlobalStartMidiCommand");
  if (gpmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(gpmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  nlmc = intset.firstChildElement("MidiCommand_GlobalNextLoopCommand");
  if (nlmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(nlmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  plmc = intset.firstChildElement("MidiCommand_GlobalPrevLoopCommand");
  if (plmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(plmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  ppmc = intset.firstChildElement("MidiCommand_GlobalPlayLoopCommand");
  if (ppmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(ppmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  firstmc = intset.firstChildElement("MidiCommand_GlobalFirst");
  if (firstmc.nodeName() == "") {
    precedente = true;
  }
  
  rewmc = intset.firstChildElement("MidiCommand_GlobalRew");
  if (rewmc.nodeName() == "") {
    precedente = true;
  }
  /*
  recmc = intset.firstChildElement("MidiCommand_GlobalRec");
  if (recmc.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(recmc.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  */
  fwdmc = intset.firstChildElement("MidiCommand_GlobalFwd");
  if (fwdmc.nodeName() == "") {
    precedente = true;
  }
  
  lastmc = intset.firstChildElement("MidiCommand_GlobalLast");
  if (lastmc.nodeName() == "") {
    precedente = true;
  }
  
  if (precedente)
    openError(tr("This file is generated by a previous version of %1 (%2),\nsaving it will be upgraded to version %3") .arg(appName) .arg(flvers) .arg(appVersion));
  
  QDomElement name = rad.firstChildElement(qfi.baseName());
  if (name.tagName() != qfi.baseName()) {
    openError(tr("Filename not equivalent to project name.\nTAG: %1\n\n%2") .arg(name.tagName()) .arg(filepath));
    file.close();
    return false;
  }
  
  elements = name.firstChild();
  if (elements.nodeName() == "") {
    openError(tr("Not a valid project file.\nTAG: %1\n\n%2") .arg(elements.nodeName()) .arg(filepath));
    file.close();
    return false;
  }
  
  file.close();
  return isPl();
}

bool AuxCycle::isPl() {
  bool ok = false;
  bool catched = false;
  for (int ss = 0; ss < main->sessions->sessionsCount; ss++) {
    for (int lp = 0; lp < main->sessions->musicTable[ss]->rowCount(); lp++) {
      if ((main->sessions->loopz[lp][ss]->state != 0) && (main->sessions->loopz[lp][ss]->state != 5))
	catched = true;
    }
  }
  if (catched) {
    if (QMessageBox::warning(main, tr("Cyclone - Information"), tr("This operation will stop audio. Do you want to proceed?"), QMessageBox::Ok, QMessageBox::Cancel) == QMessageBox::Ok) {
      ok = true;  
    } else {
      QApplication::restoreOverrideCursor();  
    }
  } else {
    ok = true;
  }
  return ok;
}
  
void AuxCycle::open(QString prj) {
  if (!main->closeProject()) {
    QApplication::restoreOverrideCursor();  
    return;
  }
  
  QFileInfo qfi(prj);  
  QDomNode sessions = elements.firstChild();
  int scount = 0;
  int lcount = 0;
  while (!sessions.isNull()) {
    QDomElement ses = sessions.toElement();
    if (!ses.isNull()) {
      main->sessions->addSession(0, scount, ses.attribute("name", ""));
    }
    main->sessions->setSession(scount);
    QDomNode loop = ses.firstChild();
    while (!loop.isNull()) {
      QDomElement loo = loop.toElement();
      main->sessions->loopz[lcount][scount] = new CyLoop(setupCyLoop(loo, lcount, scount));
      if (main->sessions->loopz[lcount][scount] != NULL) {
	if (main->sessions->addLoop((main->sessions->loopz[lcount][scount]), scount))
	  lcount++;
      }
      loop = loop.nextSibling();
    }
    main->cymidi->refresh(scount);
    lcount = 0;
    scount++;
    sessions = sessions.nextSibling();
  }
  main->sessions->infinite_count = scount;
  main->sessions->sessionsCount = scount;
  main->sessions->setSession(0);
  
  midicmd mc1 = nodeToMidicmd(&gvmc);
  main->globalmidifunctions[0] = mc1;
  midicmd mc2 = nodeToMidicmd(&nsmc);
  main->globalmidifunctions[4] = mc2;
  midicmd mc3 = nodeToMidicmd(&psmc);
  main->globalmidifunctions[6] = mc3;
  midicmd mc4 = nodeToMidicmd(&gpmc);
  main->globalmidifunctions[5] = mc4;
  midicmd mc5 = nodeToMidicmd(&nlmc);
  main->globalmidifunctions[3] = mc5;
  midicmd mc6 = nodeToMidicmd(&plmc);
  main->globalmidifunctions[1] = mc6;
  midicmd mc7 = nodeToMidicmd(&ppmc);
  main->globalmidifunctions[2] = mc7; 
  midicmd mc8 = nodeToMidicmd(&firstmc);
  main->globalmidifunctions[7] = mc8;
  midicmd mc9 = nodeToMidicmd(&rewmc);
  main->globalmidifunctions[8] = mc9;
  /*midicmd mc10 = nodeToMidicmd(&recmc);
  main->globalmidifunctions[9] = mc10;*/
  midicmd mc11 = nodeToMidicmd(&fwdmc);
  main->globalmidifunctions[10] = mc11;
  midicmd mc12 = nodeToMidicmd(&lastmc);
  main->globalmidifunctions[11] = mc12;
  /*midicmd mc13 = nodeToMidicmd(&);
  main->globalmidifunctions[12] = mc13;*/
  
  main->cymidi->refreshGlobal();
  
  main->volume->setValue(intsetting.attribute("GlobalVolume", "").toInt());
  main->undo->oldvol = main->volume->value();
  standardLocationProject = qfi.dir().absolutePath();
  currentFile = prj;
  projectName = qfi.baseName();  
  emit titleChanged(projectName);
  proj->~QDomDocument();
  proj = new QDomDocument(appName);
  main->start();
  QApplication::restoreOverrideCursor();
}

void AuxCycle::openError(QString error) {
  QMessageBox::warning(main, tr("Cyclone - Open"), error, QMessageBox::Cancel);
  QApplication::restoreOverrideCursor();
  return;
}

bool AuxCycle::save(QString prj) {
  QFileInfo qfi(prj);
  standardLocationProject = qfi.dir().absolutePath();
  QFile file(prj);
  
  if (!file.open(QIODevice::WriteOnly)) {
    QMessageBox::warning(main, tr("Cyclone - Save"),
                        tr("Cannot open file for write."), QMessageBox::Cancel);
    return false;
  }
  projectName = qfi.baseName();
  proj->clear();
  setupXml();
  QTextStream ts(&file);
  ts << proj->toString();
  currentFile = prj;
  file.close();
  emit titleChanged(projectName);
  proj->~QDomDocument();
  proj = new QDomDocument(appName);
  return true;
}


bool AuxCycle::saveFile() {
  bool risp;
  if (currentFile.isEmpty())
    risp = saveFileAs();
  else
    risp = save(currentFile);
  return risp;
}

bool AuxCycle::saveFileAs() {
  bool catched = false;
  for (int y = 0; y < main->sessions->sessionsCount; y++) {
    for (int x = 0; x < main->sessions->loopCount(y); x++) {
      catched = true;
    } 
  }
  if (!catched) {
    QMessageBox::warning(main, tr("Cyclone - Save"),
                        tr("No Loops in this Project."), QMessageBox::Cancel);
    return false;
  }
  QString prj = QFileDialog::getSaveFileName(main, tr("Select File Name"), (standardLocationProject + "/" + projectName), tr("Cyclone Project(%1);; All Files (%2)") .arg("*.cyp") .arg("*.*"));
  if (prj == "")
    return false;
  
  if (!prj.endsWith(".cyp", Qt::CaseInsensitive))
    prj += ".cyp";
  
  return save(prj);
}

CyLoop *AuxCycle::setupCyLoop(QDomElement loo, int row, int ssn) {
  CyLoop *loop;
  
  QDomElement index = loo.firstChildElement("index");
  if (index.tagName() == "") {
    openError(tr("Error reading loop index on session %1") .arg(ssn));
    return NULL;
  }
  
  QDomElement name = loo.firstChildElement("name");
  if (name.tagName() == "") {
    openError(tr("Error reading loop name on session %1") .arg(ssn));
    return NULL;
  }
  
  QDomElement path = loo.firstChildElement("path");
  if (path.tagName() == "") {
    openError(tr("Error reading loop path on session %1") .arg(ssn));
    return NULL;
  }
  
  QDomElement sooperlooper_mode = loo.firstChildElement("sooperlooper_mode");
  if (sooperlooper_mode.tagName() == "") {
    openError(tr("Error reading loop sooperlooper_mode on session %1") .arg(ssn));
    return NULL;
  }

  QDomElement end_mode = loo.firstChildElement("end_mode");
  if (end_mode.tagName() == "") {
    openError(tr("Error reading loop end_mode on session %1") .arg(ssn));
    return NULL;
  }

  QDomElement volume = loo.firstChildElement("volume");
  if (volume.tagName() == "") {
    openError(tr("Error reading loop volume on session %1") .arg(ssn));
    return NULL;
  }

  QDomElement midi_command = loo.firstChildElement("midi_command");
  if (midi_command.tagName() == "") {
    openError(tr("Error reading loop midi_command on session %1") .arg(ssn));
    return NULL;
  }

  QDomElement midi_command_volume = loo.firstChildElement("midi_command_volume");
  if (midi_command_volume.tagName() == "") {
    openError(tr("Error reading loop midi_command_volume on session %1") .arg(ssn));
    return NULL;
  }
  
  QString looppath = path.attribute("param", "");
  loop = new CyLoop(looppath, row, ssn);
  loop->setIndex(row + 1);
  loop->setName(name.attribute("param", ""));
  /*@@@ read SL functions @@@*/
  if (sooperlooper_mode.attribute("param", "") == "Record")
    loop->setSLmode(Record);
  if (sooperlooper_mode.attribute("param", "") == "Multiply")
    loop->setSLmode(Multiply);
  if (sooperlooper_mode.attribute("param", "") == "RecOverdub")
    loop->setSLmode(RecOverdub);
  if (sooperlooper_mode.attribute("param", "") == "MultiOverdub")
    loop->setSLmode(MultiOverdub);
  if (sooperlooper_mode.attribute("param", "") == "None")
    loop->setSLmode(None);
  /*@@@ read S functions @@@*/
  if (end_mode.attribute("param", "") == "EndLoop")
    loop->setSmode(EndLoop);
  if (end_mode.attribute("param", "") == "Instantaneous")
    loop->setSmode(Instantaneous);
  if (end_mode.attribute("param", "") == "Once")
    loop->setSmode(Once);
  if (end_mode.attribute("param", "") == "Asynchronous")
    loop->setSmode(Asynchronous);
  
  loop->setVolume(volume.attribute("param", "").toInt());
  
  midiPlayFunction mpf;
  if (midi_command.attribute("fn", "") == "PC")
    mpf.fn = PC;
  else
    mpf.fn = NT;
  mpf.ch = midi_command.attribute("ch", "").toInt();
  mpf.val = midi_command.attribute("val", "").toInt();
  loop->setMidi(mpf);
  
  midiVolumeFunction mvf;
  mvf.ch = midi_command_volume.attribute("ch", "").toInt();
  mvf.val = midi_command_volume.attribute("val", "").toInt();
  loop->setMidiV(mvf);  
  if (standardDebug) {
    QString string = QString("********* %1 %2\n") .arg(loop->getIndex()) .arg(loop->getName());
    string += QString("** loopindex: index:%1 session:%2\n") .arg(loop->loopindex.index) .arg(loop->loopindex.session);
    string += QString("** path: %1\n") .arg(loop->loopPath());
    string += QString("** slmode: %1\n") .arg(slmodeToIndex(loop->getSLmode()));
    string += QString("** smode: %1\n") .arg(smodeToIndex(loop->getSmode()));
    string += QString("** volume: %1\n") .arg(loop->getVolume());
    midiPlayFunction z1 = loop->getMidi();
    QString tp;
    if (z1.fn == NT)
      tp = "NT";
    else
      tp = "PC";
    string += QString("** midi_play: type:%1 ch:%2 val:%3\n") .arg(tp) .arg(z1.ch) .arg(z1.val);
    midiVolumeFunction z2 = loop->getMidiV();
    string += QString("** midi_volume: ch:%1 val:%2\n") .arg(z2.ch) .arg(z2.val);
    string += QString("*************************\n");
    if (standardDebug)
      main->printOutput(string);
  }
  return loop;
}

void AuxCycle::copyLoop() {
  QString path = main->sessions->loopz[main->sessions->currentLoop][main->sessions->currentSession]->loopPath();
  QSize coord = main->sessions->generateCoords();
  pasteCyLoop = new CyLoop(path, coord.width(), coord.height());
  pasteCyLoop->setSLmode(main->sessions->loopz[main->sessions->currentLoop][main->sessions->currentSession]->getSLmode());
  pasteCyLoop->setSmode(main->sessions->loopz[main->sessions->currentLoop][main->sessions->currentSession]->getSmode());
  pasteCyLoop->setVolume(main->sessions->loopz[main->sessions->currentLoop][main->sessions->currentSession]->getVolume());
  pasteCyLoop->setMidi(main->sessions->loopz[main->sessions->currentLoop][main->sessions->currentSession]->getMidi());
  pasteCyLoop->setMidiV(main->sessions->loopz[main->sessions->currentLoop][main->sessions->currentSession]->getMidiV());
  pasteL = true;
  if (standardDebug)
      main->printOutput(QString("Copying Loop: %1") .arg(pasteCyLoop->passName()));
  main->refreshPaste();
}

void AuxCycle::pasteLoop() {
  if (main->sessions->loopCount(main->sessions->currentSession) >= 128) {
    QMessageBox::warning(main, tr("Cyclone - Add Loops"),
		  tr("Maximum number of loop for this session reached.\n"
		      "Please create a new session or delete loops."), QMessageBox::Cancel);
    return;
  }
  if (standardDebug)
    main->printOutput(QString("Pasting Loop: %1") .arg(pasteCyLoop->passName()));
  if (pasteL)
    main->undo->push(new CommandPasteLoop(main));
}

bool AuxCycle::verifyCoords(int x, int y, int coord) {
  if (coord == 0)
    return true;
  bool ok = true;
  for (int ziz = 0; ziz < coord; ziz++) {
    if ((pasteCyTable[ziz]->loopindex.index == x) && (pasteCyTable[ziz]->loopindex.session == y)) {
      ok = false;
      if (standardDebug)
	main->printOutput(QString("AUX: Index/Session paste_loop catched %1") .arg(ziz));
    }
  }
  return ok;
}

void AuxCycle::copySession() {
  QApplication::setOverrideCursor(Qt::WaitCursor);
  int numloop = main->sessions->loopCount(main->sessions->currentSession);
  pctCount = numloop;
  int curses = main->sessions->currentSession;
  if (numloop < 1) {
    QApplication::restoreOverrideCursor();
    return;
  }
  QSize coord = main->sessions->generateCoords();
  int x, y;
  for (int it = 0; it < numloop; it++) {
      QString path = main->sessions->loopz[it][curses]->loopPath();
      x = coord.width() + it;
      if (x > 127)
	x -= 128;
      y = coord.height();
      while ((!main->sessions->verifyCoords(x, y)) || (!verifyCoords(x, y, it))) {
	x++;
	if (x > 127) {
	  x = 0;
	  y++;
	}
      }
      pasteCyTable[it] = new CyLoop(path, x, y);
      pasteCyTable[it]->setSLmode(main->sessions->loopz[it][curses]->getSLmode());
      pasteCyTable[it]->setSmode(main->sessions->loopz[it][curses]->getSmode());
      pasteCyTable[it]->setVolume(main->sessions->loopz[it][curses]->getVolume());
      pasteCyTable[it]->setMidi(main->sessions->loopz[it][curses]->getMidi());
      pasteCyTable[it]->setMidiV(main->sessions->loopz[it][curses]->getMidiV());
      if (standardDebug)
	  main->printOutput(QString("Copying Loop: %1") .arg(pasteCyTable[it]->passName()));
  }
  pctName = main->sessions->sessionName(main->sessions->currentSession);
  pasteS = true;
  main->refreshPaste();
  QApplication::restoreOverrideCursor();
}

void AuxCycle::pasteSession() {
  if (pasteS && (main->sessions->sessionsCount <= 128)) {
    main->undo->push(new CommandPasteSession(main));
  }  
}

void AuxCycle::website() {
  QDesktopServices::openUrl((QUrl)"http://revol.altervista.org/cyclone.html");
}

void AuxCycle::help() {
  QDesktopServices::openUrl((QUrl)QString("file://"APPPREF"/share/cyclone/help/index.html"));
}
