/* AuxCycle.h
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 


#ifndef TARGET_PATH

#define TARGET_PATH 0

#endif

#ifndef AUXCYCLE_H
#define AUXCYCLE_H

#include <QObject>
#include <QDomDocument>
#include <QTableWidget>

#include "CyLoop.h"
#include "typedefs.h"

class WinCycle;
class QSettings;

class AuxCycle : public QObject {
    Q_OBJECT

public:
    AuxCycle(WinCycle *parent);
    void writeSettings();
    void readSettings();
    int colid, colname, colsl, cols, colvol, colbtn, colsel;
    void setupXml();
    int slmodeToIndex(slmode n);
    int smodeToIndex(smode n);
    slmode indexToSlmode(int n);
    smode indexToSmode(int n);
    void reloadLastSession();
    bool testOpen(QString prj);
    void open(QString prj);
    bool isPl();
    
    QString grsys; 
    int rowHeight;
    int pctCount;
    QString pctName;
    int graphicsfx;
    int graphicsfxnext;
    QString standardLocation;   
    slmode standardSL;
    smode standardS;
    int standardDebug;
    int restoreLastSession;
    int standardV;
    int standardBuffer;
    int standardQuickfade;
    int sooloovol;
    QString projectName;
    QString appName;
    QString appVersion;
    QString appOrg;
    QString appAuthor;
    QString currentFile;
    bool pasteL;
    bool pasteS;
    CyLoop *pasteCyLoop;
    CyLoop *pasteCyTable[128];
    
  signals:
    void titleChanged(QString title);

public slots:
    void help();
    void website();
    void openFile();
    bool saveFile();
    bool saveFileAs();
    void copyLoop();
    void pasteLoop();
    void copySession();
    void pasteSession();
    
private:
  void openError(QString error);
  bool save(QString prj);
  CyLoop *setupCyLoop(QDomElement loo, int row, int ssn);
  QDomElement setCyLoop(CyLoop *loop, int x, int y);
  QDomElement CyLoopToNode();
  QDomElement GlobalToNode();
  midicmd nodeToMidicmd(QDomElement *qdome);
  QDomElement midicmdToNode(midicmd mc, QString name);
  void reset();
  bool verifyCoords(int x, int y, int coord);
    
  QDomDocument *proj;
  QString standardLocationProject;
  QString appProj;
  
  /* Document standards */
  QDomElement gvmc;
  QDomElement nsmc;
  QDomElement psmc;
  QDomElement gpmc;
  QDomElement nlmc;
  QDomElement plmc;
  QDomElement ppmc;
  QDomElement firstmc;
  QDomElement rewmc;
  //QDomElement recmc;
  QDomElement fwdmc;
  QDomElement lastmc;
  //QDomElement ??mc;
  QDomNode elements;
  QDomElement intsetting;
  
  WinCycle *main; 
};

#endif
