 /* CyLoop.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 
 
#include "CyLoop.h"

extern "C" { 
  #include "bridge.h"
  #include "loopin.h"
}

CyLoop::CyLoop(QString nPath, int indx, int ssn) {
  strncpy(loopindex.path, nPath.toLatin1(), sizeof(loopindex.path) - 1);
  loopindex.index = indx;
  loopindex.session = ssn;
}

CyLoop::CyLoop(CyLoop *cl) {
  state = cl->state;
  loopindex = cl->loopindex;
  cyError = cl->cyError;
  index = cl->index;
  cyname = cl->cyname;
  cyslm = cl->cyslm;
  cysm = cl->cysm;
  cyvolume = cl->cyvolume;
  cympf = cl->cympf;
  cymvf = cl->cymvf;
}

bool CyLoop::activate() {
  if (bridge(loopindex.path, &loops[loopindex.index][loopindex.session]) == 1) {
    cyError = errorstring(&loops[loopindex.index][loopindex.session]);
    return false;
  }
  state = 0;
  return true;
}

QString CyLoop::passName() {
  QFileInfo flxinfo(loopindex.path);
  cyname = flxinfo.completeBaseName();
  return cyname;
}

QString CyLoop::passErrorString() {
  return cyError;
}

QString CyLoop::loopPath() {
  return loopindex.path;
}

void CyLoop::setIndex(int i) {
  index = i;
}

int CyLoop::getIndex() {
  return index;
}

void CyLoop::setName(QString name) {
  cyname = name;
}

QString CyLoop::getName() {
  return cyname;
}

void CyLoop::setSLmode(slmode slm) {
  cyslm = slm;
}

slmode CyLoop::getSLmode() {
  return cyslm;
}

void CyLoop::setSmode(smode sm) {
  cysm = sm;
}

smode CyLoop::getSmode() {
  return cysm;  
}

int CyLoop::getStrig() {
  return strig;
}
  
void CyLoop::setVolume(int v) {
  cyvolume = v;
  modifyvolume(&loops[loopindex.index][loopindex.session], v);
}

int CyLoop::getVolume() {
  return cyvolume;
}

void CyLoop::setMidi(midiPlayFunction mpf) {
  cympf = mpf;
}

midiPlayFunction CyLoop::getMidi() {
  return cympf;
}

void CyLoop::setMidiV(midiVolumeFunction mvf) {
  cymvf = mvf;
}

midiVolumeFunction CyLoop::getMidiV() {
  return cymvf;
}

// This is the function that creates item's buffer 
void CyLoop::setLoopBuffer(int lp, int ss) {
  dead = false;
  bridgeopenbuffer(&loops[loopindex.index][loopindex.session], lp, ss);
}

// This function refreshes loopindex 
void CyLoop::setLoopIndex(int lp, int ss) {
  dead = false;
  bridgesetindex(&loops[loopindex.index][loopindex.session], lp, ss);
}

// This function destroys item's buffer 
void CyLoop::destroyLoopBuffer() {
  deletebridge(&loops[loopindex.index][loopindex.session]);
}

// Untrigger next loop
void CyLoop::untrigger() {
  nextlooptrig = 2;
}

// This function closes item's file 
void CyLoop::closeLoop() {
  closefile(&loops[loopindex.index][loopindex.session]);
  dead = true;
}

loopresponse CyLoop::playLoop() {
  loopresponse ls;
  ls.oldindex = currentindex;
  ls.oldsession = currentsession;
  
  struct loopin loop;
  loop.index = loopindex.index;
  loop.session = loopindex.session;
  switch (cyslm) { /*@@@ modify when adding new slfunctions @@@*/
    case Record:
      loop.slmode = 2;
      break;
    case Multiply:
      loop.slmode = 1;
      break;
    case RecOverdub:
      loop.slmode = 3;
      break;
    case MultiOverdub:
      loop.slmode = 4;
      break;
    case None:
      loop.slmode = 0;
      break;
  }
  switch (cysm) { /*@@@ modify when adding new sfunctions @@@*/
    case EndLoop:
      loop.smode = 1;
      break;
    case Instantaneous:
      loop.smode = 0;
      break;
    case Once:
      loop.smode = 2;
      break;
    case Asynchronous:
      loop.smode = 3;
      break;
  }
  
  loop.state = state;
  
  state = trigger(loop);
  ls.oldstate = oldstate;
  /*if (oldstate < 0)
    oldstate = 0;*/
  ls.currentindex = currentindex;
  ls.currentsession = currentsession;
  if (cysm == 3) {
    ls.currentindex = loops[loopindex.index][loopindex.session].index;
    ls.currentsession = loops[loopindex.index][loopindex.session].session;
    ls.oldstate = -1;
  }
  ls.currentstate = state;
  
  return ls;
}

QString CyLoop::minutes() {
  QString str;
  long sr = loops[loopindex.index][loopindex.session].sfnfo.samplerate;
  long fr = loops[loopindex.index][loopindex.session].sfnfo.frames;
  int mins = (int)((fr / sr) / 60);
  if (mins < 10) {
    str.setNum(mins);
    str.prepend("0");
  } else {
    str.setNum(mins);
  }
  return str;
}

QString CyLoop::seconds() {
  QString str;
  long sr = loops[loopindex.index][loopindex.session].sfnfo.samplerate;
  long fr = loops[loopindex.index][loopindex.session].sfnfo.frames;
  int sec = (int)((fr / sr) % 60);
  if (sec < 10) {
    str.setNum(sec);
    str.prepend("0");
  } else {
    str.setNum(sec);
  }
  return str;
}