/* CyMidi.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#include <QtGui>
#include "CyMidi.h"
#include "WinCycle.h"
#include "CyTab.h"

CyMidi::CyMidi(WinCycle *parent) {
  main = parent;
}


void CyMidi::midi(int type, int num, int val, int ch) {
  if (catchlock) {
    main->mix.clearInput();
  }
  bool esci = false;
  midicmd mcd;
  mcd.type = type;
  mcd.num = num;
  mcd.val = val;
  mcd.ch = ch;
  if (getListenMidi()) {
    bool free;
    if (localSearch) {
      if ((!findMidiCurrent(mcd, main->sessions->currentSession)) && (!findMidiGlobal(mcd)))
	free = true;
    } else {
      bool catched;
      for (int h = 0; h < main->sessions->sessionsCount; h++) {
	catched = findMidiCurrent(mcd, h);
	if (catched)
	  break;
      }
      if (catched) {
	free = false;
      } else {
	if (!findMidiGlobal(mcd))
	  free = true;
	else
	  free = false;
      }
    }
    if (((type == 0) || (type == 2)) && (localTrig)) {
      mc.type = type;
      mc.num = num;
      mc.val = val;
      mc.ch = ch;
      main->mix.signalTrig(type, num, val, ch, free);
    }
    if (((type == 1) || (type > 2)) && (!localTrig)) {
      mc.type = type;
      mc.num = num;
      mc.val = val;
      mc.ch = ch;
      main->mix.signalTrig(type, num, val, ch, free);
    }
  } else {
    if (findMidiGlobal(mcd)) {
      switch (catchglobalindex) {
	case 0:
	  main->volume->setValue(catchglobalval);
	  esci = true;
	  break;
	case 1:
	  main->nextUp->click();
	  esci = true;
	  break;
	case 2:
	  main->playL->click();
	  esci = true;
	  break;
	case 3:
	  main->nextDown->click();
	  esci = true;
	  break;
	case 4:
	  main->nextRight->click();
	  esci = true;
	  break;
	case 5:
	  main->stopL->click();
	  esci = true;
	  break;
	case 6:
	  main->nextLeft->click();
	  esci = true;
	  break;
	case 7:
	  main->firstL->click();
	  esci = true;
	  break;
	case 8:
	  main->rewL->click();
	  esci = true;
	  break;
	/*case 9:
	  main->recL->click();
	  esci = true;
	  break;*/
	case 10:
	  main->fwdL->click();
	  esci = true;
	  break;
	case 11:
	  main->lastL->click();
	  esci = true;
	  break;
	/*case 12:
	  main->????????->click();
	  esci = true;
	  break;*/
      }
    }
  }
  if ((findMidiCurrent(mcd, main->sessions->currentSession)) && (!esci) && (!getListenMidi())) {
    QPushButton *psbtn;
    QSlider *sldr;
    switch (catchtype) {
      case 0:
	psbtn = (QPushButton *)main->sessions->musicTable[main->sessions->currentSession]->cellWidget(catchindex, 5);
	psbtn->click();
	break;
      case 1:
	sldr = (QSlider *)main->sessions->musicTable[main->sessions->currentSession]->cellWidget(catchindex, 4);
	sldr->setValue(catchval);
	break;
    }
  }
}

QString CyMidi::inUseString() {
  QString str("<center>" + tr("<br>Assigned to:<br> <b>"));
  if (catchglobalindex < 255) {
    switch (catchglobalindex) {
      case 0:
	str = str + tr("Global Volume");
	break;
      case 1:
	str = str + tr("Global Previous Loop");
	break;
      case 2:
	str = str + tr("Transport Play");
	break;
      case 3:
	str = str + tr("Global Next Loop");
	break;
      case 4:
	str = str + tr("Global Next Session");
	break;
      case 5:
	str = str + tr("Transport Stop");
	break;
      case 6:
	str = str + tr("Global Previous Session");
	break;
      case 7:
	str = str + tr("Transport First");
	break;
      case 8:
	str = str + tr("Transport Rew");
	break;
      case 9:
	str = str + tr("Transport Rec");
	break;
      case 10:
	str = str + tr("Transport Fwd");
	break;
      case 11:
	str = str + tr("Transport Last");
	break;
      case 12:
	str = str + "?????????????";
	break;
    }
  }
  if (catchindex < 255) {
    if (catchtype == 0)
      str = str + (tr("</b>Session<b> %1 </b>Loop<b> %2 </b>---<b> Play Button") .arg(main->sessions->musicTableTitle[catchsession]) .arg(catchindex + 1));
    else
      str = str + (tr("</b>Session<b> %1 </b>Loop<b> %2 </b>---<b> Volume Slider") .arg(main->sessions->musicTableTitle[catchsession]) .arg(catchindex + 1));
  }
  str = str + "</center></b>";
  
  return str;
}

bool CyMidi::findMidiCurrent(midicmd midic, int sex) {
  catchlock = true;
  bool catched = false;
  if (midic.type == 0) {
    if (main->mix.midix[0][midic.ch][midic.num][sex].active) {
      catched = true;
      catchindex = main->mix.midix[0][midic.ch][midic.num][sex].index;
      catchsession = sex;
      catchglobalindex = 255;
      catchtype = 0;
    }
  }
  if (midic.type == 2) {
    if (main->mix.midix[1][midic.ch][midic.num][sex].active) {
      catched = true;
      catchindex = main->mix.midix[1][midic.ch][midic.num][sex].index;
      catchsession = sex;
      catchglobalindex = 255;
      catchtype = 0;
    }
  }
  if ((midic.type == 1) || (midic.type > 2)) {
    if (main->mix.midix[2][midic.ch][midic.num][sex].active) {
      catched = true;
      catchindex = main->mix.midix[2][midic.ch][midic.num][sex].index;
      catchsession = sex;
      catchglobalindex = 255;
      catchval = midic.val;
      catchtype = 1;
    }
  }
  catchlock = false;
  
  return catched;
}

bool CyMidi::findMidiGlobal(midicmd midic) {
  catchlock = true;
  bool catched = false;
  if (midic.type == 0) {
    if (main->mix.midig[0][midic.ch][midic.num].active) {
      catched = true;
      catchglobalindex = main->mix.midig[0][midic.ch][midic.num].index;
      catchindex = 255;
    }
  }
  if (midic.type == 2) {
    if (main->mix.midig[1][midic.ch][midic.num].active) {
      catched = true;
      catchglobalindex = main->mix.midig[1][midic.ch][midic.num].index;
      catchindex = 255;
    }
  }
  if ((midic.type == 1) || (midic.type > 2)) {
    if (main->mix.midig[2][midic.ch][midic.num].active) {
      catched = true;
      catchglobalindex = main->mix.midig[2][midic.ch][midic.num].index;
      catchindex = 255;
      catchglobalval = midic.val;
    }
  }
  catchlock = false;
  
  return catched;  
}

void CyMidi::refresh(int session) {
  if (session > 127)
    return;
  main->mix.reset(session);
  midiPlayFunctionT mpf;
  midiVolumeFunctionT mvf;
  for (int lp = 0; lp < main->sessions->musicTable[session]->rowCount(); lp++) {
    mpf = main->sessions->loopz[lp][session]->getMidi();
    if (mpf.val < 128) {
      if (mpf.fn == NT) {
	main->mix.midix[0][mpf.ch][mpf.val][session].active = true;
	main->mix.midix[0][mpf.ch][mpf.val][session].index = lp;
      } else {
	main->mix.midix[1][mpf.ch][mpf.val][session].active = true;
	main->mix.midix[1][mpf.ch][mpf.val][session].index = lp;
      }
    }
    mvf = main->sessions->loopz[lp][session]->getMidiV();
    if (mvf.val < 128) {
      main->mix.midix[2][mvf.ch][mvf.val][session].active = true;
      main->mix.midix[2][mvf.ch][mvf.val][session].index = lp;
    }
  }
}

void CyMidi::refreshGlobal() {
  main->mix.resetGlobal();
  midicmd mcmd = main->globalmidifunctions[0];
  if (mcmd.num < 128) {
    main->mix.midig[2][mcmd.ch][mcmd.num].active = true;
    main->mix.midig[2][mcmd.ch][mcmd.num].index = 0;
  }
  for (int gf = 1; gf < 13 /* Actual Global Functions */; gf++) {
    midicmd mcmd = main->globalmidifunctions[gf];
    if (mcmd.num < 128) {
      if (mcmd.type == 0) {
	main->mix.midig[0][mcmd.ch][mcmd.num].active = true;
	main->mix.midig[0][mcmd.ch][mcmd.num].index = gf;
      }
      if (mcmd.type == 2) {
	main->mix.midig[1][mcmd.ch][mcmd.num].active = true;
	main->mix.midig[1][mcmd.ch][mcmd.num].index = gf;
      }
    }
  }
}
