/* CyTab.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#include <QtGui>
#include "CyTab.h"
#include "Dialogs.h"
#include "WinCycle.h"


CyTabWidget::CyTabWidget(WinCycle *parent) : QTabWidget(parent) {
  setAcceptDrops(true);
  main = parent;
  signalMapper = new QSignalMapper(this);
  connect(signalMapper, SIGNAL(mapped(QString)), this, SLOT(tableWidgetChanged(QString)));
  for (int a = 0; a < 128; a++)
    sel[a] = 255;
  movingLoop = false;
}

void CyTabWidget::moveTabLeft(int index) {
  tabBar()->moveTab(index, index - 1);
  setCurrentIndex(index);
  setCurrentIndex(index - 1);
  emit currentChanged(index - 1);
}

void CyTabWidget::moveTabRight(int index) {
  tabBar()->moveTab(index, index + 1);
  setCurrentIndex(index);
  setCurrentIndex(index + 1);
  emit currentChanged(index + 1);
}

void CyTabWidget::addSession(int count, int ssn, QString title) {
    musicTable[ssn] = new QTableWidget(0, 7);
    connect(musicTable[ssn], SIGNAL(cellDoubleClicked(int,int)), this, SLOT(activateNameChange(int,int)));
    QPalette pal;
    pal.setBrush(QPalette::Highlight, pal.brush(QPalette::ButtonText));
    pal.setBrush(QPalette::HighlightedText, pal.brush(QPalette::Button));
    musicTable[ssn]->setPalette(pal);
    
    QHeaderView *qhv = musicTable[ssn]->verticalHeader();
    qhv->setDefaultSectionSize(main->aux->rowHeight);
    qhv->hide();
    
    QPalette *bgj = new QPalette();
    QTableWidgetItem *iItem = new QTableWidgetItem(tr(" "));
    QTableWidgetItem *nItem = new QTableWidgetItem(tr("Name"));
    QTableWidgetItem *slItem = new QTableWidgetItem("Sooperlooper");
    QTableWidgetItem *sItem = new QTableWidgetItem(tr("Mode"));
    QTableWidgetItem *mItem = new QTableWidgetItem(tr("Play"));
    QTableWidgetItem *vItem = new QTableWidgetItem(tr("Volume"));
    QTableWidgetItem *pItem = new QTableWidgetItem(tr("State"));
    QBrush bgh = bgj->brush(QPalette::Window);
    iItem->setBackground(bgh);
    nItem->setBackground(bgh);
    slItem->setBackground(bgh);
    sItem->setBackground(bgh);
    mItem->setBackground(bgh);
    vItem->setBackground(bgh);
    pItem->setBackground(bgh);
   
    QColor bgc = bgj->color(QPalette::Midlight);
    QString bg = "background: rgb(%1, %2, %3)";
    musicTable[ssn]->setStyleSheet(bg .arg(bgc.red()) .arg(bgc.green()) .arg(bgc.blue()));
    musicTable[ssn]->setHorizontalHeaderItem(0, iItem);
    musicTable[ssn]->setColumnWidth(0, main->aux->colid);
    musicTable[ssn]->setHorizontalHeaderItem(1, nItem);
    musicTable[ssn]->setColumnWidth(1, main->aux->colname);
    musicTable[ssn]->setHorizontalHeaderItem(2, slItem);
    musicTable[ssn]->setColumnWidth(2, main->aux->colsl);
    musicTable[ssn]->setHorizontalHeaderItem(3, sItem);
    musicTable[ssn]->setColumnWidth(3, main->aux->cols);
    musicTable[ssn]->setHorizontalHeaderItem(4, vItem);
    musicTable[ssn]->setColumnWidth(4, main->aux->colvol);
    musicTable[ssn]->setHorizontalHeaderItem(5, mItem);
    musicTable[ssn]->setColumnWidth(5, main->aux->colbtn);
    musicTable[ssn]->setHorizontalHeaderItem(6, pItem);
    musicTable[ssn]->setColumnWidth(6, main->aux->colsel);
    
    musicTable[ssn]->setSelectionMode(QAbstractItemView::NoSelection);
    musicTable[ssn]->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    musicTable[ssn]->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    connect(musicTable[ssn], SIGNAL(cellPressed(int, int)), this, SLOT(tableClicked(int, int)));
    connect(musicTable[ssn], SIGNAL(cellChanged(int, int)), this, SLOT(tableChanged(int, int)));
    
    QString sez;
    if (title.isEmpty())
      sez = tr("Session %1") .arg(count);
    else
      sez = title;
    
    if (sessionsCount > 0) {
      addTab(musicTable[ssn], sez);
    } else {
      addTab(musicTable[ssn], sez);
      removeTab(0);
      sessionsCount = 1;
    }
    musicTableTitle[ssn] = sez;
}

void CyTabWidget::activateNameChange(int x, int y) {
  if (y == 1)
    main->undo->pushname = true;
}

void CyTabWidget::tableClicked(int row, int column) {
    if (sel[currentSession] < 128) {
      QTableWidgetSelectionRange qts(0, 1, musicTable[currentSession]->rowCount() - 1, musicTable[currentSession]->columnCount() - 1);
      musicTable[currentSession]->setRangeSelected(qts, false);
      if (main->aux->graphicsfx == 1)
	colorizeLoop(false);
    }
    sel[currentSession] = row;
    musicTable[currentSession]->setCurrentCell(sel[currentSession], 6);
    QTableWidgetSelectionRange qts0(row, 1, row, musicTable[currentSession]->columnCount() - 1);
    musicTable[currentSession]->setRangeSelected(qts0, true);
    main->removeLoopAction->setDisabled(false);
    main->copyLoop->setDisabled(false);
    if (main->aux->pasteL)
      main->pasteLoop->setDisabled(false);
    if (main->aux->pasteS)
      main->pasteSession->setDisabled(false);
    if (main->aux->graphicsfx == 1)
      colorizeLoop(true);
    main->durLabel->setText(QString("<b>%1:%2</b>") .arg(loopz[row][currentSession]->minutes()) .arg(loopz[row][currentSession]->seconds()));
    QHeaderView *qhv = musicTable[currentSession]->horizontalHeader();
    qhv->setHighlightSections(false);
    currentLoop = row;
    main->undo->pushname = false;
}

void CyTabWidget::colorizeLoop(bool colorize) {
  QPalette pal;
  qreal val;
  qreal str;
  if (colorize) {
    val = 1.0;
    str = 0.7;
  } else {
    val = 0.4;
    str = 0.0;
  }
  QGraphicsOpacityEffect *gfx = new QGraphicsOpacityEffect();
  gfx->setOpacity(val);
  QComboBox *qcb = (QComboBox *)musicTable[currentSession]->cellWidget(sel[currentSession], 2);
  qcb->setGraphicsEffect(gfx);
  QGraphicsOpacityEffect *gfx1 = new QGraphicsOpacityEffect();
  gfx1->setOpacity(val);
  QComboBox *qcb1 = (QComboBox *)musicTable[currentSession]->cellWidget(sel[currentSession], 3);
  qcb1->setGraphicsEffect(gfx1);
  QGraphicsOpacityEffect *gfx2 = new QGraphicsOpacityEffect();
  gfx2->setOpacity(val);
  QSlider *qcb2 = (QSlider *)musicTable[currentSession]->cellWidget(sel[currentSession], 4);
  qcb2->setGraphicsEffect(gfx2);
  QGraphicsOpacityEffect *gfx3 = new QGraphicsOpacityEffect();
  gfx3->setOpacity(val);
  QPushButton *qcb3 = (QPushButton *)musicTable[currentSession]->cellWidget(sel[currentSession], 5);
  qcb3->setGraphicsEffect(gfx3);
  QWidget *state = (QWidget *)musicTable[currentSession]->cellWidget(sel[currentSession], 6);
  QGraphicsColorizeEffect *gfx4 = new QGraphicsColorizeEffect();
  gfx4->setColor(Qt::darkYellow);
  gfx4->setStrength(str);
  state->setGraphicsEffect(gfx4);
}

void CyTabWidget::tableChanged(int row, int column) {
  if (column == 0) {
    loopz[row][currentSession]->setIndex(musicTable[currentSession]->item(row, column)->text().toInt() - 1);
    if (main->windebug)
      main->printOutput(QString("indexChanged --\t\t%1\t%2\t%3\t") .arg(currentSession) .arg(row) .arg(loopz[row][currentSession]->getIndex()));
    return;
  }
  
  if (column == 1) {
    if (main->undo->pushname) {
      main->undo->push(new CommandLoopName(main, row, currentSession, musicTable[currentSession]->item(row, column)->text()));
      main->undo->pushname = false;
      if (main->windebug)
	main->printOutput(QString("nameChanged --\t\t%1\t%2\t%3\t") .arg(currentSession) .arg(row) .arg(musicTable[currentSession]->item(row, column)->text()));
    }
    loopz[row][currentSession]->setName(musicTable[currentSession]->item(row, column)->text());
    return;
  }
}


void CyTabWidget::tableWidgetChanged(QString position) {
  int ssn, row, cols;
  QStringList coordinates = position.split("-");
  ssn = coordinates[0].toInt();
  row = coordinates[1].toInt();
  cols = coordinates[2].toInt();
  if (cols == 2) {
    if (!movingLoop) {
      int slindex = ((QComboBox *)musicTable[currentSession]->cellWidget(row, cols))->currentIndex();
      if (main->windebug)
	main->printOutput(QString("slModeChanged --\t%1\t%2\t%3\t") .arg(currentSession) .arg(row) .arg(slindex));
      main->undo->push(new CommandSLcombo(main, slindex, row, currentSession));
    }
  }
  
  if (cols == 3) {
    int sindex = ((QComboBox *)musicTable[currentSession]->cellWidget(row, cols))->currentIndex();
    if (!movingLoop) {
      if (main->windebug)
	main->printOutput(QString("sModeChanged --\t\t%1\t%2\t%3\t") .arg(currentSession) .arg(row) .arg(sindex));
      main->undo->push(new CommandScombo(main, sindex, row, currentSession));
    }
    if (sindex == 3) {
      QComboBox *ssll = (QComboBox *)musicTable[currentSession]->cellWidget(row, 2);
      ssll->setEnabled(false);
    } else {
      QComboBox *ssll = (QComboBox *)musicTable[currentSession]->cellWidget(row, 2);
      ssll->setEnabled(true); 
    }
  }
  
  if (cols == 4) {
    if (!movingLoop) {
      QSlider *slider = (QSlider *)musicTable[currentSession]->cellWidget(row, cols);
      if (main->windebug)
	main->printOutput(QString("vSliderChanged --\t%1\t%2\t%3\t") .arg(currentSession) .arg(row) .arg(slider->value()));
      loopz[row][currentSession]->setVolume(slider->value());
      if (!main->undo->oldloopvolpressed[row][currentSession]) {
	if (!slider->isSliderDown()) {
	  if (!movingLoop)
	    main->undo->push(new CommandLoopVolume(main, row, currentSession, slider->value()));
	} else {
	  main->undo->oldloopvolpressed[row][currentSession] = true;
	}
      }
    }
  }
  
  if (cols == 40) {
    if (main->windebug)
      main->printOutput(QString("ContextMenu --\t%1\t%2\tvolume midi function") .arg(currentSession) .arg(row));
    main->midiLearnButton(row, currentSession, 1);      
  }
  
  if (cols == 44) {
    loopVolumeUndo(row, currentSession);
  }
  
  if (cols == 5) {
    if (main->jax.state() == 0) {
      if (main->windebug)
	main->printOutput(QString("Play --\t%1\t%2\tButton Clicked") .arg(currentSession) .arg(row));
      loopresponse ls;
      ls = loopz[row][currentSession]->playLoop();
      tableClicked(row,6);
      if (ls.currentstate > -1) {
	processResponse(ls.oldindex, ls.oldsession, ls.oldstate, ls.currentindex, ls.currentsession, ls.currentstate);
      }
    }   
  }
    
  if (cols == 55) { 
    if (main->windebug)
      main->printOutput(QString("ContextMenu --\t%1\t%2\tplay midi function") .arg(currentSession) .arg(row));
    main->midiLearnButton(row, currentSession, 0);
  }
  
  if (cols > 97) {
    if (main->windebug)
      main->printOutput(QString("GLOBAL FUNCTION --\t%1") .arg(row));
    if (cols == 99)
      main->midiLearnButton(row, 0, 2);    
    else
      main->midiLearnButton(0, 0, 3);
  }
}


void CyTabWidget::moveSessionRight() {
  if (currentSession < (sessionsCount - 1)) {
    main->undo->push(new CommandMoveSession(main, currentSession, currentSession + 1));
  }
}

void CyTabWidget::moveSessionLeft() {  
  if (currentSession > 0) {
    main->undo->push(new CommandMoveSession(main, currentSession, currentSession - 1));
  }
}

void CyTabWidget::sessionRight() {
  if (currentSession < (sessionsCount - 1)) {
    currentSession++;
    setCurrentIndex(currentSession);
  }
}

void CyTabWidget::sessionLeft() {
  if (currentSession > 0) {
    currentSession--;
    setCurrentIndex(currentSession);
  }
}


bool CyTabWidget::addLoop(CyLoop *loop, int sex) {
  setCurrentIndex(sex);
  int num = musicTable[sex]->rowCount();
  QFileInfo fl(loop->loopPath());
  if (!fl.isReadable()) {
    QMessageBox::warning(main, tr("Cyclone - Error Loading Loops"),
		      tr("Cannot Read %1.")
		      .arg(loop->getName()));
    return false;
  }
  
  if (!loop->activate()) {
    QString errorz = loopz[num][sex]->passErrorString();
    if (errorz == "not_valid_wav")
      errorz = tr("Not a valid audio file");
    QMessageBox::warning(main, tr("Cyclone - Error Loading Loops"),
		      tr("Not a valid file %1:\n%2.")
		      .arg(loop->getName())
		      .arg(errorz));
    return false;
  }
  
  /* Adding a row to the musicTable to host new item */
  QPalette *bgj = new QPalette();
  musicTable[sex]->setRowCount(++num);
  
  /* Add index number to the item in musicTable */
  QString numero;
  numero.setNum(loop->getIndex());
  QTableWidgetItem *nItem = new QTableWidgetItem(numero);
  nItem->setBackground(bgj->brush(QPalette::Light));
  nItem->setFlags(Qt::NoItemFlags);
  musicTable[sex]->setItem((num - 1),0,nItem);
  
  /* Add name to the item in musicTable */
  if (loop->getName().isEmpty())
    loop->passName();
  QTableWidgetItem *item = new QTableWidgetItem(loop->getName());
  item->setBackground(bgj->brush(QPalette::Base));
  musicTable[sex]->setItem((num - 1),1,item);
  
  QComboBox *slcb = setupSLcombo((num - 1), loop->getSLmode());
  musicTable[sex]->setCellWidget((num - 1),2,slcb);
  
  QComboBox *scb = setupScombo((num - 1), loop->getSmode());
  musicTable[sex]->setCellWidget((num - 1),3,scb);
  if (loop->getSmode() == Asynchronous) {
    slcb->setDisabled(true);
  }
  
  QSlider *sld = setupVslider((num - 1), loop->getVolume());
  musicTable[sex]->setCellWidget((num - 1),4,sld);
  
  QPushButton *btn = setupMbutton((num - 1));
  musicTable[sex]->setCellWidget((num - 1),5,btn); 
  
  QWidget *dItem = new QWidget(musicTable[sex]);
  musicTable[sex]->setCellWidget((num - 1),6,dItem);
    
  main->copySession->setDisabled(false);
  if (main->undo->pushed) {
    main->undo->push(new CommandAddLoop(main, loop, num - 1, sex));
    main->undo->pushed = false;
  }
  if (main->aux->graphicsfx == 1) {
    int tmp = sel[currentSession];
    sel[currentSession] = (num - 1);
    colorizeLoop(false);
    sel[currentSession] = tmp;
  }
  if (main->jax.state() == 0)
    loopz[num - 1][sex]->setLoopBuffer(num - 1, sex);
  loopz[num - 1][sex]->state = 0;
  midiPlayFunction mpf = loopz[num - 1][sex]->getMidi();
  midiVolumeFunction mvf = loopz[num - 1][sex]->getMidiV();
  int mpfv, mvfv;
  mpfv = mpf.val;
  mvfv = mvf.val;
  mpf.val = 255;
  mvf.val = 255;
  loopz[num - 1][sex]->setMidi(mpf);
  loopz[num - 1][sex]->setMidiV(mvf);
  if (analyzeLoop(loopz[num - 1][sex], sex)) {
    mpf.val = mpfv;
    mvf.val = mvfv;
    loopz[num - 1][sex]->setMidi(mpf);
    loopz[num - 1][sex]->setMidiV(mvf);
  }
  return true;
}

bool CyTabWidget::analyzeLoop(CyLoop *loop, int session) {
  bool ok = true;
  midicmd mcm;
  midiPlayFunction mpf = loop->getMidi();
  mcm.ch = mpf.ch;
  mcm.num = mpf.val;
  if (mcm.num < 128) {
    if (mpf.fn == NT)
      mcm.type = 0;
    else
      mcm.type = 2;
    ok = main->cymidi->findMidiCurrent(mcm, session);
    if (ok)
      return false;
    ok = main->cymidi->findMidiGlobal(mcm);
    if (ok)
      return false;
  }
  midiVolumeFunction mvf = loop->getMidiV();
  mcm.ch = mvf.ch;
  mcm.num = mvf.val;
  if (mcm.num < 128) {
    mcm.type = 1;
    ok = main->cymidi->findMidiCurrent(mcm, session);
    if (ok)
      return false;
    ok = main->cymidi->findMidiGlobal(mcm);
    if (ok)
      return false;
  }
  return ok;
}

bool CyTabWidget::verifyCoords(int x, int y) {
  bool ok = true;
  for (int ziz = 0; ziz < sessionsCount; ziz++) {
    for (int zoz = 0; zoz < musicTable[ziz]->rowCount(); zoz++) {
      if ((loopz[zoz][ziz]->loopindex.index == x) && (loopz[zoz][ziz]->loopindex.session == y)) {
	ok = false;
	if (main->windebug)
	  main->printOutput(QString("Index/Session catched %1 - %2") .arg(x) .arg(y));
      }
    }
  }
  return ok;
}

QSize CyTabWidget::generateCoords() {
  QSize coord;
  /* Look for available index-session */
  int indice, ix;
  int sessione, sy;
  ix = 0;
  sy = 0;
  bool ok = false;
  if ((sessionsCount == 1) && (musicTable[0]->rowCount() == 0)) {
    ok = true;
    indice = 0;
    sessione = 0;
  }
  bool catched;
  while (!ok) {
    catched = false;
    for (int ziz = 0; ziz < sessionsCount; ziz++) {
      for (int zoz = 0; zoz < musicTable[ziz]->rowCount(); zoz++) {
	if ((loopz[zoz][ziz]->loopindex.index == ix) && (loopz[zoz][ziz]->loopindex.session == sy)) {
	  catched = true;
	}
      }
    }
    if (!catched) {
      ok = true;
      indice = ix;
      sessione = sy;
    }
    if (!ok) {
      if (ix < 127) {
        ix++;
      } else {
	ix = 0;
	sy++;
      }
    }
  }
  coord.setWidth(indice);
  coord.setHeight(sessione);
  return coord;
}

void CyTabWidget::addFiles() {
    QStringList files = QFileDialog::getOpenFileNames(main, tr("Select Audio Files"), main->aux->standardLocation,
	tr("Audio Files (%1);; All Files (%2)") .arg("*.wav *.aif *.flac") .arg("*.*"));
	
    if (files.isEmpty())
	return;
    QApplication::setOverrideCursor(Qt::WaitCursor);
    bool max = false;
    foreach (QString string, files) {
      if (musicTable[currentSession]->rowCount() <= 127) { 
	if (addMusicTableItem(QUrl(string).path(), currentSession)) {
	  if (main->windebug)
	    main->printOutput(QString("Loading: %1") .arg(QUrl(string).path()));
	}
      } else {
	if (!max) {
	  QApplication::restoreOverrideCursor();
	  max = true;
	  QMessageBox::warning(main, tr("Cyclone - Add Loops"),
			tr("Maximum number of loop for this session reached.\n"
			    "Please create a new session or delete loops."), QMessageBox::Cancel);
	  main->undo->pushed = false;
	  break;
	}
      }
    }
    QFileInfo qfi((QString)files.first());
    main->aux->standardLocation = qfi.dir().absolutePath();
    QApplication::restoreOverrideCursor();
}

bool CyTabWidget::addMusicTableItem(QString nome, int ssn) {
  int num = musicTable[ssn]->rowCount();
  QSize coord = generateCoords();
  loopz[num][ssn] = new CyLoop(nome, coord.width(), coord.height());
  midiPlayFunction mpf;
  midiVolumeFunction mvf;
  mpf.val = 255;
  mvf.val = 255;
  loopz[num][ssn]->setMidi(mpf);
  loopz[num][ssn]->setMidiV(mvf);
  loopz[num][ssn]->setIndex(num +1);
  
  loopz[num][ssn]->setSLmode(main->aux->standardSL);
  loopz[num][ssn]->setSmode(main->aux->standardS);
  loopz[num][ssn]->setVolume(main->aux->standardV);
  
  main->undo->pushed = true;
  if (addLoop(loopz[num][ssn], ssn)) {
    return true;
  } else {
    main->undo->pushed = false;
    return false;
  }
}
  
QComboBox *CyTabWidget::setupSLcombo(int row, slmode slm) {
   /* Add sooperlooper mode to the item in musicTable */
  QComboBox *slcb = new QComboBox(musicTable[currentSession]);
  slcb->addItem(tr("None")); /*@@@ Sooperlooper modes @@@*/
  slcb->addItem(tr("Multiply")); /*@@@ Sooperlooper modes @@@*/
  slcb->addItem(tr("Record")); /*@@@ Sooperlooper modes @@@*/
  slcb->addItem(tr("RecOverdub")); /*@@@ Sooperlooper modes @@@*/
  slcb->addItem(tr("MultiOverdub")); /*@@@ Sooperlooper modes @@@*/
  slcb->setCurrentIndex(main->aux->slmodeToIndex(slm));
  connect(slcb, SIGNAL(currentIndexChanged(int)), signalMapper, SLOT(map()));
  signalMapper->setMapping(slcb, QString("%1-%2-2") .arg(currentSession) .arg(row));
  main->undo->oldsl[row][currentSession] = main->aux->slmodeToIndex(slm);
  return slcb;
}

QComboBox *CyTabWidget::setupScombo(int row, smode sm) {  
  /* Add stop mode to the item in musicTable */
  QComboBox *scb = new QComboBox(musicTable[currentSession]);
  scb->addItem(tr("Instantaneous")); /*@@@ Stop modes @@@*/
  scb->addItem(tr("EndLoop")); /*@@@ Stop modes @@@*/
  scb->addItem(tr("Once")); /*@@@ Stop modes @@@*/
  scb->addItem(tr("Asynchronous")); /*@@@ Stop modes @@@*/
  scb->setCurrentIndex(main->aux->smodeToIndex(sm));
  connect(scb, SIGNAL(currentIndexChanged(int)), signalMapper, SLOT(map()));
  signalMapper->setMapping(scb, QString("%1-%2-3") .arg(currentSession) .arg(row));
  main->undo->olds[row][currentSession] = main->aux->smodeToIndex(sm);      
  return scb;
}

QSlider *CyTabWidget::setupVslider(int row, int vl) {
  QSlider *sld = new QSlider(Qt::Horizontal, musicTable[currentSession]); /*@@@ Volume for the loop @@@*/
  sld->setRange(0,127);
  sld->setContextMenuPolicy(Qt::CustomContextMenu);
  sld->setValue(vl);
  main->undo->oldloopvol[row][currentSession] = vl;
  main->undo->oldloopvolpressed[row][currentSession] = false;
  connect(sld, SIGNAL(valueChanged(int)), signalMapper, SLOT(map()));
  signalMapper->setMapping(sld, QString("%1-%2-4") .arg(currentSession) .arg(row));
  dummyMidiVol[row][currentSession] = new DummyWidget();
  connect(sld, SIGNAL(customContextMenuRequested(QPoint)), dummyMidiVol[row][currentSession], SLOT(signalz()));
  connect(dummyMidiVol[row][currentSession], SIGNAL(signalx()), signalMapper, SLOT(map()));
  signalMapper->setMapping(dummyMidiVol[row][currentSession], QString("%1-%2-40") .arg(currentSession) .arg(row));
  dummyVol[row][currentSession] = new DummyWidget();
  connect(sld, SIGNAL(sliderReleased()), dummyVol[row][currentSession], SLOT(signalz()));
  connect(dummyVol[row][currentSession], SIGNAL(signalx()), signalMapper, SLOT(map()));
  signalMapper->setMapping(dummyVol[row][currentSession], QString("%1-%2-44") .arg(currentSession) .arg(row));
  return sld;
}

QPushButton *CyTabWidget::setupMbutton(int row) {
   /* Add play button to the item in musicTable */
  QPushButton *btn = new QPushButton(musicTable[currentSession]);
  btn->setContextMenuPolicy(Qt::CustomContextMenu);
  btn->setIcon(QIcon(":/icons/start.png"));
  btn->setIconSize(QSize(24, 24));
  connect(btn, SIGNAL(pressed()), signalMapper, SLOT(map()));
  signalMapper->setMapping(btn, QString("%1-%2-5") .arg(currentSession) .arg(row));
  dummyButton[row][currentSession] = new DummyWidget();
  connect(btn, SIGNAL(customContextMenuRequested(QPoint)), dummyButton[row][currentSession], SLOT(signalz()));
  connect(dummyButton[row][currentSession], SIGNAL(signalx()), signalMapper, SLOT(map()));
  signalMapper->setMapping(dummyButton[row][currentSession], QString("%1-%2-55") .arg(currentSession) .arg(row));
  return btn;
}

void CyTabWidget::loopVolumeUndo(int x, int y) {
  if (main->undo->oldloopvolpressed[x][y]) {
    QSlider *sld = (QSlider *)musicTable[y]->cellWidget(x, 4);
    main->undo->push(new CommandLoopVolume(main, x, y, sld->value()), true);
    main->undo->oldloopvolpressed[x][y] = false;
  }
}

void CyTabWidget::currentSessionChanged(int csc) {
  currentSession = csc;
  if (musicTable[csc]->currentRow() < 0) {
    main->removeLoopAction->setEnabled(false);
    main->copyLoop->setEnabled(false);
  }
  if (musicTable[csc]->rowCount() > 0)
    main->copySession->setDisabled(false);
  if (sessionsCount > 1)
    main->removeSessionAction->setDisabled(false);
}

void CyTabWidget::removeSessionSlot() {
  main->undo->push(new CommandRemoveSession(main, currentSession));
  if (sessionsCount > 1) {
    main->addSessionAction->setEnabled(true);
    main->removeSessionAction->setEnabled(true);
  } else {
    main->removeSessionAction->setEnabled(false);
  }
  main->saveProjectAction->setEnabled(true);
}

void CyTabWidget::removeSession(int sex) {
  removeTab(sex);
  sessionsCount--;
}

void CyTabWidget::removeLoop() {
  if (sel[currentSession] > 127)
    return;
  
  int curo = sel[currentSession];
  main->undo->push(new CommandRemoveLoop(main, loopz[curo][currentSession], curo, currentSession));
}

void CyTabWidget::insLoop(CyLoop *loop, int x, int y) {
  unselect(currentSession);
  int tot = musicTable[y]->rowCount();
  for (int jj = tot; jj > x; jj--) {
    loopz[jj][y] = loopz[jj - 1][y];
    if (main->jax.state() == 0) {
      loopz[jj][y]->setLoopIndex(jj,y);
    }
    musicTable[y]->item(jj - 1, 0)->setText(QString("%1") .arg(jj + 1));
  }
  setCurrentIndex(y);
  loopz[x][y] = new CyLoop(loop);
  loopz[x][y]->setLoopBuffer(x,y);
  
  midiPlayFunction mpf = loopz[x][y]->getMidi();
  midiVolumeFunction mvf = loopz[x][y]->getMidiV();
  int mpfv, mvfv;
  mpfv = mpf.val;
  mvfv = mvf.val;
  mpf.val = 255;
  mvf.val = 255;
  loopz[x][y]->setMidi(mpf);
  loopz[x][y]->setMidiV(mvf);
  if (analyzeLoop(loopz[x][y], y)) {
    mpf.val = mpfv;
    mvf.val = mvfv;
    loopz[x][y]->setMidi(mpf);
    loopz[x][y]->setMidiV(mvf);
  }
  
  QFileInfo fl(loop->loopPath());
  if (!fl.isReadable()) {
    QMessageBox::warning(main, tr("Cyclone - Error Loading Loops"),
		      tr("Cannot Read %1.")
		      .arg(loop->getName()));
    return;
  }
  
  if (!loop->activate()) {
    QString errorz = loop->passErrorString();
    if (errorz == "not_valid_wav")
      errorz = tr("Not a valid audio file");
    QMessageBox::warning(main, tr("Cyclone - Error Loading Loops"),
		      tr("Not a valid file %1:\n%2.")
		      .arg(loop->getName())
		      .arg(errorz));
    return;
  }
  
  /* Inserting a row to the musicTable to host new item */
  QPalette *bgj = new QPalette();
  musicTable[y]->insertRow(x);
  
  /* Add index number to the item in musicTable */
  QString numero;
  numero.setNum(loop->getIndex() + 1);
  QTableWidgetItem *nItem = new QTableWidgetItem(numero);
  nItem->setBackground(bgj->brush(QPalette::Light));
  nItem->setFlags(Qt::NoItemFlags);
  musicTable[y]->setItem(x,0,nItem);
  
  /* Add name to the item in musicTable */
  QTableWidgetItem *item = new QTableWidgetItem(loop->passName());
  item->setBackground(bgj->brush(QPalette::Base));
  musicTable[y]->setItem(x, 1, item);
  
  QComboBox *slcb = setupSLcombo(x, loop->getSLmode());
  musicTable[y]->setCellWidget(x,2,slcb);
  
  QComboBox *scb = setupScombo(x, loop->getSmode());
  musicTable[y]->setCellWidget(x,3,scb);
  if (loop->getSmode() == Asynchronous) {
    slcb->setDisabled(true);
  }
  
  QSlider *sld = setupVslider(x, loop->getVolume());
  musicTable[y]->setCellWidget(x,4,sld);
  
  QPushButton *btn = setupMbutton(x);
  musicTable[y]->setCellWidget(x,5,btn); 
  
  QWidget *dItem = new QWidget(musicTable[y]);
  musicTable[y]->setCellWidget(x,6,dItem);
  
  musicTable[y]->setRowHeight(x,28);
  
  refreshMusicTable(y, false);
  
  tableClicked(x,6);
}

void CyTabWidget::rmLoop(int x, int y) {
  sel[currentSession] = 255;
  unsetSignalMappings(y);
  if (main->jax.state() == 0)
    loopz[x][y]->destroyLoopBuffer();
  loopz[x][y]->closeLoop();
  
  for (int jj = x; jj < (musicTable[y]->rowCount() - 1); jj++) {
    loopz[jj][y] = loopz[jj + 1][y];
    if (main->jax.state() == 0) {
      loopz[jj][y]->dead = false;
      if (main->jax.state() == 0) {
	loopz[jj][y]->setLoopIndex(jj,y);
      }
    }
  }
  for (int jj = (musicTable[y]->rowCount() - 1); jj > x; jj--) {
    musicTable[y]->setItem(jj, 0, (musicTable[y]->item(jj - 1, 0)->clone()));
  }
  
  musicTable[y]->setRowHidden(x, true);
  musicTable[y]->removeRow(x);
  
  if (musicTable[y]->rowCount() > 0) {
    if (x < musicTable[y]->rowCount()) {
      tableClicked(x, 6);
      int cs = currentSession;
      currentSession = y;
      refreshMusicTable(y, true);
      currentSession = cs;
    } else {
      sel[y] = 255;
      tableClicked(x - 1, 6);
      main->cymidi->refresh(y);
    }
  } else {
    sel[y] = 255;
    main->removeLoopAction->setDisabled(true);
    main->copyLoop->setEnabled(false);
    if (main->aux->pasteL)
      main->pasteLoop->setEnabled(false);
    if (main->aux->pasteS)
      main->pasteSession->setEnabled(false);
  }
}

void CyTabWidget::moveLoopUp() {
  int curo = sel[currentSession];
  if ((curo > 0) && (curo < 128))
    main->undo->push(new CommandMoveLoop(main, curo, curo - 1));
}

void CyTabWidget::moveLoopDown() {
  if (sel[currentSession] > 127)
    return;
  int curo = sel[currentSession];
  if (curo < (musicTable[currentSession]->rowCount() - 1))
    main->undo->push(new CommandMoveLoop(main, curo, curo + 1));
}

void CyTabWidget::moveLoop(int orig, int dest) {
  movingLoop = true;
  unsetLoopAttributes(orig, currentSession);
  unsetLoopAttributes(dest, currentSession);
  unselect(currentSession);
  main->undo->pushname = false;
  QTableWidgetItem *iItem = musicTable[currentSession]->item(orig, 0)->clone();
  QTableWidgetItem *iItem1 = musicTable[currentSession]->item(dest, 0)->clone();
  QTableWidgetItem *nItem = musicTable[currentSession]->item(orig, 1)->clone();
  QWidget *dItem = new QWidget(musicTable[currentSession]);
  QComboBox *slcomboz = (QComboBox *)musicTable[currentSession]->cellWidget(orig, 2);
  int sli = slcomboz->currentIndex();
  QComboBox *_slcombo = setupSLcombo(dest, main->aux->indexToSlmode(sli));
  QComboBox *scomboz = (QComboBox *)musicTable[currentSession]->cellWidget(orig, 3);
  int si = scomboz->currentIndex();
  QComboBox *_scombo = setupScombo(dest, main->aux->indexToSmode(si));
  QSlider *vsliderz = (QSlider *)musicTable[currentSession]->cellWidget(orig, 4);
  int vi = vsliderz->value();
  QSlider *_vslider = setupVslider(dest, vi);
  QPushButton *btnz = (QPushButton *)musicTable[currentSession]->cellWidget(orig, 5);
  QPushButton *_btn = setupMbutton(dest);
  
  musicTable[currentSession]->removeRow(orig);    
  musicTable[currentSession]->insertRow(dest);
  musicTable[currentSession]->setItem(orig, 0, iItem);
  
  musicTable[currentSession]->setItem(dest, 0, iItem1);
  musicTable[currentSession]->setItem(dest, 1, nItem);
  musicTable[currentSession]->setCellWidget(dest, 2, _slcombo);
  musicTable[currentSession]->setCellWidget(dest, 3, _scombo);
  musicTable[currentSession]->setCellWidget(dest, 4, _vslider);
  musicTable[currentSession]->setCellWidget(dest, 5, _btn);
  musicTable[currentSession]->setCellWidget(dest, 6, dItem);
  
  CyLoop *dummyloop;
  dummyloop = loopz[orig][currentSession];
  loopz[orig][currentSession] = loopz[dest][currentSession];
  loopz[dest][currentSession] = dummyloop;
  
  tableClicked(dest, 6);
  refreshMusicTable(-1, false);
  if (main->jax.state() == 0) {
    loopz[orig][currentSession]->setLoopIndex(orig, currentSession);
    loopz[dest][currentSession]->setLoopIndex(dest, currentSession);
  }
  
  delete slcomboz;
  delete scomboz;
  delete vsliderz;
  delete btnz;
  movingLoop = false;
}

void CyTabWidget::loopUp() {
  if (musicTable[currentSession]->rowCount() == 0)
    return;
  int curo;
  if (sel[currentSession] > 127)
    curo = musicTable[currentSession]->rowCount();
  else
    curo = sel[currentSession];
  
  if (curo > 0) {
    curo--;
    tableClicked(curo, 6);
  } else {
    curo = musicTable[currentSession]->rowCount() - 1;
    tableClicked(curo, 6);
  }
  cLoop();
}

void CyTabWidget::loopDown() {
  if (musicTable[currentSession]->rowCount() == 0)
    return;
  int curo;
  if (sel[currentSession] > 127)
    curo = -1;
  else
    curo = sel[currentSession];  
  
  if (curo < (musicTable[currentSession]->rowCount() - 1)) {
    curo++;
    tableClicked(curo, 6);
  } else {
    curo = 0;
    tableClicked(curo, 6);
  }
  cLoop();
}

void CyTabWidget::loopFirst() {
  if (musicTable[currentSession]->rowCount() == 0)
    return;
  
  tableClicked(0, 6);
  cLoop();
}

void CyTabWidget::loopLast() {
  if (musicTable[currentSession]->rowCount() == 0)
    return;
  
  tableClicked(musicTable[currentSession]->rowCount() - 1, 6);
  cLoop();
}

void CyTabWidget::cLoop() {
  if (playD) {
    loopresponse ls;
    if ((loopz[sel[currentSession]][currentSession]->state == 1) || (loopz[sel[currentSession]][currentSession]->state == 2)) {
      if ((loopz[sel[currentSession]][currentSession]->getSLmode() == Multiply) || (loopz[sel[currentSession]][currentSession]->getSLmode() == MultiOverdub)) {
	loopz[sel[currentSession]][currentSession]->state = 2;
	ls = loopz[sel[currentSession]][currentSession]->playLoop();
      } else {
	loopz[sel[currentSession]][currentSession]->state = 0;
	ls = loopz[sel[currentSession]][currentSession]->playLoop();
	ls.currentstate = 1;  
      }
      if (loopz[sel[currentSession]][currentSession]->getSmode() == Once)
	loopz[sel[currentSession]][currentSession]->untrigger();  
    } else {
      if (loopz[sel[currentSession]][currentSession]->getSmode() != Asynchronous) {
	int strig = loopz[sel[currentSession]][currentSession]->getStrig();
	ls = loopz[sel[currentSession]][currentSession]->playLoop();
	if ((ls.oldindex == pIndex) && (ls.oldsession == pSession) && (strig == 1) && (loopz[ls.oldindex][ls.oldsession]->state == 1)) {
	    ls.oldstate = 1;
	}
      } else {
	ls = loopz[sel[currentSession]][currentSession]->playLoop();
      }
    }
        
    if (ls.currentstate > -1) {
      processResponse(ls.oldindex, ls.oldsession, ls.oldstate, ls.currentindex, ls.currentsession, ls.currentstate);
    }   
  }
}

void CyTabWidget::processResponse(int oldi, int olds, int oldstate, int curi, int curs, int curstate) {
  if (main->windebug) {
    main->printOutput(QString("oldi: %1\tolds %2\toldstate %3") .arg(oldi) .arg(olds) .arg(oldstate));
    main->printOutput(QString("curi: %1\tcurs %2\tcurstate %3\n\t***\t***\t***") .arg(curi) .arg(curs) .arg(curstate));
  }
  if (oldstate > -1) {
    loopz[oldi][olds]->state = oldstate;
    QPushButton *butn = (QPushButton *)musicTable[olds]->cellWidget(oldi, 5);
    
    if (musicTable[olds]->cellWidget(oldi,6)) {
      QWidget *dItem = new QWidget(musicTable[olds]);
      musicTable[olds]->setCellWidget(oldi,6,dItem);
    }
    QLabel *lbl = new QLabel("");
    lbl->setTextFormat(Qt::RichText);
    
    switch (loopz[oldi][olds]->state) {
      case 0:
	butn->setIcon(QIcon(":/icons/start.png"));
	break;
      case 1:
	butn->setIcon(QIcon(":/icons/stop.png"));
	lbl->setText("<center><b><font color=green>" + tr("Play") + "</font></b></center>");
	musicTable[olds]->setCellWidget(oldi,6,lbl);  
	pIndex = oldi; 
	pSession = olds;
	played = true;
	break;
      case 2:
	butn->setIcon(QIcon(":/icons/fwd.png"));
	lbl->setText("<center><b><font color=blue>" + tr("Multi") + "</font></b></center>");
	musicTable[olds]->setCellWidget(oldi,6,lbl);    
	pIndex = oldi; 
	pSession = olds; 
	played = true;   
	break;
      case 3:
	musicTable[olds]->setCellWidget(oldi,6,lbl);
	lbl->setText("<center><b><font color=red>" + tr("Next") + "</font></b></center>");
	break;
      case 4:
        musicTable[olds]->setCellWidget(curi,6,lbl);  
        lbl->setText("<center><b><u><i>" + tr("Async") + "</i></u></b></center>");
        break;
      case 5:
	butn->setIcon(QIcon(":/icons/start.png"));
	break;
    }  
  }
  
  if (musicTable[curs]->cellWidget(curi,6)) {
    QWidget *dItem = new QWidget(musicTable[curs]);
    musicTable[curs]->setCellWidget(curi,6,dItem);
  }
  QLabel *lbl1 = new QLabel("");
  lbl1->setTextFormat(Qt::RichText);
  loopz[curi][curs]->state = curstate;
  QPushButton *butn1 = (QPushButton *)musicTable[curs]->cellWidget(curi, 5);
  switch (loopz[curi][curs]->state) {
    case 0:
      butn1->setIcon(QIcon(":/icons/start.png"));
      break;
    case 1:
      butn1->setIcon(QIcon(":/icons/stop.png"));  
      lbl1->setText("<center><b><font color=green>" + tr("Play") + "</font></b></center>");
      musicTable[curs]->setCellWidget(curi,6,lbl1);
      pIndex = curi; 
      pSession = curs;
      played = true;
      break;
    case 2:
      butn1->setIcon(QIcon(":/icons/fwd.png")); 
      lbl1->setText("<center><b><font color=blue>" + tr("Multi") + "</font></b></center>");
      musicTable[curs]->setCellWidget(curi,6,lbl1);  
      pIndex = curi; 
      pSession = curs;
      played = true;
      break;  
    case 3:
      musicTable[curs]->setCellWidget(curi,6,lbl1);  
      lbl1->setText("<center><b><font color=red>" + tr("Next") + "</font></b></center>");
      break;   
    case 4:
      butn1->setIcon(QIcon(":/icons/stop.png"));
      musicTable[curs]->setCellWidget(curi,6,lbl1);  
      lbl1->setText("<center><b><u><i>" + tr("Async") + "</i></u></b></center>");
      break; 
    case 5:
      butn1->setIcon(QIcon(":/icons/start.png"));
  }
}

void CyTabWidget::doScroll(int item, int ssn) {
  setCurrentIndex(ssn);
  QModelIndex mi(musicTable[ssn]->indexAt(QPoint(100, musicTable[ssn]->rowViewportPosition(item))));
  musicTable[ssn]->scrollTo(mi, QAbstractItemView::EnsureVisible);
}


void CyTabWidget::pLoop() {
  if (main->jax.state() != 0)
    return;
  if (playD) { /* stopping */
    playD = false;
    main->playL->setIcon(QIcon(":/icons/start.png"));
    if (played) {
      if (loopz[pIndex][pSession]->state > 0) {
	loopresponse ls;
	if (loopz[pIndex][pSession]->state == 2)
	  ls = loopz[pIndex][pSession]->playLoop();
	ls = loopz[pIndex][pSession]->playLoop();
	if (ls.currentstate > -1) {
	  processResponse(ls.oldindex, ls.oldsession, ls.oldstate, ls.currentindex, ls.currentsession, ls.currentstate);
	}
      }
      played = false;
    }
  } else { /* starting */
    if (musicTable[currentSession]->rowCount() < 1)
      return;    
    if (sel[currentSession] > 127) {
      sel[currentSession] = 0;
      tableClicked(0, 6);
    }
    playD = true;
    main->playL->setIcon(QIcon(":/icons/pause.png"));
    if ((loopz[sel[currentSession]][currentSession]->state == 0) || (loopz[sel[currentSession]][currentSession]->state == 5)) {
      loopresponse ls;
      ls = loopz[sel[currentSession]][currentSession]->playLoop();
      if (ls.currentstate != 1)
	played = false;
      if (ls.currentstate > -1) {
	processResponse(ls.oldindex, ls.oldsession, ls.oldstate, ls.currentindex, ls.currentsession, ls.currentstate);
      }
    } else {
      loopresponse ls;
      if ((loopz[sel[currentSession]][currentSession]->getSLmode() == Multiply) || (loopz[sel[currentSession]][currentSession]->getSLmode() == MultiOverdub)) {
	loopz[sel[currentSession]][currentSession]->state = 2;
      } else {
	loopz[sel[currentSession]][currentSession]->state = 0;	
      }
      ls = loopz[sel[currentSession]][currentSession]->playLoop();
      if (loopz[sel[currentSession]][currentSession]->getSmode() == Asynchronous) {
	played = false;
	ls.currentstate = 4;
      } else {
	ls.currentstate = 1;
      }
      processResponse(ls.oldindex, ls.oldsession, ls.oldstate, ls.currentindex, ls.currentsession, ls.currentstate);
    }
  }
}


void CyTabWidget::dragEnterEvent(QDragEnterEvent *event) {
  QStringList filez = event->mimeData()->text().split(QRegExp("\n"));
  QFileInfo *qfi;
  foreach (QString file, filez) {
    if (!file.isEmpty()) {
      qfi = new QFileInfo(file);
      if ((qfi->suffix().contains("wav", Qt::CaseInsensitive)) || (qfi->suffix().contains("aif", Qt::CaseInsensitive)) || (qfi->suffix().contains("flac", Qt::CaseInsensitive))) {
	if (main->windebug)
	  main->printOutput(QString("Receiving: %1") .arg(QUrl(file).path()));
	event->acceptProposedAction();
      }
    }
  }
}

void CyTabWidget::dropEvent(QDropEvent *event) {
  QStringList filez = event->mimeData()->text().split(QRegExp("\n"));
  QFileInfo *qfi;
  bool max = false;
  QApplication::setOverrideCursor(Qt::WaitCursor);
  foreach (QString file, filez) {
    if (!file.isEmpty()) {
      qfi = new QFileInfo(file);
      if ((qfi->suffix().contains("wav", Qt::CaseInsensitive)) || (qfi->suffix().contains("aif", Qt::CaseInsensitive)) || (qfi->suffix().contains("flac", Qt::CaseInsensitive))) {
	if (musicTable[currentSession]->rowCount() <= 127) { 
	  if (addMusicTableItem(QUrl(file).path(), currentSession)) {
	    if (main->windebug)
	      main->printOutput(QString("Loading: %1") .arg(QUrl(file).path()));
	  }
	} else {
	  if (!max) {
	    QApplication::restoreOverrideCursor();
	    max = true;
	    QMessageBox::warning(main, tr("Cyclone - Add Loops"),
			  tr("Maximum number of loop for this session reached.\n"
			      "Please create a new session or delete loops."), QMessageBox::Cancel);
	    main->undo->pushed = false;
	    break;
	  }
	}
      }
    }
  }
  QApplication::restoreOverrideCursor();  
}

void CyTabWidget::refreshMusicTable(int previousSession, bool move) {
  int start, end;
  int rowcur = musicTable[currentSession]->currentRow();
  int rowcnt = musicTable[currentSession]->rowCount();
  if (previousSession < 0) {
    if (rowcur > 0) {
      start = rowcur - 1;
    } else {
      start = 0;
    }
    if (rowcur < (rowcnt - 1)) {
      end = rowcur + 2;
    } else {
      end = rowcnt;
    }
    previousSession = currentSession;
  } else {
    start = 0;
    end = rowcnt;
    if (musicTable[previousSession]->rowCount() > rowcnt) {
      end = musicTable[previousSession]->rowCount();
    }
  }
  
  QApplication::setOverrideCursor(Qt::WaitCursor);
  
  for (int loopItem = start; loopItem < end; loopItem++) {
    if (currentSession != previousSession) {
      CyLoop *dummy = loopz[loopItem][currentSession];
      loopz[loopItem][currentSession] = loopz[loopItem][previousSession];
      loopz[loopItem][previousSession] = dummy;
      if (move) {
	setLoopAttributes(loopItem, previousSession);
      } else {
	unsetLoopAttributes(loopItem, previousSession);
	unsetLoopAttributes(loopItem, currentSession);
      }
    }
    setLoopAttributes(loopItem, currentSession);
  }
  if ((currentSession != previousSession) && move)
    main->cymidi->refresh(previousSession);
  main->cymidi->refresh(currentSession);
  QApplication::restoreOverrideCursor();
}

void CyTabWidget::unsetSignalMappings(int sex) {
  int cnt = musicTable[currentSession]->rowCount();
  if (cnt == 0)
    return;
  for (int lp = 0; lp < cnt; lp++) {
    unsetLoopAttributes(lp, sex);
  }
}
void CyTabWidget::unsetLoopAttributes(int lp, int sex) {
  if (lp < musicTable[sex]->rowCount()) {
    QComboBox *slcombo = (QComboBox *)musicTable[sex]->cellWidget(lp, 2);
    signalMapper->removeMappings(slcombo);
    QComboBox *scombo = (QComboBox *)musicTable[sex]->cellWidget(lp, 3);
    signalMapper->removeMappings(scombo);
    QSlider *slider = (QSlider *)musicTable[sex]->cellWidget(lp, 4);
    signalMapper->removeMappings(slider);
    signalMapper->removeMappings(dummyVol[lp][sex]);
    signalMapper->removeMappings(dummyMidiVol[lp][sex]);
    QPushButton *btn = (QPushButton *)musicTable[sex]->cellWidget(lp, 5);
    signalMapper->removeMappings(btn);
    signalMapper->removeMappings(dummyButton[lp][sex]);
  }
}

void CyTabWidget::setLoopAttributes(int lp, int sex) {
  if (lp < musicTable[sex]->rowCount()) {
    if (loopz[lp][sex]->dead)
      return;
    loopz[lp][sex]->setLoopIndex(lp, sex);
    loopz[lp][sex]->setIndex(musicTable[sex]->item(lp, 0)->text().toInt() - 1);    
    loopz[lp][sex]->setName(musicTable[sex]->item(lp, 1)->text());
    
    QComboBox *slcombo = (QComboBox *)musicTable[sex]->cellWidget(lp, 2);
    signalMapper->removeMappings(slcombo);
    signalMapper->setMapping(slcombo, QString("%1-%2-2") .arg(sex) .arg(lp));
    loopz[lp][sex]->setSLmode(main->aux->indexToSlmode(slcombo->currentIndex()));
    
    QComboBox *scombo = (QComboBox *)musicTable[sex]->cellWidget(lp, 3);
    signalMapper->removeMappings(scombo);
    signalMapper->setMapping(scombo, QString("%1-%2-3") .arg(sex) .arg(lp));
    loopz[lp][sex]->setSmode(main->aux->indexToSmode(scombo->currentIndex()));
    
    if (scombo->currentIndex() == 3) {
      slcombo->setEnabled(false);
    } else {
      slcombo->setEnabled(true); 
    }
    
    QSlider *slider = (QSlider *)musicTable[sex]->cellWidget(lp, 4);
    main->undo->oldloopvol[lp][sex] = slider->value();
    main->undo->oldsl[lp][sex] = slcombo->currentIndex();
    main->undo->olds[lp][sex] = scombo->currentIndex();
    signalMapper->removeMappings(slider);
    signalMapper->setMapping(slider, QString("%1-%2-4") .arg(sex) .arg(lp));
    signalMapper->removeMappings(dummyVol[lp][sex]);
    signalMapper->removeMappings(dummyMidiVol[lp][sex]);
    delete dummyVol[lp][sex];
    delete dummyMidiVol[lp][sex];
    dummyVol[lp][sex] = new DummyWidget();
    dummyMidiVol[lp][sex] = new DummyWidget();
    connect(slider, SIGNAL(customContextMenuRequested(QPoint)), dummyMidiVol[lp][sex], SLOT(signalz()));
    connect(dummyMidiVol[lp][sex], SIGNAL(signalx()), signalMapper, SLOT(map()));
    signalMapper->setMapping(dummyMidiVol[lp][sex], QString("%1-%2-40") .arg(sex) .arg(lp));
    connect(slider, SIGNAL(sliderReleased()), dummyVol[lp][sex], SLOT(signalz()));
    connect(dummyVol[lp][sex], SIGNAL(signalx()), signalMapper, SLOT(map()));
    signalMapper->setMapping(dummyVol[lp][sex], QString("%1-%2-44") .arg(sex) .arg(lp));
    loopz[lp][sex]->setVolume(slider->value());  
    
    QPushButton *btn = (QPushButton *)musicTable[sex]->cellWidget(lp, 5);
    signalMapper->removeMappings(btn);
    signalMapper->setMapping(btn, QString("%1-%2-5") .arg(sex) .arg(lp));
    signalMapper->removeMappings(dummyButton[lp][sex]);
    delete dummyButton[lp][sex];
    dummyButton[lp][sex] = new DummyWidget();
    connect(btn, SIGNAL(customContextMenuRequested(QPoint)), dummyButton[lp][sex], SLOT(signalz()));
    connect(dummyButton[lp][sex], SIGNAL(signalx()), signalMapper, SLOT(map()));
    signalMapper->setMapping(dummyButton[lp][sex], QString("%1-%2-55") .arg(sex) .arg(lp));
  }
}

void CyTabWidget::unselect(int sex) {
  if (musicTable[sex]->rowCount() == 0) {
    sel[sex] = 255;
    return;
  }
  if (sel[sex] < 128) {
    QTableWidgetSelectionRange qts(0, 0, musicTable[sex]->rowCount() - 1, musicTable[sex]->columnCount() - 1);
    musicTable[sex]->setRangeSelected(qts, false);
    if ((main->aux->graphicsfx == 1) && (sel[sex] < 128)) {
      int cs = currentSession;
      currentSession = sex;
      colorizeLoop(false);
      currentSession = cs;
    }
    main->copyLoop->setDisabled(true);
    sel[sex] = 255;
  } 
}

void CyTabWidget::addSessionSlot() {
  if (sessionsCount < 127) {
    infinite_count++;
    main->undo->push(new CommandAddSession(main));
    sessionsCount++;
  }
  if (sessionsCount <= 126) {
    main->addSessionAction->setEnabled(true);
    main->removeSessionAction->setEnabled(true);
  } else {
    main->addSessionAction->setDisabled(true);
  }
}

void CyTabWidget::renameSession() {
  RenameSession rs(main);
  rs.lineEdit->setText(tabText(currentSession));
  rs.lineEdit->setSelection(0,13);
  if (rs.exec()) {
    main->undo->push(new CommandSessionName(main, currentSession, rs.lineEdit->text()));
  }
}

void CyTabWidget::setSession(int sex) {
  setCurrentIndex(sex);
}

QString CyTabWidget::sessionName(int sex) {
  return musicTableTitle[sex];    
}

int CyTabWidget::loopCount(int sex) {
  return musicTable[sex]->rowCount();    
}

void CyTabWidget::first() {
  if (main->focus == "Resources") {
    if (main->windebug)
      main->printOutput(QString("--->first"));
    loopFirst();
  }
}

void CyTabWidget::rew() {
  if (main->focus == "Resources") {
    if (main->windebug)
      main->printOutput(QString("--->rew"));
    loopUp();
  }
}
/*
void CyTabWidget::rec() {
  if (main->focus = "Resources") {
    if (main->windebug)
    printOutput(QString("--->rec"));
  }
}
*/
void CyTabWidget::fwd() {
  if (main->focus == "Resources") {
    if (main->windebug)
      main->printOutput(QString("--->fwd"));
    loopDown();
  }
}

void CyTabWidget::last() {
  if (main->focus == "Resources") {
    if (main->windebug)
      main->printOutput(QString("--->last"));
    loopLast();
  }
}

void CyTabWidget::play() {
  if (main->focus == "Resources") {
    if (main->windebug)
      main->printOutput(QString("--->play"));
    pLoop();
  }
}

void CyTabWidget::stop() {
  if (main->focus == "Resources") {
    if (main->windebug)
      main->printOutput(QString("--->stop"));
    main->stop();
  }
}