/* CyTab.h
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#ifndef CYTAB_H
#define CYTAB_H

#include <QTableWidget>
#include <QSignalMapper>
#include <QComboBox>
#include <QSlider>
#include <QPushButton>
#include <QToolBar>

#include "CyLoop.h"
#include "typedefs.h"

class DummyWidget;
class WinCycle;

class CyTabWidget : public QTabWidget {
    Q_OBJECT
    
public:
  CyTabWidget(WinCycle *parent);
  
  void addSession(int count, int ssn, QString title = "");
  bool addLoop(CyLoop *loop, int sex);
  bool verifyCoords(int x, int y);
  QSize generateCoords();
  void refreshMusicTable(int previousSession, bool move);
  bool addMusicTableItem(QString nome, int ssn);
  void setSession(int sex);
  QString sessionName(int sex);
  int loopCount(int sex);
  void removeSession(int sex);
  void moveTabLeft(int index);
  void moveTabRight(int index);
  void moveLoop(int orig, int dest);
  void rmLoop(int x, int y);
  void insLoop(CyLoop *loop, int x, int y);
  void colorizeLoop(bool colorize);
  void unselect(int sex);
  void unsetSignalMappings(int sex);
  
  QTableWidget *musicTable[128];
  QString musicTableTitle[128];
  CyLoop *loopz[128][128];
  bool playD;
  bool played;
  int currentLoop;
  int currentSession;
  int sessionsCount;
  int infinite_count;
  QSignalMapper* signalMapper;
  QString currentSessionName;
  QToolBar *toolBar;
  int sel[128];
  bool movingLoop;
   
  public slots:
    void addFiles();
    void sessionRight();
    void sessionLeft();
    void moveSessionRight();
    void moveSessionLeft();
    void tableClicked(int row, int column);
    void removeLoop();
    void moveLoopUp();
    void moveLoopDown();
    void loopUp();
    void loopDown();
    void pLoop();
    void addSessionSlot();
    void renameSession();
    void removeSessionSlot();
    void doScroll(int item, int ssn);
    void processResponse(int oldi, int olds, int oldstate, int curi, int curs, int curstate);
    void activateNameChange(int x, int y);
    void first();
    void rew();
    //void rec();
    void fwd();
    void last();
    void play();
    void stop();
    
  private:
    QComboBox *setupSLcombo(int row, slmode slm);
    QComboBox *setupScombo(int row, smode sm);
    QSlider *setupVslider(int row, int vl);
    QPushButton *setupMbutton(int row);
    void cLoop();
    void loopVolumeUndo(int x, int y);
    void unsetLoopAttributes(int lp, int sex);
    void setLoopAttributes(int lp, int sex);
    bool analyzeLoop(CyLoop *loop, int session);
    void loopFirst();
    void loopLast();
    
    WinCycle *main; 
    int pIndex, pSession;
    DummyWidget *dummyButton[128][128];
    DummyWidget *dummyVol[128][128];
    DummyWidget *dummyMidiVol[128][128];
    
   
private slots:
  void currentSessionChanged(int csc);
  void tableChanged(int row, int column);
  void tableWidgetChanged(QString position);
  
  protected:
   virtual void dragEnterEvent(QDragEnterEvent *event);
   virtual void dropEvent(QDropEvent *event);
   

};

#endif
 
