/* CyUi.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#include <QtGui>
#include "CyUi.h"
#include "WinCycle.h"

CyUi::CyUi(WinCycle *parent) {
  main = parent;
  setupActions();
  setupMenus();
}


void CyUi::setupUi() { 
    main->printOutput(tr("Starting GUI Interface"));
    createToolBar();
    main->sessions->setUsesScrollButtons(true);
    connect(main->sessions, SIGNAL(currentChanged(int)), main->sessions, SLOT(currentSessionChanged(int)));
    
    main->sessions->currentSession = 0;
    main->sessions->infinite_count = 1;
    main->sessions->sessionsCount = 1;
    main->sessions->addSession(main->sessions->infinite_count, main->sessions->currentSession);
    
    setupButtons();
    
    setupResources();
    resources->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::TopDockWidgetArea | Qt::RightDockWidgetArea);
    setupControls();
    controls->setAllowedAreas(Qt::BottomDockWidgetArea);
    
    main->addDockWidget(Qt::TopDockWidgetArea, resources, Qt::Horizontal);
    main->addDockWidget(Qt::BottomDockWidgetArea, controls, Qt::Horizontal);
    
    QWidget *widget = new QWidget(main);
    widget->setMaximumHeight(0);
    //main->setCentralWidget(widget);
    createStatusBar();
}

QDockWidget *CyUi::setupControls() { 
    QHBoxLayout *secondo = new QHBoxLayout; 
    secondo->setSpacing(0);
    secondo->addWidget(main->ejectL);
    secondo->addWidget(main->firstL);
    secondo->addWidget(main->rewL);
    //secondo->addWidget(main->recL);
    secondo->addWidget(main->playL);
    secondo->addWidget(main->stopL);
    secondo->addWidget(main->fwdL);
    secondo->addWidget(main->lastL);
    
    /*QHBoxLayout *terzo = new QHBoxLayout;    
    terzo->setContentsMargins(10,1,10,1);
    terzo->setAlignment(Qt::AlignHCenter);
    terzo->setSpacing(2);
    terzo->addWidget(main->playStop);*/
    
    sigMeter = new Meter(main);
    
    QHBoxLayout *quarto1 = new QHBoxLayout;
    quarto1->addWidget(main->volume);
    quarto1->addWidget(volLabel);
    
    QVBoxLayout *quarto2 = new QVBoxLayout; 
    quarto2->addLayout(quarto1);
    quarto2->addWidget(sigMeter);
    
    QHBoxLayout *quarto = new QHBoxLayout; 
    quarto->setContentsMargins(10,1,5,1);
    quarto->addWidget(main->mute);
    quarto->addLayout(quarto2);
    connect(&main->jax, SIGNAL(sigChanged(int,int)),this, SLOT(updateStatusSig(int,int)));
    
    QHBoxLayout *quinto = new QHBoxLayout;
    quinto->setContentsMargins(10,0,0,0);
    /*quinto->addLayout(terzo);
    quinto->addStretch(10);*/
    quinto->addLayout(secondo);
    quinto->addStretch(10);
    quinto->addLayout(quarto);
    
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(0,0,0,0);
    mainLayout->addLayout(quinto);

    QWidget *widget = new QWidget;
    widget->setLayout(mainLayout);
    widget->setMaximumHeight(50);
    
    controls = new QDockWidget("Controls");
    QWidget *title = new QWidget;
    controls->setTitleBarWidget(title);
    controls->setWidget(widget);
    controls->setFeatures(QDockWidget::NoDockWidgetFeatures);
    sigMeter->startPeaks();
    return controls;
}

QDockWidget *CyUi::setupResources() { 
    QHBoxLayout *primo = new QHBoxLayout;
    primo->setContentsMargins(2,0,2,0);
    primo->setAlignment(Qt::AlignVCenter);
    primo->addWidget(main->sessions);
    
    QVBoxLayout *secondo = new QVBoxLayout;
    secondo->setAlignment(Qt::AlignVCenter);
    secondo->setSpacing(2);
    secondo->addWidget(main->moveUp);
    secondo->addWidget(main->nextUp);
    secondo->addWidget(main->nextDown);
    secondo->addWidget(main->moveDown);
    primo->addLayout(secondo);
    
    QHBoxLayout *terzo = new QHBoxLayout;    
    terzo->setContentsMargins(10,1,10,1);
    terzo->setAlignment(Qt::AlignHCenter);
    terzo->setSpacing(2);
    terzo->addWidget(main->moveLeft);
    terzo->addWidget(main->nextLeft);
    terzo->addWidget(main->nextRight);
    terzo->addWidget(main->moveRight);
    
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(0,0,0,0);
    mainLayout->addLayout(primo);
    mainLayout->addLayout(terzo);

    QWidget *widget = new QWidget;
    widget->setLayout(mainLayout);
    resources = new QDockWidget(tr("Resources"));
    resources->setWidget(widget);
    resources->setFeatures(QDockWidget::DockWidgetMovable);
    return resources;
}

void CyUi::setupMenus()
{
    QMenu *fileMenu = main->menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(main->newProjectAction);
    fileMenu->addAction(main->openProjectAction);
    fileMenu->addAction(main->saveProjectAction);
    fileMenu->addAction(main->saveAsProjectAction);
    fileMenu->addSeparator();
    fileMenu->addAction(main->exitAction);
    
    QMenu *editMenu = main->menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(main->undoAction);
    editMenu->addAction(main->redoAction);
    editMenu->addSeparator();
    editMenu->addAction(main->addFilesAction);
    editMenu->addAction(main->removeLoopAction);
    editMenu->addAction(main->copyLoop);
    editMenu->addAction(main->pasteLoop);
    editMenu->addSeparator();
    editMenu->addAction(main->renameSessionAction);
    editMenu->addAction(main->addSessionAction);
    editMenu->addAction(main->removeSessionAction);
    editMenu->addAction(main->copySession);
    editMenu->addAction(main->pasteSession);
    editMenu->addSeparator();
    editMenu->addAction(main->preferencesAction);
    
    QMenu *jackMenu = main->menuBar()->addMenu(tr("&Jack"));
    jackMenu->addAction(main->stopJack);
    jackMenu->addAction(main->startJack);
    fileMenu->addSeparator();
    jackMenu->addAction(main->connectSooperlooper);

    QMenu *aboutMenu = main->menuBar()->addMenu(tr("&Help"));
    aboutMenu->addAction(main->helpAction);
    aboutMenu->addAction(main->websiteAction);
    aboutMenu->addAction(main->aboutAction);
    aboutMenu->addAction(main->aboutQtAction);
}

void CyUi::setupActions()
{
    main->addFilesAction = new QAction(tr("Add &Loops"), main);
    main->addFilesAction->setStatusTip(tr("Add Loops"));
    main->addFilesAction->setIcon(QIcon(":/icons/add.png"));
    main->addFilesAction->setShortcut(tr("Ctrl+L"));
    main->addSessionAction = new QAction(tr("Add &Session"), main);
    main->removeSessionAction = new QAction(tr("&Remove Current Session"), main);
    main->removeSessionAction->setDisabled(true);
    main->removeLoopAction = new QAction(tr("Remo&ve Current Loop"), main);
    main->removeLoopAction->setStatusTip(tr("Remove Current Loop"));
    main->removeLoopAction->setIcon(QIcon(":/icons/remove.png"));
    main->removeLoopAction->setShortcut(tr("Del"));
    main->removeLoopAction->setDisabled(true);
    main->renameSessionAction = new QAction(tr("Re&name Current Session"), main);
    main->preferencesAction = new QAction(tr("Pre&ferences"), main);
    main->stopJack = new QAction(tr("Stop Jac&k client"), main);
    main->startJack = new QAction(tr("Star&t Jack client"), main);
    main->connectSooperlooper = new QAction(tr("Con&nect to Sooperlooper"), main);
    main->exitAction = new QAction(tr("E&xit"), main);
    main->exitAction->setShortcut(tr("Ctrl+Q"));
    main->websiteAction = new QAction(tr("&Website"), main);
    main->websiteAction->setIcon(QIcon(":/icons/website.png"));
    main->helpAction = new QAction(tr("H&elp"), main);
    main->helpAction->setIcon(QIcon(":/icons/help.png"));
    main->aboutAction = new QAction(tr("A&bout"), main);
    main->aboutAction->setIcon(QIcon(":/icons/cyclone.png"));
    main->aboutQtAction = new QAction(tr("About &Qt"), main);
    main->aboutQtAction->setIcon(QIcon(":/icons/qt4.png"));
    main->openProjectAction = new QAction(tr("&Open Project"), main);
    main->openProjectAction->setStatusTip(tr("Open Project"));
    main->openProjectAction->setIcon(QIcon(":/icons/open.png"));
    main->openProjectAction->setShortcut(tr("Ctrl+O"));
    main->saveProjectAction = new QAction(tr("&Save Project"), main);
    main->saveProjectAction->setStatusTip(tr("Save Project"));
    main->saveProjectAction->setIcon(QIcon(":/icons/save.png"));
    main->saveProjectAction->setDisabled(true);
    main->saveProjectAction->setShortcut(tr("Ctrl+S"));
    main->saveAsProjectAction = new QAction(tr("Save Project &as.."), main);
    main->saveAsProjectAction->setDisabled(true);
    main->newProjectAction = new QAction(tr("&New Project"), main);
    main->newProjectAction->setStatusTip(tr("New Project"));
    main->newProjectAction->setIcon(QIcon(":/icons/new.png"));
    main->newProjectAction->setShortcut(tr("Ctrl+N"));
    main->copyLoop = new QAction(tr("&Copy Loop"), main);
    main->copyLoop->setStatusTip(tr("Copy Current Loop"));
    main->copyLoop->setIcon(QIcon(":/icons/copy.png"));
    main->copyLoop->setDisabled(true);
    main->copyLoop->setShortcut(tr("Ctrl+C"));
    main->copySession = new QAction(tr("Cop&y Session"), main);
    main->copySession->setDisabled(true);
    main->pasteLoop = new QAction(tr("&Paste Loop"), main);
    main->pasteLoop->setStatusTip(tr("Paste Loop into Current Session"));
    main->pasteLoop->setIcon(QIcon(":/icons/paste.png"));
    main->pasteLoop->setDisabled(true);
    main->pasteLoop->setShortcut(tr("Ctrl+V"));
    main->pasteSession = new QAction(tr("Pas&te Session"), main);
    main->pasteSession->setDisabled(true);
    main->undoAction = main->undo->createUndoAction(main, tr("Undo"));
    main->undoAction->setIcon(QIcon(":/icons/undo.png"));
    main->undoAction->setShortcut(QKeySequence::Undo);
    main->redoAction = main->undo->createRedoAction(main, tr("Redo"));
    main->redoAction->setIcon(QIcon(":/icons/redo.png"));
    main->redoAction->setShortcut(QKeySequence::Redo);
    
    connect(main->addFilesAction, SIGNAL(triggered()), main->sessions, SLOT(addFiles()));
    connect(main->newProjectAction, SIGNAL(triggered()), main, SLOT(newProject()));
    connect(main->addSessionAction, SIGNAL(triggered()), main->sessions, SLOT(addSessionSlot()));
    connect(main->removeLoopAction, SIGNAL(triggered()), main->sessions, SLOT(removeLoop()));
    connect(main->removeSessionAction, SIGNAL(triggered()), main->sessions, SLOT(removeSessionSlot()));
    connect(main->renameSessionAction, SIGNAL(triggered()), main->sessions, SLOT(renameSession()));
    connect(main->preferencesAction, SIGNAL(triggered()), main, SLOT(preferences()));
    connect(main->stopJack, SIGNAL(triggered()), main, SLOT(stopJ()));
    connect(main->startJack, SIGNAL(triggered()), main, SLOT(startJ()));
    connect(main->connectSooperlooper, SIGNAL(triggered()), main, SLOT(csl()));
    connect(main->exitAction, SIGNAL(triggered()), main, SLOT(close()));
    connect(main->helpAction, SIGNAL(triggered()), main->aux, SLOT(help()));
    connect(main->websiteAction, SIGNAL(triggered()), main->aux, SLOT(website()));
    connect(main->aboutAction, SIGNAL(triggered()), main, SLOT(about()));
    connect(main->aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    connect(main->openProjectAction, SIGNAL(triggered()), main->aux, SLOT(openFile()));
    connect(main->saveProjectAction, SIGNAL(triggered()), main->aux, SLOT(saveFile()));
    connect(main->saveAsProjectAction, SIGNAL(triggered()), main->aux, SLOT(saveFileAs()));
    connect(main->copyLoop, SIGNAL(triggered()), main->aux, SLOT(copyLoop()));
    connect(main->copySession, SIGNAL(triggered()), main->aux, SLOT(copySession()));
    connect(main->pasteLoop, SIGNAL(triggered()), main->aux, SLOT(pasteLoop()));
    connect(main->pasteSession, SIGNAL(triggered()), main->aux, SLOT(pasteSession()));
    
    connect(&main->jax, SIGNAL(tabChanged(int,int,int,int,int,int)), main->sessions, SLOT(processResponse(int,int,int,int,int,int)));
    connect(&main->jax, SIGNAL(scroll(int,int)), main->sessions, SLOT(doScroll(int,int)));
    connect(&main->jax, SIGNAL(started()), main, SLOT(jackUp()));
    connect(&main->jax, SIGNAL(jackFinished()), main, SLOT(jackDown()));
    connect(&main->jax, SIGNAL(finished()), main, SLOT(jackDown()));
    
    /* Buttons */
    main->mfS = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Right),main);
    main->mbS = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Left),main);
    main->nextS = new QShortcut(QKeySequence(Qt::Key_Right),main);
    main->prevS = new QShortcut(QKeySequence(Qt::Key_Left),main);
    main->pL = new QShortcut(QKeySequence(Qt::Key_Space),main);
    main->sL = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Return),main);
    main->mfL = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Down),main);
    main->mbL = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Up),main);
    main->nextL = new QShortcut(QKeySequence(Qt::Key_Down),main);
    main->prevL = new QShortcut(QKeySequence(Qt::Key_Up),main);
    connect(main->mfS, SIGNAL(activated()), main->sessions, SLOT(moveSessionRight()));
    connect(main->mbS, SIGNAL(activated()), main->sessions, SLOT(moveSessionLeft()));
    connect(main->nextS, SIGNAL(activated()), main->sessions, SLOT(sessionRight()));
    connect(main->prevS, SIGNAL(activated()), main->sessions, SLOT(sessionLeft()));
    connect(main->pL, SIGNAL(activated()), main->sessions, SLOT(pLoop()));
    connect(main->sL, SIGNAL(activated()), main, SLOT(stop()));
    connect(main->mfL, SIGNAL(activated()), main->sessions, SLOT(moveLoopDown()));
    connect(main->mbL, SIGNAL(activated()), main->sessions, SLOT(moveLoopUp()));
    connect(main->nextL, SIGNAL(activated()), main->sessions, SLOT(loopDown()));
    connect(main->prevL, SIGNAL(activated()), main->sessions, SLOT(loopUp()));
    if (main->jax.state() == 0) {
      main->jackUp();
    } else {
      main->jackDown();
    }
    main->sessions->playD = false;
}


void CyUi::setupButtons() { 
  main->moveUp = new QPushButton(main);
  main->moveUp->setToolTip(tr("Move Loop Up (Ctrl + UpArrow)"));
  main->moveUp->setIcon(QIcon(":/icons/go-top.png"));
  main->moveUp->setIconSize(QSize(16, 16));
  connect(main->moveUp, SIGNAL(released()),main->sessions, SLOT(moveLoopUp()));
  main->moveDown = new QPushButton(main);
  main->moveDown->setToolTip(tr("Move Loop Down (Ctrl + DownArrow)"));
  main->moveDown->setIcon(QIcon(":/icons/go-bottom.png"));
  main->moveDown->setIconSize(QSize(16, 16));
  connect(main->moveDown, SIGNAL(released()),main->sessions, SLOT(moveLoopDown()));
  main->moveLeft = new QPushButton(main);
  main->moveLeft->setToolTip(tr("Move Session Backwards (Ctrl + LeftArrow)"));
  main->moveLeft->setIcon(QIcon(":/icons/go-first.png"));
  main->moveLeft->setIconSize(QSize(16, 16));
  connect(main->moveLeft, SIGNAL(released()),main->sessions, SLOT(moveSessionLeft()));
  main->moveRight = new QPushButton(main);
  main->moveRight->setToolTip(tr("Move Session Forwards (Ctrl + RightArrow)"));  
  main->moveRight->setIcon(QIcon(":/icons/go-last.png"));
  main->moveRight->setIconSize(QSize(16, 16));
  connect(main->moveRight, SIGNAL(released()),main->sessions, SLOT(moveSessionRight()));
  main->nextUp = new QPushButton(main);
  main->nextUp->setContextMenuPolicy(Qt::CustomContextMenu);
  main->nextUp->setToolTip(tr("Select/Play Previous Loop (UpArrow)"));
  main->nextUp->setIcon(QIcon(":/icons/go-up.png"));
  main->nextUp->setIconSize(QSize(16, 16));
  connect(main->nextUp, SIGNAL(released()),main->sessions, SLOT(loopUp()));
  connect(main->nextUp, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->nextUp, QString("0-0-99"));
  main->nextDown = new QPushButton(main);
  main->nextDown->setContextMenuPolicy(Qt::CustomContextMenu);
  main->nextDown->setToolTip(tr("Select/Play Next Loop (DownArrow)"));
  main->nextDown->setIcon(QIcon(":/icons/go-down.png"));
  main->nextDown->setIconSize(QSize(16, 16));
  connect(main->nextDown, SIGNAL(released()),main->sessions, SLOT(loopDown()));
  connect(main->nextDown, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->nextDown, QString("0-2-99"));
  main->nextLeft = new QPushButton(main);
  main->nextLeft->setContextMenuPolicy(Qt::CustomContextMenu);
  main->nextLeft->setToolTip(tr("Go to Previous Session (LeftArrow)"));
  main->nextLeft->setIcon(QIcon(":/icons/go-previous.png"));
  main->nextLeft->setIconSize(QSize(16, 16));
  connect(main->nextLeft, SIGNAL(released()),main->sessions, SLOT(sessionLeft()));
  connect(main->nextLeft, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->nextLeft, QString("0-5-99"));
  main->nextRight = new QPushButton(main);
  main->nextRight->setContextMenuPolicy(Qt::CustomContextMenu);
  main->nextRight->setToolTip(tr("Go to Next Session (RightArrow)"));
  main->nextRight->setIcon(QIcon(":/icons/go-next.png"));
  main->nextRight->setIconSize(QSize(16, 16));
  connect(main->nextRight, SIGNAL(released()),main->sessions, SLOT(sessionRight()));
  connect(main->nextRight, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->nextRight, QString("0-3-99"));
  main->mute = new QPushButton(main);
  main->mute->setToolTip(tr("Mute"));
  main->mute->setCheckable(true);
  main->mute->setIcon(QIcon(":/icons/mute.png"));
  main->mute->setIconSize(QSize(24, 24));
  connect(main->mute, SIGNAL(toggled(bool)),main, SLOT(muteButton(bool)));
  volLabel = new QLabel("0");
  volLabel->setFixedWidth(42);
  volLabel->setAlignment(Qt::AlignHCenter);
  main->volume = new QSlider(Qt::Horizontal, main);
  main->volume->setToolTip(tr("General Volume"));
  main->volume->setContextMenuPolicy(Qt::CustomContextMenu);
  main->volume->setTickInterval(2);
  main->volume->setTickPosition(QSlider::TicksAbove);
  main->volume->setFixedWidth(256);
  main->volume->setRange(0,127);
  main->volume->setSliderPosition(100);
  main->undo->oldvol = 100;
  main->gvpress = false;
  connect(main->volume, SIGNAL(valueChanged(int)), main, SLOT(generalVolume(int)));
  connect(main->volume, SIGNAL(sliderReleased()), main, SLOT(generalVolumeUndo()));
  connect(main->volume, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->volume, QString("0-255-98"));
  
  main->stopL = new QPushButton(main);
  main->stopL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->stopL->setToolTip(tr("Stop Looping (Ctrl + Return)"));
  main->stopL->setIcon(QIcon(":/icons/stop.png"));
  main->stopL->setIconSize(QSize(24, 24));
  connect(main->stopL, SIGNAL(released()),main->sessions, SLOT(stop()));
  connect(main->stopL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->stopL, QString("0-4-99"));
  main->playL = new QPushButton(main);
  main->playL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->playL->setToolTip(tr("Play/Stop Current Loop (Space)"));
  main->playL->setIcon(QIcon(":/icons/start.png"));
  main->playL->setIconSize(QSize(24, 24));
  connect(main->playL, SIGNAL(released()),main->sessions, SLOT(play()));
  connect(main->playL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->playL, QString("0-1-99"));
  main->ejectL = new QPushButton(main);
  main->ejectL->setToolTip(tr("Open Project (Ctrl+O)"));
  main->ejectL->setIcon(QIcon(":/icons/eject.png"));
  main->ejectL->setIconSize(QSize(24, 24));
  connect(main->ejectL, SIGNAL(released()), main->aux, SLOT(openFile()));
  main->firstL = new QPushButton(main);
  main->firstL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->firstL->setToolTip(tr("First"));
  main->firstL->setIcon(QIcon(":/icons/first.png"));
  main->firstL->setIconSize(QSize(24, 24));
  connect(main->firstL, SIGNAL(released()),main->sessions, SLOT(first()));
  connect(main->firstL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->firstL, QString("0-6-99"));
  main->fwdL = new QPushButton(main);
  main->fwdL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->fwdL->setToolTip(tr("Forwards"));
  main->fwdL->setIcon(QIcon(":/icons/fwd.png"));
  main->fwdL->setIconSize(QSize(24, 24));
  connect(main->fwdL, SIGNAL(released()),main->sessions, SLOT(fwd()));
  connect(main->fwdL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->fwdL, QString("0-9-99"));
  main->lastL = new QPushButton(main);
  main->lastL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->lastL->setToolTip(tr("Last"));
  main->lastL->setIcon(QIcon(":/icons/last.png"));
  main->lastL->setIconSize(QSize(24, 24));
  connect(main->lastL, SIGNAL(released()),main->sessions, SLOT(last()));
  connect(main->lastL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->lastL, QString("0-10-99"));
  /*main->recL = new QPushButton(main);
  main->recL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->recL->setToolTip(tr("Record"));
  main->recL->setIcon(QIcon(":/icons/rec.png"));
  main->recL->setIconSize(QSize(24, 24));
  connect(main->recL, SIGNAL(released()),main->sessions, SLOT(rec()));
  connect(main->recL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->recL, QString("0-8-99"));*/
  main->rewL = new QPushButton(main);
  main->rewL->setContextMenuPolicy(Qt::CustomContextMenu);
  main->rewL->setToolTip(tr("Rew"));
  main->rewL->setIcon(QIcon(":/icons/rew.png"));
  main->rewL->setIconSize(QSize(24, 24));
  connect(main->rewL, SIGNAL(released()),main->sessions, SLOT(rew()));
  connect(main->rewL, SIGNAL(customContextMenuRequested(QPoint)), main->sessions->signalMapper, SLOT(map()));
  main->sessions->signalMapper->setMapping(main->rewL, QString("0-7-99"));
  
  main->jax.setVolume(100); 
  for (int a = 0; a < 13 /* Actual global functions */; a++) {
    main->globalmidifunctions[a].num = 255;
  }
}

void CyUi::createStatusBar()
{
    jackLabel = new QLabel(tr("Client Status"));
    jackLabel->setTextFormat(Qt::RichText);
    jackLabel->setAlignment(Qt::AlignHCenter);
    jackLabel->setFixedWidth(60);

    messageLabel = new QLabel(tr("Ready"));
    messageLabel->setIndent(5);

    main->durLabel = new QLabel("<b>00:00</b>");
    main->durLabel->setIndent(5);
    posWidget = new CyPosition(main);

    main->statusBar()->addWidget(jackLabel);
    main->statusBar()->addWidget(messageLabel, 1);
    main->statusBar()->addWidget(main->durLabel, 0);
    main->statusBar()->addWidget(posWidget);
    connect(&main->jax, SIGNAL(statusChanged(QString)),this, SLOT(updateStatusBar(QString)));
    connect(&main->jax, SIGNAL(posChanged(int,int,bool)),this, SLOT(updateStatusPos(int,int,bool)));
}

void CyUi::updateStatusPos(int curpos, int curasyncpos, bool async) {
  posWidget->setPosition(curpos, curasyncpos, async);
}

void CyUi::updateStatusSig(int cursig, int ch) {
  sigMeter->setSignal(cursig,ch);
}

void CyUi::updateStatusBar(QString msg)
{
  if (main->jax.state() == 0) {
    jackLabel->setText("<b><font color=green>" + tr("Jack OK") + "</font></b>");    
  } else {
    jackLabel->setText("<b><font color=red>" + tr("Jack KO") + "</font></b>");
  }
  if (msg == "Jack0") {
      messageLabel->setText(tr("Jack kicked all of us out!"));
      main->shutdownJ();
  }
  if (msg == "Jack1")
      messageLabel->setText(tr("Jack Client interrupted by user"));
  if (msg == "Jack2")
      messageLabel->setText(tr("Jack server not running?"));
  if (msg == "Jack3")
      messageLabel->setText(tr("Cannot activate Jack client"));
  if (msg == "Jack4")
      messageLabel->setText(tr("Cannot find any physical playback ports"));
  if (msg == "Jack5") {
      messageLabel->setText(tr("Buffer Size too low"));
      bufferSizeLow();
  }
  if (msg == "Jack6")
      messageLabel->setText(tr("Problem within async process"));
  if (msg == "Jack7")
      messageLabel->setText(tr("Jack client started"));
  if (msg == "Alsa0")
      messageLabel->setText(tr("Error opening ALSA sequencer"));
  if (msg == "Alsa1")
      messageLabel->setText(tr("Error creating sequencer input port"));
  if (msg == "Alsa2")
      messageLabel->setText(tr("Error creating sequencer output port"));
  if (msg == "Alsa3") {
      messageLabel->setText(tr("Cannot connect to Sooperlooper"));
      main->connectSooperlooper->setEnabled(true);
  }
  if (msg == "Alsa4") {
      messageLabel->setText(tr("Connected to Sooperlooper"));
      main->connectSooperlooper->setDisabled(true);
  }
  if (msg == "Alsa5") {
      messageLabel->setText(tr("Disconnected from Sooperlooper"));
      main->connectSooperlooper->setEnabled(true);
  }
  if (msg == "General0")
      messageLabel->setText(tr("Error: no name supplied for Jack/Alsa clients"));
}

void CyUi::createToolBar() {
  main->sessions->toolBar = main->addToolBar(tr("&Resources Tools"));
  main->sessions->toolBar->addAction(main->newProjectAction);
  main->sessions->toolBar->addAction(main->openProjectAction);
  main->sessions->toolBar->addAction(main->saveProjectAction);
  main->sessions->toolBar->addSeparator();
  main->sessions->toolBar->addAction(main->undoAction);
  main->sessions->toolBar->addAction(main->redoAction);
  main->sessions->toolBar->addSeparator();
  main->sessions->toolBar->addAction(main->addFilesAction);
  main->sessions->toolBar->addAction(main->removeLoopAction);
  main->sessions->toolBar->addAction(main->copyLoop);
  main->sessions->toolBar->addAction(main->pasteLoop);
}

void CyUi::bufferSizeLow() {
  main->stop(true);
  main->stopJ();
  QMessageBox::warning(main, tr("Cyclone - Buffer Size too Low!"), tr("In order to get Cyclone working on this computer,\nyou must increase Buffer Size from\n\n'Edit' -> 'Preferences' -> 'Buffer Size'\n\nPreferences window will now open."));
  main->preferences(true);
  main->startJ();
}
