/* CyUndo.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#include "CyUndo.h"
#include "WinCycle.h"
#include "CyTab.h"

CyUndo::CyUndo(WinCycle *parent) {
  glvol = QObject::tr("Global Volume Change");
  lvol = QObject::tr("Volume Change");
  slstr = QObject::tr("Loop Mode Change");
  sstr = QObject::tr("End Mode Change");
  mergez << glvol << lvol << slstr << sstr;
  main = parent;
  pushed = false;
  pushname = false;
}

void CyUndo::clean() {
  clear();
  setIndex(0);
  emit canUndoChanged(false);
}

void CyUndo::push(QUndoCommand *cmd, bool force) {
  if ((cmd->text() == tr("Move Session")) || (cmd->text() == tr("Remove Loop")) || (cmd->text() == tr("Remove Current Session")) || (cmd->text() == tr("Move Loop"))) {
    if (stop()) {
      return;
    } else {
      QUndoStack::push(cmd);
      start();
      return;
    }
  }
  bool catched = false;
  foreach (QString string, mergez) {
    if (text(index() - 1).contains(string)) {
      catched = true;
      break;
    }
  }
  if ((cmd->text() == text(index() - 1)) && catched && !force) {
    cmd->mergeWith(command(index() - 1));
    setIndex(index() - 1);
    QUndoStack::push(cmd);
  } else {
    QUndoStack::push(cmd);
  }
}

void CyUndo::undo() {
  int doit = 0;
  QString cmd = text(index() - 1);
  if ((cmd == tr("Move Session")) || (cmd == tr("Remove Loop")) || (cmd == tr("Remove Current Session")) || (cmd == tr("Move Loop")) || (cmd == tr("Add Loop")) || (cmd == tr("Paste Loop")) || (cmd == tr("Paste Session"))) {
    if (stop())
      doit = 1;
    else
      doit = 2;
  }
  if ((doit == 0) || (doit == 2))
    QUndoStack::undo();
  if (doit == 2)
    start();
}

void CyUndo::redo() {
  int doit = 0;
  QString cmd = text(index());
  if ((cmd == tr("Move Session")) || (cmd == tr("Remove Loop")) || (cmd == tr("Remove Current Session")) || (cmd == tr("Move Loop"))) {
    if (stop())
      doit = 1;
    else
      doit = 2;
  }
  if ((doit == 0) || (doit == 2))
    QUndoStack::redo();
  if (doit == 2)
    start();
}

void CyUndo::start() {
  main->start();  
}

bool CyUndo::stop() {
  if (main->jax.state() == 0) {
    if (main->aux->isPl()) {
      main->stop(false);
      return false;
    } else {
      return true;
    }
  }
  return false;
}

CommandAddSession::CommandAddSession(WinCycle *parent) {
  main = parent;
  setText(QObject::tr("Add Session"));
  sex = main->sessions->sessionsCount;
  ic = main->sessions->infinite_count;
}

void CommandAddSession::undo() {
  main->sessions->removeSession(sex);
}

void CommandAddSession::redo() {
  main->sessions->addSession(ic, sex);
}

CommandRemoveSession::CommandRemoveSession(WinCycle *parent, int ss) {
  main = parent;
  setText(QObject::tr("Remove Current Session"));
  sex = ss;
}

void CommandRemoveSession::undo() {
  QApplication::setOverrideCursor(Qt::WaitCursor);
  main->sessions->unsetSignalMappings(sex);
  main->sessions->insertTab(sex, mt, tableTitle);
  for (int a = main->sessions->sessionsCount - 1; a >= sex; a--) {
    main->sessions->currentSession = a + 1;
    main->sessions->musicTable[a + 1] = main->sessions->musicTable[a];
    main->sessions->setTabText(a + 1, main->sessions->musicTableTitle[a]);
    main->sessions->musicTableTitle[a + 1] = main->sessions->musicTableTitle[a];
    main->sessions->sel[a + 1] = main->sessions->sel[a];
    main->sessions->refreshMusicTable(a, false);
  }
  main->sessions->musicTable[sex] = mt;
  main->sessions->musicTableTitle[sex] = tableTitle;
  while (mt->rowCount()) {
    main->sessions->musicTable[sex]->removeRow(0);
  }
  main->sessions->setSession(sex);
  for (int di = 0; di < lcont; di++) {
    loopz[di]->setIndex(di + 1);
    main->sessions->loopz[di][sex] = new CyLoop(loopz[di]);
    main->sessions->addLoop(main->sessions->loopz[di][sex], sex);
  }
  main->sessions->sessionsCount++;
  main->sessions->unselect(sex);
  QApplication::restoreOverrideCursor();
}

void CommandRemoveSession::redo() {
  QApplication::setOverrideCursor(Qt::WaitCursor);
  lcont = 0;
  main->sessions->unsetSignalMappings(sex);
  tableTitle = main->sessions->musicTableTitle[sex];
  mt = main->sessions->musicTable[sex];
  for (int di = 0; di < main->sessions->musicTable[sex]->rowCount(); di++) {
    loopz[di] = new CyLoop(main->sessions->loopz[di][sex]);
    if (main->jax.state() == 0)
      main->sessions->loopz[di][sex]->destroyLoopBuffer();
    main->sessions->loopz[di][sex]->closeLoop();
    lcont++;
  }
  if (sex < (main->sessions->sessionsCount - 1)) {
    int cs = main->sessions->currentSession;
    for (int a = sex; a < (main->sessions->sessionsCount - 1); a++) {
      main->sessions->currentSession = a;
      main->sessions->musicTable[a] = main->sessions->musicTable[a + 1];
      main->sessions->musicTableTitle[a] = main->sessions->musicTableTitle[a + 1];
      main->sessions->refreshMusicTable(a + 1, false);
      main->sessions->sel[a] = main->sessions->sel[a + 1];
    }
    main->sessions->currentSession = cs;
  }
  main->sessions->removeSession(sex);
  QApplication::restoreOverrideCursor();
}

CommandMoveSession::CommandMoveSession(WinCycle *parent, int src, int dest) {
  main = parent;
  setText(QObject::tr("Move Session"));
  source = src;
  destination = dest;
}

void CommandMoveSession::undo() {
  main->sessions->unselect(destination);
  main->sessions->unselect(source);
  main->sessions->unsetSignalMappings(destination);
  main->sessions->unsetSignalMappings(source);
  QString dummyTitle;
  QTableWidget *dummyTable;
  dummyTitle = main->sessions->musicTableTitle[destination];
  dummyTable = main->sessions->musicTable[destination];
  main->sessions->musicTable[destination] = main->sessions->musicTable[source];
  main->sessions->musicTableTitle[destination] = main->sessions->musicTableTitle[source];
  main->sessions->musicTableTitle[source] = dummyTitle;
  main->sessions->musicTable[source] = dummyTable;
  if (destination < source) {
    main->sessions->moveTabRight(destination);
    main->sessions->refreshMusicTable(main->sessions->currentSession - 1, true); 
  } else {
    main->sessions->moveTabLeft(destination);
    main->sessions->refreshMusicTable(main->sessions->currentSession + 1, true); 
  } 
}

void CommandMoveSession::redo() {
  main->sessions->unselect(destination);
  main->sessions->unselect(source);
  main->sessions->unsetSignalMappings(destination);
  main->sessions->unsetSignalMappings(source);
  QString dummyTitle;
  QTableWidget *dummyTable;
  dummyTitle = main->sessions->musicTableTitle[source];
  dummyTable = main->sessions->musicTable[source];
  main->sessions->musicTable[source] = main->sessions->musicTable[destination];
  main->sessions->musicTableTitle[source] = main->sessions->musicTableTitle[destination];
  main->sessions->musicTableTitle[destination] = dummyTitle;
  main->sessions->musicTable[destination] = dummyTable;
  if (source < destination) {
    main->sessions->moveTabRight(source);
    main->sessions->refreshMusicTable(main->sessions->currentSession - 1, true); 
  } else {
    main->sessions->moveTabLeft(source);
    main->sessions->refreshMusicTable(main->sessions->currentSession + 1, true); 
  } 
}

CommandMoveLoop::CommandMoveLoop(WinCycle *parent, int src, int dest) {
  main = parent;
  sex = main->sessions->currentSession;
  setText(QObject::tr("Move Loop"));
  source = src;
  destination = dest;
}

void CommandMoveLoop::undo() {
  main->sessions->setSession(sex);
  main->sessions->moveLoop(destination, source);
}

void CommandMoveLoop::redo() {
  main->sessions->setSession(sex);
  main->sessions->moveLoop(source, destination);
}

CommandAddLoop::CommandAddLoop(WinCycle *parent, CyLoop *lp, int x, int y) {
  main = parent;
  setText(QObject::tr("Add Loop"));
  num = x;
  sex = y;
  loop = new CyLoop(lp);
}

void CommandAddLoop::undo() {
  main->sessions->setSession(sex);
  main->sessions->tableClicked(num, 6);
  main->sessions->removeLoop();
}

void CommandAddLoop::redo() {
  if (!main->undo->pushed) {
    main->sessions->setSession(sex);
    loop->setIndex(num + 1);
    main->sessions->loopz[num][sex] = new CyLoop(loop);
    main->sessions->addLoop(loop, sex);
  }
}

CommandRemoveLoop::CommandRemoveLoop(WinCycle *parent, CyLoop *lp, int x, int y) {
  main = parent;
  setText(QObject::tr("Remove Loop"));
  num = x;
  sex = y;
  loop = new CyLoop(lp);
}

void CommandRemoveLoop::undo() {
  main->sessions->setSession(sex);
  main->sessions->insLoop(loop, num, sex);
}

void CommandRemoveLoop::redo() {
  main->sessions->setSession(sex);
  main->sessions->unselect(sex);
  main->sessions->rmLoop(num, sex);
}

CommandPasteLoop::CommandPasteLoop(WinCycle *parent) {
  main = parent; 
  setText(QObject::tr("Paste Loop"));
  y = main->sessions->currentSession;
  x = main->sessions->loopCount(y);
  loop = new CyLoop(main->aux->pasteCyLoop);
}

void CommandPasteLoop::undo() {
  main->sessions->setSession(y);
  main->sessions->rmLoop(x, y);
  main->cymidi->refresh(y);
}

void CommandPasteLoop::redo() {
  main->sessions->setSession(y);
  loop->setIndex(x + 1);
  main->sessions->loopz[x][y] = loop;
  if (main->sessions->addLoop(main->sessions->loopz[x][y], y)) {
    main->sessions->tableClicked(x, 6);
    main->aux->copyLoop();
  }
  main->cymidi->refresh(y);
}

CommandPasteSession::CommandPasteSession(WinCycle *parent) {
  main = parent;
  setText(QObject::tr("Paste Session"));
  sex = main->sessions->sessionsCount;
  name = main->aux->pctName;
  count = main->aux->pctCount;
  for (int it = 0; it < count; it++) {
    main->aux->pasteCyTable[it]->setIndex(it + 1);
    loopz[it] = new CyLoop(main->aux->pasteCyTable[it]);
  }
}

void CommandPasteSession::undo() {
  main->sessions->unsetSignalMappings(sex);
  lcont = 0;
  for (int di = 0; di < count; di++) {
    loopz[di]->setIndex(di + 1);
    main->sessions->loopz[di][sex]->closeLoop();
  }
  main->sessions->removeTab(sex);
  main->sessions->sessionsCount--;
  main->mix.reset(sex);
}

void CommandPasteSession::redo() {
    QApplication::setOverrideCursor(Qt::WaitCursor);
    main->sessions->addSession(0, sex, name);
    main->sessions->setSession(sex);
    for (int it = 0; it < count; it++) {
      if (main->aux->standardDebug)
	  main->printOutput(QString("Pasting Loop: %1") .arg(loopz[it]->passName()));
      main->sessions->loopz[it][sex] = loopz[it];
      main->sessions->addLoop(main->sessions->loopz[it][sex], sex);
    }
    main->sessions->unsetSignalMappings(sex);
    main->sessions->refreshMusicTable(sex, false);
    main->cymidi->refresh(sex);
    main->sessions->sessionsCount++;
    main->aux->copySession();
    QApplication::restoreOverrideCursor();
}

CommandGlobalVolume::CommandGlobalVolume(WinCycle *parent, int oldvol, int newvol) {
  main = parent;
  setText(main->undo->glvol);
  ovol = oldvol;
  nvol = newvol;
}

void CommandGlobalVolume::undo() {
  main->gvpress = true;
  main->volume->setValue(ovol);
  main->gvpress = false;
  main->undo->oldvol = ovol;
}

void CommandGlobalVolume::redo() {
  main->gvpress = true;
  main->volume->setValue(nvol);
  main->gvpress = false;
  main->undo->oldvol = nvol;
}

bool CommandGlobalVolume::mergeWith(const QUndoCommand *other) {
  if (other->id() != id()) // make sure other is also an AppendText command
      return false;
  const CommandGlobalVolume *cgv = static_cast<const CommandGlobalVolume *>(other);
  ovol = cgv->ovol;
  return true;
}

CommandLoopVolume::CommandLoopVolume(WinCycle *parent, int row, int sex, int newvol) {
  main = parent;
  x = row;
  y = sex;
  setText(QObject::tr("Loop %1 Session %2 %3") .arg(x + 1) .arg(y + 1) .arg(main->undo->lvol));
  ovol = main->undo->oldloopvol[x][y];
  nvol = newvol;
  main->undo->oldloopvol[x][y] = newvol;
}

void CommandLoopVolume::undo() {
  main->sessions->setSession(y);
  QSlider *sld = (QSlider *)main->sessions->musicTable[y]->cellWidget(x, 4);
  main->undo->oldloopvolpressed[x][y] = true;
  sld->setValue(ovol);
  main->undo->oldloopvolpressed[x][y] = false;
  main->undo->oldloopvol[x][y] = ovol;
}

void CommandLoopVolume::redo() {
  main->sessions->setSession(y);
  QSlider *sld = (QSlider *)main->sessions->musicTable[y]->cellWidget(x, 4);
  main->undo->oldloopvolpressed[x][y] = true;
  sld->setValue(nvol);
  main->undo->oldloopvolpressed[x][y] = false;
  main->undo->oldloopvol[x][y] = nvol;
}

bool CommandLoopVolume::mergeWith(const QUndoCommand *other) {
  if (other->id() != id()) // make sure other is also an AppendText command
      return false;
  const CommandLoopVolume *clv = static_cast<const CommandLoopVolume *>(other);
  ovol = clv->ovol;
  return true;
}

CommandLoopName::CommandLoopName(WinCycle *parent, int row, int sex, QString name) {
  main = parent;
  x = row;
  y = sex;
  setText(QObject::tr("Loop %1 Session %2 Name Change") .arg(x + 1) .arg(y + 1));
  oldname = main->sessions->loopz[x][y]->getName();
  newname = name;
}

void CommandLoopName::undo() {
  main->sessions->setSession(y);
  main->sessions->musicTable[y]->item(x, 1)->setText(oldname);
}

void CommandLoopName::redo() {
  main->sessions->setSession(y);
  main->sessions->musicTable[y]->item(x, 1)->setText(newname);
}

CommandSessionName::CommandSessionName(WinCycle *parent, int sex, QString name) {
  main = parent;
  s = sex;
  setText(QObject::tr("Session %1 Name Change") .arg(s + 1));
  oldname = main->sessions->tabText(s);
  newname = name;
}

void CommandSessionName::undo() {
  main->sessions->setTabText(s, oldname);
  main->sessions->musicTableTitle[s] = oldname;
}

void CommandSessionName::redo() {
  main->sessions->setTabText(s, newname);
  main->sessions->musicTableTitle[s] = newname;
}

CommandMidiAssign::CommandMidiAssign(WinCycle *parent, int row, int sex, midicmd mcmd, int type) {
  main = parent;
  x = row;
  y = sex;
  t = type;
  setText(QObject::tr("Midi Assign Loop %1 Session %2") .arg(x + 1) .arg(y + 1));
  switch (t) {
    case 0: /* table play */
      if (mcmd.type == 2)
	mpf.fn = PC;
      else
	mpf.fn = NT;
      mpf.ch = mcmd.ch;
      mpf.val = mcmd.num;
      oldmpf = main->sessions->loopz[x][y]->getMidi();
      break;
    case 1: /* table volume  */
      mvf.ch = mcmd.ch;
      mvf.val = mcmd.num;
      oldmvf = main->sessions->loopz[x][y]->getMidiV();
      break;
  }
}

void CommandMidiAssign::undo() {
  switch (t) {
    case 0: /* table play */
      main->sessions->loopz[x][y]->setMidi(oldmpf);
      break;
    case 1: /* table volume  */
      main->sessions->loopz[x][y]->setMidiV(oldmvf);
      break;
  }
  main->cymidi->refresh(y);
}

void CommandMidiAssign::redo() {
  switch (t) {
    case 0: /* table play */
      main->sessions->loopz[x][y]->setMidi(mpf);
      break;
    case 1: /* table volume  */
      main->sessions->loopz[x][y]->setMidiV(mvf);
      break;
  }
  main->cymidi->refresh(y);
}

CommandGlobalMidiAssign::CommandGlobalMidiAssign(WinCycle *parent, midicmd mcmd, int type, int funct) {
  main = parent;
  t = abs(type - 3);
  fn = funct;
  setText(QObject::tr("Global Midi Assign"));
  cmd = mcmd;
  oldcmd = main->globalmidifunctions[fn + t];
}

void CommandGlobalMidiAssign::undo() {
  main->globalmidifunctions[fn + t] = oldcmd;
  main->cymidi->refreshGlobal();
}

void CommandGlobalMidiAssign::redo() {
  main->globalmidifunctions[fn + t] = cmd;
  main->cymidi->refreshGlobal();
}

CommandSLcombo::CommandSLcombo(WinCycle *parent, int cindex, int row, int sex) {
  main = parent;
  x = row;
  y = sex;
  oldi = main->undo->oldsl[x][y];
  i = cindex;
  setText(QObject::tr("%3 Loop %1 Session %2") .arg(x + 1) .arg(y + 1) .arg(main->undo->slstr));
}

void CommandSLcombo::undo() {
  main->sessions->setSession(y);
  main->sessions->movingLoop = true;
  main->sessions->loopz[x][y]->setSLmode(main->aux->indexToSlmode(oldi));
  ((QComboBox *)main->sessions->musicTable[y]->cellWidget(x, 2))->setCurrentIndex(oldi);
  main->undo->oldsl[x][y] = oldi;
  main->sessions->movingLoop = false;
}

void CommandSLcombo::redo() {
  main->sessions->setSession(y);
  main->sessions->movingLoop = true;
  main->sessions->loopz[x][y]->setSLmode(main->aux->indexToSlmode(i));
  ((QComboBox *)main->sessions->musicTable[y]->cellWidget(x, 2))->setCurrentIndex(i);
  main->undo->oldsl[x][y] = i;
  main->sessions->movingLoop = false;
}

bool CommandSLcombo::mergeWith(const QUndoCommand *other) {
  if (other->id() != id()) // make sure other is also an AppendText command
      return false;
  const CommandSLcombo *cslc = static_cast<const CommandSLcombo *>(other);
  oldi = cslc->oldi;
  return true;
}

CommandScombo::CommandScombo(WinCycle *parent, int cindex, int row, int sex) {
  main = parent;
  x = row;
  y = sex;
  oldi = main->undo->olds[x][y];
  i = cindex;
  setText(QObject::tr("%3 Loop %1 Session %2") .arg(x + 1) .arg(y + 1) .arg(main->undo->sstr));
}

void CommandScombo::undo() {
  main->sessions->setSession(y);
  main->sessions->movingLoop = true;
  main->sessions->loopz[x][y]->setSmode(main->aux->indexToSmode(oldi));
  ((QComboBox *)main->sessions->musicTable[y]->cellWidget(x, 3))->setCurrentIndex(oldi);
  main->undo->olds[x][y] = oldi;
  main->sessions->movingLoop = false;
}

void CommandScombo::redo() {
  main->sessions->setSession(y);
  main->sessions->movingLoop = true;
  main->sessions->loopz[x][y]->setSmode(main->aux->indexToSmode(i));
  ((QComboBox *)main->sessions->musicTable[y]->cellWidget(x, 3))->setCurrentIndex(i);
  main->undo->olds[x][y] = i;
  main->sessions->movingLoop = false;
}

bool CommandScombo::mergeWith(const QUndoCommand *other) {
  if (other->id() != id()) // make sure other is also an AppendText command
      return false;
  const CommandScombo *csc = static_cast<const CommandScombo *>(other);
  oldi = csc->oldi;
  return true;
}
