/* CyUndo.h
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#ifndef CYUNDO_H
#define CYUNDO_H

#include <QObject>
#include <QString>
#include <QUndoStack>
#include <QUndoCommand>

#include "WinCycle.h"
#include "typedefs.h"
#include "CyLoop.h"

class CyUndo : public QUndoStack {
    Q_OBJECT
    
public:
  CyUndo(WinCycle *parent);
  
  void clean();
  bool stop();
  void push(QUndoCommand *cmd, bool force = false);
  
  int oldvol;
  int oldsl[128][128];
  int olds[128][128];
  int oldloopvol[128][128];
  bool oldloopvolpressed[128][128];
  bool pushed;
  bool pushname;
  /* Commands to be merged */
  QStringList mergez;
  // BEGIN
  QString glvol;
  QString lvol;
  QString slstr;
  QString sstr;
  // END
  
  public slots:
    void undo();
    void redo();
    
  private:
    void start();
    WinCycle *main;
  
};

class DummyWidget : public QObject {
    Q_OBJECT
    
public:
  DummyWidget() { };
  
  signals:
    void signalx();
    
   public slots:
    void signalz() { emit signalx(); };
  
};

class CommandAddLoop : public QUndoCommand {
  
  public:
    CommandAddLoop(WinCycle *parent, CyLoop *lp, int x, int y);
    
    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    int num, sex;
    CyLoop *loop;
    
};

class CommandRemoveLoop : public QUndoCommand {

  public:
    CommandRemoveLoop(WinCycle *parent, CyLoop *lp, int x, int y);
    virtual void undo();
    virtual void redo();
    
  private:
    WinCycle *main;
    int num, sex;
    CyLoop *loop;
  
};

class CommandAddSession : public QUndoCommand {

  public:
    CommandAddSession(WinCycle *parent);

    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    int sex;
    int ic;
};

class CommandRemoveSession : public QUndoCommand {

  public:
    CommandRemoveSession(WinCycle *parent, int ss);

    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    int sex;
    CyLoop *loopz[128];
    int lcont;
    QString tableTitle;
    QTableWidget *mt;
  
};

class CommandMoveLoop : public QUndoCommand {

  public:
    CommandMoveLoop(WinCycle *parent, int src, int dest);
    
    virtual void undo();
    virtual void redo();
    
  private:
    WinCycle *main;
    
    int source, destination, sex;
  
};

class CommandMoveSession : public QUndoCommand {

  public:
    CommandMoveSession(WinCycle *parent, int src, int dest);

    virtual void undo();
    virtual void redo();
    
  private:
    WinCycle *main;
    int source, destination;
  
};

class CommandPasteLoop : public QUndoCommand {

  public:
    CommandPasteLoop(WinCycle *parent);
    
    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    CyLoop *loop;    
    int x, y;
  
};

class CommandPasteSession : public QUndoCommand {

  public:
    CommandPasteSession(WinCycle *parent);
    
    virtual void undo();
    virtual void redo();
    
  private:
    WinCycle *main;
    int sex;
    CyLoop *loopz[128];
    int lcont;
    QString tableTitle;
    QTableWidget *mt;
    QString name;
    int count;
};

class CommandGlobalVolume : public QUndoCommand {

  public:
    CommandGlobalVolume(WinCycle *parent, int oldvol, int newvol);
    
    virtual void undo();
    virtual void redo();
    virtual bool mergeWith(const QUndoCommand *other);
    
    int ovol, nvol;
    
  private:
    WinCycle *main;
  
};

class CommandLoopVolume : public QUndoCommand {

  public:
    CommandLoopVolume(WinCycle *parent, int row, int sex, int newvol);

    virtual void undo();
    virtual void redo();
    virtual bool mergeWith(const QUndoCommand *other);
    
    int ovol, nvol;
    
  private:
    WinCycle *main;
    int x, y;
  
};

class CommandLoopName : public QUndoCommand {

  public:
    CommandLoopName(WinCycle *parent, int row, int sex, QString name);
    
    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    int x, y;
    QString newname;
    QString oldname;
  
};

class CommandSessionName : public QUndoCommand {

  public:
    CommandSessionName(WinCycle *parent, int sex, QString name);
    
    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    int s;
    QString newname;
    QString oldname;
  
};

class CommandSLcombo : public QUndoCommand {

  public:
    CommandSLcombo(WinCycle *parent, int cindex, int row, int sex);
    
    virtual void undo();
    virtual void redo();
    virtual bool mergeWith(const QUndoCommand *other);
    
    int oldi;
    
  private:
    WinCycle *main;
    int x, y, i;
  
};


class CommandScombo : public QUndoCommand {

  public:
    CommandScombo(WinCycle *parent,  int cindex, int row, int sex);

    virtual void undo();
    virtual void redo();
    virtual bool mergeWith(const QUndoCommand *other);
    
    int oldi;
    
    
  private:
    WinCycle *main;
    int x, y, i;
  
};

class CommandMidiAssign : public QUndoCommand {

  public:
    CommandMidiAssign(WinCycle *parent, int row, int sex, midicmd mcmd, int type);
    
    virtual void undo();
    virtual void redo();

  private:
    WinCycle *main;
    int x, y, t;
    midiPlayFunction mpf, oldmpf;
    midiVolumeFunction mvf, oldmvf;
  
};

class CommandGlobalMidiAssign : public QUndoCommand {

  public:
    CommandGlobalMidiAssign(WinCycle *parent, midicmd mcmd, int type, int funct);
    
    virtual void undo();
    virtual void redo();
    
  private:
    WinCycle *main;
    int t, fn;
    midicmd cmd, oldcmd;
  
};

#endif
 
