/* Dialogs.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#include "typedefs.h"
#include "Dialogs.h"
#include "WinCycle.h"

RenameSession::RenameSession(QWidget *parent) : QDialog(parent) {
    setModal(true);
    QLabel *label = new QLabel(tr("Enter the new name for current session:"));
    buttonBox = new QDialogButtonBox(this);
    lineEdit = new QLineEdit;
    buttonBox->addButton(QDialogButtonBox::Ok);
    buttonBox->addButton(QDialogButtonBox::Cancel);
    buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    QRegExp regExp("[A-Za-z0-9 _]{1,13}");
    lineEdit->setValidator(new QRegExpValidator(regExp, this));
    
    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
    connect(lineEdit, SIGNAL(textChanged(QString)), this, SLOT(enableOkButton(QString)));
    
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(label);
    layout->addWidget(lineEdit);
    layout->addWidget(buttonBox);  
    setLayout(layout);
    setWindowTitle(tr("Rename Current Session"));
    setFixedSize(sizeHint());
    lineEdit->setFocus();
    show();
}

void RenameSession::enableOkButton(QString nm) {
    buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!nm.isEmpty());
}


Meter::Meter(QWidget *parent) : QWidget(parent) {
    setFixedSize(300,22);
    peakleft = 0;
    peakright = 0;
    sigleft = 0;
    sigright = 0;
    clippedl = 0;
    clippedr = 0;
    
    setBackgroundRole(QPalette::NoRole);
    setAutoFillBackground(false);
}


void Meter::mousePressEvent(QMouseEvent *event) {
    QPointF point = event->pos();
    if ((point.x() > 270) && (point.x() < 299)) {
      clippedl = 0;
      clippedr = 0;  
      resetPeaks();
      startPeaks();
      update();
    }
}


void Meter::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    draw(&painter);
}

void Meter::draw(QPainter *painter)
{
    QPen thinPen(palette().foreground(), 0.5, Qt::SolidLine);
    QBrush gp(QColor(Qt::darkGreen).darker(250));
    QBrush ga(Qt::green);
    QBrush yp(QColor(Qt::darkYellow).darker(250));
    QBrush ya(Qt::yellow);
    QBrush rp(QColor(Qt::darkRed).darker(250));
    QBrush ra(Qt::red);
    QColor peakc(Qt::white);
    peakc.setAlpha(80);
    QBrush pk(peakc);

    painter->setPen(thinPen);
    painter->drawRect(0, 0, 258, 10);
    painter->drawRect(0, 12, 258, 10);
    painter->drawRect(270, 0, 28, 10);
    painter->drawRect(270, 12, 28, 10);
    
    painter->setBrush(gp);
    painter->drawRect(1, 1, 150, 10);
    painter->drawRect(1, 13, 150, 10);
    painter->setBrush(yp);
    painter->drawRect(151, 1, 71, 10);
    painter->drawRect(151, 13, 71, 10);
    painter->setBrush(rp);
    painter->drawRect(222, 1, 35, 10);
    painter->drawRect(222, 13, 35, 10);
    
    /* draw signal */
    if (sigleft) {  
      painter->setBrush(ga);
      if (sigleft > 150) {
	painter->drawRect(1, 1, 150, 10);
      } else {
	painter->drawRect(1, 1, sigleft, 10);
      }
      
      painter->setBrush(ya);
      if (sigleft > 150) {
	if (sigleft > 221) {
	  painter->drawRect(151, 1, 71, 10);
	} else {
	  painter->drawRect(151, 1, sigleft - 150, 10);
	}
      }
      
      painter->setBrush(ra);
      if (sigleft > 221) {
	if (sigleft > 255) {
	  painter->drawRect(222, 1, 35, 10);
	  clippedl = 1;
	} else {
	  painter->drawRect(222, 1, sigleft - 221, 10);
	}
      }
    }
    
    if (sigleft) {  
      painter->setBrush(ga);
      if (sigright > 150) {
	painter->drawRect(1, 13, 150, 10);
      } else {
	painter->drawRect(1, 13, sigright, 10);
      }
      
      painter->setBrush(ya);
      if (sigright > 150) {
	if (sigright > 221) {
	  painter->drawRect(151, 13, 71, 10);
	} else {
	  painter->drawRect(151, 13, sigright - 150, 10);
	}
      }
      
      QBrush pk(peakc);
      painter->setBrush(ra);
      if (sigright > 221) {
	if (sigright > 256) {
	  painter->drawRect(222, 13, 35, 10);
	  clippedr = 1;
	} else {
	  painter->drawRect(222, 13, sigright - 221, 10);
	}
      }
    
    }
    
    painter->setBrush(ra);
    if (clippedl)
      painter->drawRect(271, 1, 26, 8);
    if (clippedr)
      painter->drawRect(271, 13, 26, 8);
    
    /* draw peaks */
    if ((sigleft > peakleft) && (peakleft != -1))
      peakleft = sigleft;
    if ((sigright > peakright) && (peakright != -1))
      peakright = sigright;
    
    painter->setBrush(pk);
    
    if ((peakleft > 0) && (peakleft < 256)) {
      painter->drawRect(1 + peakleft, 1, 3, 8);
    }
    
    if ((peakright > 0) && (peakright < 256)) {
      painter->drawRect(1 + peakright, 13, 3, 8);
    }
    
}

void Meter::setSignal(int sig, int ch) {
  switch (ch) {
    case 0:
      sigleft = sig;
      break;
    case 1:
      sigright = sig;
      break;
  }
  update();
}

void Meter::resetPeaks() {
  peakleft = -1;
  peakright = -1;
  update();
}

void Meter::startPeaks() {
  peakleft = 0;
  peakright = 0;
  update();
}

CyPosition::CyPosition(QWidget *parent) : QWidget(parent) {
    setFixedSize(258,20);
    position = 0;    
    setBackgroundRole(QPalette::NoRole);
    setAutoFillBackground(false);
    asyncEnabled = false;
}

void CyPosition::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    draw(&painter);
}

void CyPosition::draw(QPainter *painter)
{
    QPen thinPen(palette().foreground(), 0.5, Qt::SolidLine);
    QBrush pt(Qt::white);
    QBrush bp(QColor(Qt::darkBlue).darker(300));
    QBrush ba(QColor(Qt::blue).lighter());
    QBrush ap(QColor(Qt::darkYellow).darker(300));
    QBrush aa(QColor(Qt::yellow).lighter());

    painter->setPen(thinPen);
    painter->drawRect(0, 0, 258, 20);
    painter->setBrush(bp);
    painter->drawRect(1, 1, 258, 18);
    painter->setBrush(ba);
    painter->drawRect(1, 1, (position - 3), 18);  
    
    /* draw pointer */    
    painter->setBrush(pt);
    painter->drawRect(1 + (position - 3), 1, 3, 18);
    
    /* async loop */
    if (asyncEnabled) {
      painter->setBrush(ap);
      painter->drawRect(1, 11, 258, 8);
      painter->setBrush(aa);
      painter->drawRect(1, 11, (asyncposition - 3), 8);  
      
      /* draw pointer */    
      painter->setBrush(pt);
      painter->drawRect(1 + (asyncposition - 3), 11, 3, 8);      
    }
       
}

void CyPosition::setPosition(int pos, int asyncpos, bool async) {
  asyncposition = asyncpos;
  asyncEnabled = async;
  position = pos;
  update();
}

MidiLearn::MidiLearn(QWidget *parent) : QDialog(parent) {
  main = parent;
  zero = false;
  setModal(true);
  setFixedSize(QSize(400,200));
  QLabel *label = new QLabel(tr("Please send a MIDI signal to Cyclone or click Cancel"));
  label1 = new QLabel("");
  label2 = new QLabel("");
  label3 = new QLabel("");
  label4 = new QLabel("<br><br>");
  label1->setTextFormat(Qt::RichText);
  label2->setTextFormat(Qt::RichText);
  label3->setTextFormat(Qt::RichText);
  label4->setTextFormat(Qt::RichText);
  label1->setText("<center><b>" + tr("Status:") + " <i>" + tr("Waiting...") + "</i></b></center>");
  label2->setText(tr("Midi Command:"));
  reset = new QPushButton(tr("Reset"));
  buttonBox = new QDialogButtonBox(this);
  buttonBox->addButton(QDialogButtonBox::Ok);
  buttonBox->addButton(QDialogButtonBox::Cancel);
  buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
  
  connect(reset, SIGNAL(clicked()), this, SLOT(resetted()));
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
  
  QVBoxLayout *layout = new QVBoxLayout;
  QHBoxLayout *hlayout = new QHBoxLayout;
  layout->addWidget(label);
  layout->addWidget(label3);
  layout->addWidget(label1);
  layout->addWidget(label2);
  layout->addWidget(label4);
  hlayout->addWidget(reset);
  hlayout->addStretch();
  hlayout->addWidget(buttonBox); 
  layout->addLayout(hlayout);
  setLayout(layout);
  setWindowTitle(tr("Midi Learn"));
  setWindowIcon(QIcon(":/icons/midi.png"));
  show();
}

void MidiLearn::resetted() {
  zero = true;
  emit accept();
  close();
}

void MidiLearn::enableOkButton(bool enable) {
    buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enable);
}

void MidiLearn::currentSettings(QString cs) {
  label3->setText(cs);
}

void MidiLearn::midiSlot(int type, int num, int val, int ch, bool free) {
  QString command;
  switch (type) {
    case 0:
      command = QString("Note %1</b>, midi ch <b>%2") .arg(num) .arg(ch + 1);
      break;
    case 1:
      command =  QString("Control Change nr. %1</b>, midi ch <b>%2") .arg(num) .arg(ch + 1);
      break;
    case 2:
      command = QString("Program Change %1</b>, midi ch <b>%2") .arg(num) .arg(ch + 1);
      break;
    case 3:
      command = QString("Control Change 14 nr. %1</b>, midi ch <b>%2") .arg(num) .arg(ch + 1);
      break;
    case 4:
      command = QString("Non Registered Parameter %1</b>, midi ch <b>%2") .arg(num) .arg(ch + 1);
      break;
    case 5:
      command = QString("Registered Parameter %1</b>, midi ch <b>%2") .arg(num) .arg(ch + 1);
      break;
  }
  enableOkButton(free);
  if (buttonBox->button(QDialogButtonBox::Ok)->isEnabled()) {
    label1->setText("<center><b>" + tr("Status:") + " <i><font color=green>" + tr("Free :-)") + "</font></i></b></center>");
    label2->setText(tr("Midi Command:") + " <b>" + command + "</b>");
    label4->setText("<br><br>");
  } else {
    label1->setText("<center><b>" + tr("Status:") + " <i><font color=red>" + tr("in Use :-(") + "</font></i></b></center>");
    label2->setText(tr("Midi Command:") + " <b>" + command + "</b>");
    label4->setText(((WinCycle *)main)->cymidi->inUseString());
  }
}

Preferences::Preferences(AuxCycle *aux, QWidget *parent) : QDialog(parent) {
    setModal(true);
    quickFade = new QLabel(tr("&Quick Fade Time"));
    buffer = new QLabel(tr("&Buffer Size"));
    reloadSession = new QLabel(tr("&Reload last project at startup"));
    standardSL = new QLabel(tr("&Set standard SooperLooper mode"));
    standardS = new QLabel(tr("Set standard &End mode"));
    standardV = new QLabel(tr("Set standard &Volume"));
    sooLooVol = new QLabel(tr("Set standard &Sooperlooper Volume"));
    debug = new QLabel(tr("&Debug"));
    graphicSystem = new QLabel("");
    graphicSystem->setTextFormat(Qt::RichText);
    graphicSystem->setText(tr("&OpenGL Graphics System - <small><u><b>Requires App Restart</b></u><br>Use graphics card acceleration<br>may not work with your graphics card.</small>"));
    graphicFx = new QLabel("");
    graphicFx->setText(tr("&Graphics Effects"));
    quickFadeVal = new QLabel(tr("msec"));
    quickFadeVal->setAlignment(Qt::AlignHCenter);
    standardVVal = new QLabel("");;
    standardVVal->setAlignment(Qt::AlignHCenter);
    standardsooLooVol = new QLabel("");;
    standardsooLooVol->setAlignment(Qt::AlignHCenter);
    qmsec =  new QSlider(Qt::Horizontal, this);
    qmsec->setRange(10,500);
    connect(qmsec, SIGNAL(valueChanged(int)), this, SLOT(qmsecChanged(int)));
    sv =  new QSlider(Qt::Horizontal, this);
    sv->setRange(0,127);
    connect(sv, SIGNAL(valueChanged(int)), this, SLOT(svChanged(int)));
    slv =  new QSlider(Qt::Horizontal, this);
    slv->setRange(0,127);
    connect(slv, SIGNAL(valueChanged(int)), this, SLOT(slvChanged(int)));
    buf = new QComboBox(this);
    buf->addItem(tr("128 - fast computer, SSD (SolidState Disk)"));
    buf->addItem(tr("256 - fast computer, fast HD"));
    buf->addItem(tr("512 - normal computer, normal HD"));
    buf->addItem(tr("1024 - normal computer, slow HD"));
    buf->addItem(tr("2048 - old computer, slow HD"));
    ss = new QComboBox(this);
    ss->addItem(tr("Instantaneous")); /*@@@ Stop modes @@@*/
    ss->addItem(tr("EndLoop")); /*@@@ Stop modes @@@*/
    ss->addItem(tr("Once")); /*@@@ Stop modes @@@*/
    ss->addItem(tr("Asynchronous")); /*@@@ Stop modes @@@*/
    ssl = new QComboBox(this);
    ssl->addItem(tr("None")); /*@@@ Sooperlooper modes @@@*/
    ssl->addItem(tr("Multiply")); /*@@@ Sooperlooper modes @@@*/
    ssl->addItem(tr("Record")); /*@@@ Sooperlooper modes @@@*/
    ssl->addItem(tr("RecOverdub")); /*@@@ Sooperlooper modes @@@*/
    ssl->addItem(tr("MultiOverdub")); /*@@@ Sooperlooper modes @@@*/
    rels = new QCheckBox(this);
    dbg = new QCheckBox(this);
    grsys = new QCheckBox(this);
    grfx = new QCheckBox(this);
    buttonBox = new QDialogButtonBox(this);
    buttonBox->addButton(QDialogButtonBox::Ok);
    buttonBox->addButton(QDialogButtonBox::Cancel);
    
    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
    
    QHBoxLayout *debu = new QHBoxLayout;
    debu->setAlignment(Qt::AlignLeft);
    debu->addWidget(dbg);
    debug->setBuddy(dbg);
    debu->addWidget(debug);
    QHBoxLayout *relo = new QHBoxLayout;
    relo->setAlignment(Qt::AlignLeft);
    relo->addWidget(rels);
    reloadSession->setBuddy(rels);
    relo->addWidget(reloadSession);
    QHBoxLayout *grs = new QHBoxLayout;
    grs->setAlignment(Qt::AlignLeft);
    grs->addWidget(grsys);
    graphicSystem->setBuddy(grsys);
    grs->addWidget(graphicSystem);
    QHBoxLayout *grf = new QHBoxLayout;
    grf->setAlignment(Qt::AlignLeft);
    grf->addWidget(grfx);
    graphicFx->setBuddy(grfx);
    grf->addWidget(graphicFx);
    QHBoxLayout *bufr = new QHBoxLayout;
    bufr->addWidget(buffer);
    buffer->setBuddy(buf);
    bufr->addWidget(buf);
    QHBoxLayout *stsl = new QHBoxLayout;
    stsl->addWidget(standardSL);
    standardSL->setBuddy(ssl);
    stsl->addWidget(ssl);
    QHBoxLayout *stem = new QHBoxLayout;
    stem->addWidget(standardS);
    standardS->setBuddy(ss);
    stem->addWidget(ss);
    QVBoxLayout *stvo = new QVBoxLayout;
    stvo->setSpacing(5);
    stvo->setContentsMargins(10,20,10,5);
    stvo->addWidget(standardV);
    standardV->setBuddy(sv);
    stvo->addWidget(sv);
    stvo->addWidget(standardVVal);
    QVBoxLayout *stsvo = new QVBoxLayout;
    stsvo->setSpacing(5);
    stsvo->setContentsMargins(10,20,10,5);
    stsvo->addWidget(sooLooVol);
    sooLooVol->setBuddy(slv);
    stsvo->addWidget(slv);
    stsvo->addWidget(standardsooLooVol);
    QVBoxLayout *qufa = new QVBoxLayout;
    qufa->setSpacing(5);
    qufa->setContentsMargins(10,5,10,40);
    qufa->addWidget(quickFade);
    quickFade->setBuddy(qmsec);
    qufa->addWidget(qmsec);
    qufa->addWidget(quickFadeVal);
    
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setAlignment(Qt::AlignLeft);
    layout->addLayout(debu);
    layout->addLayout(relo);
    layout->addLayout(grs);
    layout->addLayout(grf);
    layout->addLayout(bufr);  
    layout->addLayout(stsl);
    layout->addLayout(stem);
    layout->addLayout(stvo);
    layout->addLayout(stsvo);
    layout->addLayout(qufa);
    layout->addWidget(buttonBox); 
    setLayout(layout);
    setWindowTitle(tr("Preferences"));
    setFixedSize(370, sizeHint().height());
    
    switch (aux->standardBuffer) {
      case 128:
	buf->setCurrentIndex(0);
	break;
      case 256:
	buf->setCurrentIndex(1);
	break;
      case 512:
	buf->setCurrentIndex(2);
	break;
      case 1024:
	buf->setCurrentIndex(3);
	break;
      case 2048:
	buf->setCurrentIndex(4);
	break;
    }
    if (aux->grsys == "opengl")
      grsys->setChecked(true);
    else
      grsys->setChecked(false);
    if (aux->graphicsfxnext < 5)
      grfx->setChecked((bool)aux->graphicsfxnext);
    else
      grfx->setChecked((bool)aux->graphicsfx);
    rels->setChecked((bool)aux->restoreLastSession);
    dbg->setChecked((bool)aux->standardDebug);
    ssl->setCurrentIndex(aux->slmodeToIndex(aux->standardSL));
    ss->setCurrentIndex(aux->smodeToIndex(aux->standardS));
    qmsec->setValue(aux->standardQuickfade);
    sv->setValue(aux->standardV);
    slv->setValue(aux->sooloovol);
    show();
}

void Preferences::qmsecChanged(int v) {
  QString ddd = QString("%1msec") .arg(v);
  quickFadeVal->setText(ddd);
}

void Preferences::svChanged(int v) {
  standardVVal->setText(QString("%1") .arg(v));
}

void Preferences::slvChanged(int v) {
  standardsooLooVol->setText(QString("%1") .arg(v));
}
