/* Dialogs.h
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#ifndef DIALOGS_H
#define DIALOGS_H

#include <QDialog>
#include <QWidget>
#include <QtGui>
#include "AuxCycle.h"

class QPaintEvent;
class QMouseEvent;

class RenameSession : public QDialog {
    Q_OBJECT
    
public:
   RenameSession(QWidget *parent = 0);
   
   QLineEdit *lineEdit;
    
private:
   QDialogButtonBox *buttonBox;

private slots:
    void enableOkButton(QString nm);
    
};

class Meter : public QWidget
{
    Q_OBJECT

public:
    Meter(QWidget *parent = 0);

    void setSignal(int sig, int ch);    
    void draw(QPainter *painter);
    void resetPeaks();
    void startPeaks();

protected:
    void paintEvent(QPaintEvent *event);
    void mousePressEvent(QMouseEvent *event);
    
private:
    int sigleft, sigright;
    int peakleft, peakright;
    bool clippedl, clippedr;
};



class CyPosition : public QWidget
{
    Q_OBJECT

public:
    CyPosition(QWidget *parent = 0);

    void setPosition(int pos, int asyncpos, bool async);    
    void draw(QPainter *painter);

protected:
    void paintEvent(QPaintEvent *event);

private:
    int position;
    int asyncposition;
    bool asyncEnabled;
};

class MidiLearn : public QDialog {
    Q_OBJECT
    
public:
  MidiLearn(QWidget *parent = 0);
  void enableOkButton(bool enable);
  void currentSettings(QString cs);
  
  QPushButton *reset;
  bool zero;
  
  public slots:
    void midiSlot(int type, int num, int val, int ch, bool free);
    void resetted();
       
private:
  QWidget *main;
  QDialogButtonBox *buttonBox;
  QLabel *label1;
  QLabel *label2;
  QLabel *label3;
  QLabel *label4;
   
};

class Preferences : public QDialog {
    Q_OBJECT
    
public:
   Preferences(AuxCycle *aux, QWidget *parent = 0);
   
   QSlider *qmsec;
   QComboBox *buf;
   QComboBox *ssl;
   QComboBox *ss;
   QSlider *sv;
   QSlider *slv;
   QCheckBox *grfx;
   QCheckBox *grsys;
   QCheckBox *rels;
   QCheckBox *dbg;
   
private:
   QDialogButtonBox *buttonBox;
   QLabel *quickFade;
   QLabel *quickFadeVal;
   QLabel *buffer;
   QLabel *reloadSession;
   QLabel *standardSL;
   QLabel *standardS;
   QLabel *standardV;
   QLabel *sooLooVol;
   QLabel *standardVVal;
   QLabel *standardsooLooVol;
   QLabel *graphicSystem;
   QLabel *graphicFx;
   QLabel *debug;
   
  private slots:
    void qmsecChanged(int v);
    void svChanged(int v);
    void slvChanged(int v);
    
};


#endif
 
