/* JackThread.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#include "JackThread.h"

extern "C" {
   #include "cyclonic.h"
}

JackThread::JackThread() {
    clientName = "Cyclone";
    rstate();
}

void JackThread::run() {
    char nome[16];
    strncpy(nome, clientName.toLatin1(), sizeof(nome) - 1);
    int yes = cyclonic(nome);
    if (!yes) {
      exec();
    } else {
      checkStatus(cyclonic_msg());
      exit(1);
    }
}

void JackThread::exec() {
    while (state() == 0) {
      checkStatus(cyclonic_msg());
      sl_verify();
      usleep(10000);
    }
    checkStatus(cyclonic_msg());
    emit jackFinished();
    exit(0);    
}

void JackThread::stop() {
    if (state() == 0) {
      cyclonic_stop();
    }
    exit(0);
}

void JackThread::killMidi() {
  cyclonic_kill_midi();
}


void JackThread::stopLooping() {
  cyclonic_stop_looping();
}

int JackThread::state() {
    return cyclonic_state();
}

void JackThread::checkStatus(const char *msg) {
  if (strcmp(msg,msg_old) != 0) {
    emit statusChanged(msg);
    strcpy(msg_old, msg);
  }
  struct statechange sc = check_bridge();
  if (sc.changed) {
    emit tabChanged(sc.oldi, sc.olds, sc.oldstate, sc.newi, sc.news, sc.newstate);
    if (sc.newstate > 0)
      emit scroll(sc.newi, sc.news);
  }
  if (pos != position() || ((asyncEnabled != async()) || (apos != asyncposition()))) {
    asyncEnabled = async();
    pos = position();
    apos = asyncposition();
    emit posChanged(pos, apos, (bool)async());
  }
  if (sigL != signalL()) {
    sigL = signalL();
    emit sigChanged(sigL,0);
  }
  if (sigR != signalR()) {
    sigR = signalR();
    emit sigChanged(sigR,1);
  }
}

int JackThread::sooperlooper_connect() {
  int ok = sl_connect();
  if (ok == 0)
    setsooloovol(slvol);
  return ok;
}

void JackThread::sooperlooper_volume() {
  if (cystate == 0)
    setsooloovol(slvol);
}

void JackThread::setDebug(int dbg) {
  setDebugCyclonic(dbg);
}

void JackThread::setBufferFrames(int bf) {
  setBufferFramesCyclonic(bf);
}

void JackThread::setQuickFade(int qf) {
  setQuickFadeCyclonic(qf);
}

void JackThread::setVolume(int volume) {
  set_general_volume(volume);
}

void JackThread::setStandardCoord(int ii, int ss) {
  set_standard_coord(ii, ss);
}

snd_seq_t *JackThread::getMidiSeq() {
  return seqcy;
}

int JackThread::getMidiPort() {
  return midi_in_id;
}

void JackThread::rstate() {
  cystate = -1;
}
