/* .cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */

#include "MidiThread.h"

MidiThread::MidiThread() {
  runmidi = true;
  //reset();
}

void MidiThread::resetGlobal() {
  for (int a = 0; a < 3; a++)
    for (int b = 0; b < 16; b++)
      for (int c = 0; c < 128; c++)
	midig[a][b][c].active = false;  
}

void MidiThread::reset(int session) {
  int s_start, s_end;
  bool gtable = false;
  if (session == 255) {
    s_start = 0;
    s_end = 128;
    gtable = true;
  } else {
    s_start = session;
    s_end = session + 1;
  }
  for (int a = 0; a < 3; a++)
    for (int b = 0; b < 16; b++)
      for (int c = 0; c < 128; c++) {
	if (gtable)
	  midig[a][b][c].active = false;
	for (int d = s_start; d < s_end; d++)
	  midix[a][b][c][d].active = false;
      }
}

void MidiThread::run() {
  if (debug)
    fprintf(stdout, "Starting MIDI thread\n");
  exec();
  runmidi = true;
}

void MidiThread::exec() {  
  midicmd mc;
  while (runmidi) {
    snd_seq_event_t *event;
    snd_seq_event_input(cyseq, &event);
    if (event)
      mc = processMidi(event);
    if (mc.type < 128) {
      emit midiCommand(mc.type, mc.num, mc.val, mc.ch);
      mc.type = 128;
    }
    usleep(100);
  }
  if (debug)
    fprintf(stdout, "Stopping MIDI thread\n");
  exit(0);    
}

void MidiThread::stop() {
  runmidi = false;
}

void MidiThread::clearInput() {
  snd_seq_drop_input(cyseq);
}

void MidiThread::setDebug(int dbg) {
  debug = dbg;
}

void MidiThread::setSeq(snd_seq_t *seq, int port) {
  cyseq = seq;
  cyport = port;
  snd_seq_nonblock(cyseq,1);
}

midicmd MidiThread::processMidi(snd_seq_event_t *event) {
  midicmd mc;
  mc.type = 128;
  mc.num = 0;
  mc.val = 0;
  mc.ch = 0;
  switch (event->type) {
    case SND_SEQ_EVENT_NOTEON:
      mc.type = 0;
      mc.ch = event->data.note.channel;
      mc.num = event->data.note.note;
      mc.val = event->data.note.velocity;
      break;
    case SND_SEQ_EVENT_CONTROLLER:
      mc.type = 1;
      mc.ch = event->data.control.channel;
      mc.num = event->data.control.param;
      mc.val = event->data.control.value;
      break;
    case SND_SEQ_EVENT_PGMCHANGE:
      mc.type = 2;
      mc.ch = event->data.control.channel;
      mc.num = event->data.control.value;
      mc.val = 0;
      break;
    case SND_SEQ_EVENT_CONTROL14:
      mc.type = 3;
      mc.ch = event->data.control.channel;
      mc.num = event->data.control.param;
      mc.val = event->data.control.value;
      break;
    case SND_SEQ_EVENT_NONREGPARAM:
      mc.type = 4;
      mc.ch = event->data.control.channel;
      mc.num = event->data.control.param;
      mc.val = event->data.control.value;
      break;
    case SND_SEQ_EVENT_REGPARAM:
      mc.type = 5;
      mc.ch = event->data.control.channel;
      mc.num = event->data.control.param;
      mc.val = event->data.control.value;
      break;
  }
  if (debug) {
    switch (mc.type) {
      case 0:
	fprintf(stdout,"Note\t\t\tval:%d\t\tch:%d\t\tvel:%d\n", mc.num, mc.ch, mc.val);
	break;
      case 1:
	fprintf(stdout,"Control Change\t\tnum:%d\t\tch:%d\t\tval:%d\n", mc.num, mc.ch, mc.val);
	break;
      case 2:
	fprintf(stdout,"Program Change\t\tprg:%d\t\tch:%d\n", mc.num, mc.ch);
	break;
      case 3:
	fprintf(stdout,"Control Change14\t\tnum:%d\t\tch:%d\t\tval:%d\n", mc.num, mc.ch, mc.val);
	break;
      case 4:
	fprintf(stdout,"Non reg param\t\tnum:%d\t\tch:%d\t\tval:%d\n", mc.num, mc.ch, mc.val);
	break;
      case 5:
	fprintf(stdout,"Reg param\t\t\tnum:%d\t\tch:%d\t\tval:%d\n", mc.num, mc.ch, mc.val);
	break;
    }
  }
  return mc;
}

void MidiThread::signalTrig(int type, int num, int val, int ch, bool free) {
  emit mlSignal(type, num, val, ch, free);
}