/* MidiThread.h
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Sergio Atzori, 2010
 *
 *
 */ 

#ifndef MIDITHREAD_H
#define MIDITHREAD_H

#include <alsa/asoundlib.h>
#include <QThread>
#include "typedefs.h"

class MidiThread : public QThread {
    Q_OBJECT

public:
    MidiThread();
    void stop();
    void reset(int session = 255);
    void resetGlobal();
    void setDebug(int dbg);
    void setSeq(snd_seq_t *seq, int port);
    void clearInput();
    void signalTrig(int type, int num, int val, int ch, bool free);
    
    /* Midi Table: [type(NT-PC-CC)][midi_ch][event][session] */
    midicell midix[3][16][128][128];
    /* Global Midi Table: [type(NT-PC-CC)][midi_ch][event] */
    midicell midig[3][16][128];
    
  signals:
    void mlSignal(int type, int num, int val, int ch, bool free);
    void midiCommand(int type, int num, int val, int ch);
    
protected:
    void run();
    void exec();

private:
    void checkStatus(const char *msg);
    midicmd processMidi(snd_seq_event_t *event);
    
    snd_seq_t *cyseq;
    int cyport;
    bool runmidi;
    int debug;
};

#endif