 /* SooCycle.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  Sergio Atzori, 2010
 *
 *
 */ 

#include <QtGui>
#include <QtXml>

#include "SooCycle.h"

SooCycle::SooCycle(QWidget *parent) {
  parentWidget = parent;
  QDesktopServices home;
  slconf = new QFile(QString("%1/.sooperlooper/gui_config.xml") .arg(home.storageLocation(QDesktopServices::HomeLocation)));
  if (!slconf->exists()) {
    printOutput(QString("Soopelooper configuration file not found :-( %1") .arg(slconf->fileName()));
    return;
  }
  for (int a = 0; a < 4/* SooperLooper commands */; a++) {
    exist[a]= false;
  }
  if (handleConf())
    printOutput(QString("Soopelooper configuration file succesfully processed."));
  slconf->close();
}

void SooCycle::printOutput(QString string) {
  QTextStream cout(stdout, QIODevice::WriteOnly);
  cout << string << QString("\n");
}

bool SooCycle::handleConf() {
  if (!slconf->open(QIODevice::ReadOnly)) {
    printOutput(QString("Cannot open Sooperlooper config file."));
    return false;
  }
  QDomDocument *slconfig = new QDomDocument("xml");
  if (!slconfig->setContent(slconf)) {
    printOutput(QString("Cannot understand Sooperlooper config, maybe it's corrupted.."));
    return false;
  }
  QDomElement rad = slconfig->documentElement();
  if (rad.tagName() != "SLConfig") {
    printOutput(QString("Cannot understand Sooperlooper config ROOT TAG, maybe it's corrupted.."));
    return false;
  }
  QDomElement spconf = rad.firstChildElement("SpawnConfig");
  if (spconf.tagName() == "") {
    printOutput(QString("Cannot understand Sooperlooper config SPAWNCONFIG TAG, maybe it's corrupted.."));
    return false;
  }
  QDomElement context = spconf.firstChildElement("context");
  if (context.tagName() == "") {
    printOutput(QString("Cannot understand Sooperlooper config CONTEXT TAG, maybe it's corrupted.."));
    return false;
  }
  QString midibind = context.attribute("midi_bind_path", "");
  QString midibindfile;
  QDesktopServices home;
  if (midibind == "") {
    midibindfile = QString("%1/.sooperlooper/default_midi.slb") .arg(home.storageLocation(QDesktopServices::HomeLocation));
  } else {
    midibindfile = midibind;
  }
  midi_bind = new QFile(midibindfile);
  if (!midi_bind->open(QIODevice::ReadOnly)) {
    printOutput(QString("Cannot open Sooperlooper midi_bind file."));
    return false;
  }
  bool ok = handleMidibind();
  midi_bind->close();
  if (!ok)
    ok = modifyMidibind();
  return ok;
}

bool SooCycle::handleMidibind() {
  char str[128];
  bool ok = false;
  while (!midi_bind->atEnd()) {
    midi_bind->readLine(str, (qint64)128);
    QString midiline(str);
    if (midiline.contains("15") && midiline.contains("n 0") && midiline.contains("record") && midiline.contains("-1") && midiline.contains("norm") && midiline.contains("127"))
      exist[0] = true;
    if (midiline.contains("15") && midiline.contains("n 1") && midiline.contains("overdub") && midiline.contains("-1") && midiline.contains("norm") && midiline.contains("127"))
      exist[1] = true;
    if (midiline.contains("15") && midiline.contains("n 2") && midiline.contains("mute_on") && midiline.contains("-1") && midiline.contains("norm") && midiline.contains("127"))
      exist[2] = true;
    if (midiline.contains("15") && midiline.contains("cc 0") && midiline.contains("wet") && midiline.contains("-1") && midiline.contains("norm") && midiline.contains("127"))
      exist[3] = true;
  }
  for (int a = 0; a < 4/* SooperLooper commands */; a++) {
    if (exist[a]) {
      ok = true;
    } else {
      ok = false;
      break;
    }
  }
  return ok;
}

bool SooCycle::modifyMidibind() {
  printOutput(QString("Modifing current Soopelooper config."));
  if (!midi_bind->open(QIODevice::ReadWrite)) {
    printOutput(QString("Cannot modify Sooperlooper midi_bind file."));
    return false;
  }
  QTextStream mb(midi_bind);
  while (!mb.atEnd())
    mb.readLine((qint64)128);
  QString record = "15 n 0  hit record -1  0 1  norm 0 127\n";
  QString overdub = "15 n 1  hit overdub -1  0 1  norm 0 127\n";
  QString mute_on = "15 n 2  hit mute_on -1  0 1  norm 0 127\n";
  QString volume = "15 cc 0  set wet -1  0 1  norm 0 127\n";
  if (!exist[0])
    mb << record;
  if (!exist[1])
    mb << overdub;
  if (!exist[2])
    mb << mute_on;
  if (!exist[3])
    mb << volume;
  midi_bind->close();
  QMessageBox::warning(parentWidget, tr("Cyclone - Soopelooper config"), 
		       tr("Sooperloop configuration refreshed to receive "
			   "Cyclone commands. Restart Sooperlooper if launched."), QMessageBox::Ok);
  return true;
}