 /* WinCycle.cpp
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  Sergio Atzori, 2010
 *
 *
 */ 

#include <QtGui>

#include "WinCycle.h"
#include "SooCycle.h"

WinCycle::WinCycle(QString file) {
  fStart = true;
  QSize sh(695,520);
  resize(sh);
  SooCycle sooploop(this);
  undo = new CyUndo(this);
  sessions = new CyTabWidget(this);
  cymidi = new CyMidi(this);
  aux = new AuxCycle(this);
  aux->readSettings();
  ui = new CyUi(this);
  QApplication::setGraphicsSystem(aux->grsys);
  windebug = aux->standardDebug;
  connect(aux, SIGNAL(titleChanged(QString)), this, SLOT(refreshTitle(QString)));
  connect(undo, SIGNAL(canUndoChanged(bool)), this, SLOT(winMod(bool)));
  refreshTitle(aux->projectName);
  QSize minDim(600,520);
  setMinimumSize(minDim);
  ui->setupUi();
  sessions->played = false;
  jax.setDebug(windebug);
  jax.setBufferFrames(aux->standardBuffer);
  jax.setQuickFade(aux->standardQuickfade);
  jax.start();
  jax.slvol = aux->sooloovol;
  cymidi->setListenMidi(false);
  setWindowIcon(QIcon(":icons/cyclone.png"));
  while (jax.state() == -1)
    usleep(1000);
  if (jax.state() == 0) {
    mix.setSeq(jax.getMidiSeq(), jax.getMidiPort());
    mix.setDebug(windebug);
    mix.start();
    connect(&mix, SIGNAL(midiCommand(int,int,int,int)), cymidi, SLOT(midi(int,int,int,int)));
    csl();
  } else {
    jax.rstate();
  }
  if (file.isEmpty()) {
    if (aux->restoreLastSession == 1) {
      if (aux->testOpen(aux->currentFile))
	aux->reloadLastSession();
    } else {
      usleep(2000000);
    }
  } else {
    if (aux->testOpen(file))
      aux->open(file);
  }
  fStart = false;
  start();
  focus = "Resources"; /* Set focus to Resources, the only widget for now */
}

void WinCycle::refreshTitle(QString title) {
  sessions->currentSessionName = title;
  setWindowTitle(tr("%1[*] - %2 v.%3") .arg(sessions->currentSessionName) .arg(aux->appName) .arg(aux->appVersion));
  undo->clean();
}

void WinCycle::printOutput(QString string) {
  QTextStream cout(stdout, QIODevice::WriteOnly);
  cout << string << QString("\n");
}
void WinCycle::jackUp() {
  startJack->setDisabled(true);
  stopJack->setEnabled(true);
  connectSooperlooper->setEnabled(true);
}

void WinCycle::jackDown() {
  connectSooperlooper->setDisabled(true);
  stopJack->setDisabled(true);
  startJack->setEnabled(true); 
}

void WinCycle::stopJ() {
  mix.stop();
  mix.wait();
  jax.stop();
  jax.wait();
  jax.rstate();
  stop(true);
  for (int ss = 0; ss < sessions->sessionsCount; ss++) {
    for (int lp = 0; lp < sessions->musicTable[ss]->rowCount(); lp++) {
      sessions->loopz[lp][ss]->destroyLoopBuffer();   
    }
  }
}

void WinCycle::shutdownJ() {
  jax.rstate();
  mix.stop();
  mix.wait();  
  jax.killMidi();
  stop(true);
}

void WinCycle::csl() {
  jax.sooperlooper_connect();
}

void WinCycle::startJ() {
  jax.start();
  while (jax.state() == -1)
    usleep(1000);
  if (jax.state() == 0) {
    mix.setSeq(jax.getMidiSeq(), jax.getMidiPort());
    mix.setDebug(windebug);
    mix.start();
    connect(&mix, SIGNAL(midiCommand(int,int,int,int)), cymidi, SLOT(midi(int,int,int,int)));
    csl();
    QApplication::setOverrideCursor(Qt::WaitCursor);
    stop(true);
    int csfake;
    csfake = sessions->currentSession;
    for (int ss = 0; ss < sessions->sessionsCount; ss++) {
      sessions->currentSession = ss;
      sessions->refreshMusicTable(ss, false);
      for (int lp = 0; lp < sessions->musicTable[ss]->rowCount(); lp++) {
	sessions->loopz[lp][ss]->setLoopBuffer(lp,ss);   
      }
    }
    sessions->currentSession = csfake;
    QApplication::restoreOverrideCursor();
  }
}

void WinCycle::about()
{
  QMessageBox::information(this, aux->appName,
      tr("<h3><b>%1</b> version %2 - <small>svn %5</small></h3><h4 align=right>by <b>%3</b>  </h4>"
	  "<h6>This Application lets you enjoy your favorite loops</h6>"
	  "<h6>You need a working JACK setup</h6>"
	  "<h6>Simply drag&drop loops into %4 and start looping!<br></h6>"
	  " This program is free software: you can redistribute it and/or modify"
	  " it under the terms of the GNU General Public License version 2 as"
	  " published by the Free Software Foundation.</h6>") .arg(aux->appName) .arg(aux->appVersion) .arg(aux->appAuthor) .arg(aux->appName) .arg(SVN_VERS));
}

bool WinCycle::okToContinue() {
  if (isWindowModified()) {
    int res = QMessageBox::warning(this, aux->appName, tr("The project has been modified.\n"
	  "Do you want to save your changes?"), QMessageBox::Yes|QMessageBox::No|QMessageBox::Cancel);
    if (res == QMessageBox::Yes) {
      return aux->saveFile();
    } else {
      if (res == QMessageBox::Cancel)
	return false;
    }
  }
  return true;
}

void WinCycle::closeEvent(QCloseEvent *event)
{
  QApplication::setOverrideCursor(Qt::WaitCursor);
  if (aux->isPl() && closeProject()) {
    aux->colid = sessions->musicTable[sessions->currentSession]->columnWidth(0);
    aux->colname = sessions->musicTable[sessions->currentSession]->columnWidth(1);
    aux->colsl = sessions->musicTable[sessions->currentSession]->columnWidth(2);
    aux->cols = sessions->musicTable[sessions->currentSession]->columnWidth(3);
    aux->colvol = sessions->musicTable[sessions->currentSession]->columnWidth(4);
    aux->colbtn = sessions->musicTable[sessions->currentSession]->columnWidth(5);
    aux->colsel = sessions->musicTable[sessions->currentSession]->columnWidth(6);
    aux->writeSettings();
    stopJ();
    jax.wait();
    jax.terminate();
    mix.wait();
    mix.terminate();
    QApplication::restoreOverrideCursor();
    event->accept(); 
  } else {
    QApplication::restoreOverrideCursor(); 
    event->ignore();
  }
}

void WinCycle::midiLearnButton(int x, int y, int type) {
  MidiLearn ml(this);
  bool rst = false;
  QString cs;
  midiPlayFunction pfz;
  midiVolumeFunction pvz;
  switch (type) {
    case 0:
      cymidi->localSearch = true;
      cymidi->localTrig = true;
      pfz = sessions->loopz[x][y]->getMidi();
      if (pfz.fn == NT)
	cs = tr("Current Setting: Note <b>%1</b>, midi ch <b>%2</b>") .arg(pfz.val) .arg(pfz.ch + 1);
      else
	cs = tr("Current Setting: Program Change <b>%1</b>, midi ch <b>%2</b>") .arg(pfz.val) .arg(pfz.ch + 1);
      break;
    case 1:
      cymidi->localSearch = true;
      cymidi->localTrig = false;
      pvz = sessions->loopz[x][y]->getMidiV();
      cs = tr("Current Setting: Controller <b>%1</b>, midi ch <b>%2</b>") .arg(pvz.val) .arg(pvz.ch + 1);
      break;
    case 2:
      cymidi->localSearch = false;
      cymidi->localTrig = true;
      if (globalmidifunctions[x + 1].type == 0) {
	cs = tr("Current Setting: Note <b>%1</b>, midi ch <b>%2</b>")
				      .arg(globalmidifunctions[x + 1].num) .arg(globalmidifunctions[x + 1].ch + 1);
      } else {
	cs = tr("Current Setting: Program Change <b>%1</b>, midi ch <b>%2</b>")
				      .arg(globalmidifunctions[x + 1].num) .arg(globalmidifunctions[x + 1].ch + 1);
      }
      if (globalmidifunctions[x + 1].num == 255) {
	cs = tr("Current Setting: <b>NONE</b>");
	rst = true;
	ml.reset->setEnabled(false);
      }
      break;
    case 3:
      cymidi->localSearch = false;
      cymidi->localTrig = false;
      cs = tr("Current Setting: Controller <b>%1</b>, midi ch <b>%2</b>") .arg(globalmidifunctions[0].num) .arg(globalmidifunctions[0].ch + 1);
      if (globalmidifunctions[0].num == 255) {
	cs = tr("Current Setting: <b>NONE</b>"); 
	rst = true;    
	ml.reset->setEnabled(false);
      }
      break;
  }
  if ((type < 2) && ((pfz.val == 255) || (pvz.val == 255))) {
    cs = tr("Current Setting: <b>NONE</b>");
    rst = true;
    ml.reset->setEnabled(false);
  }
  cymidi->setListenMidi(true);
  cymidi->catchlock = false;
  connect(&mix, SIGNAL(mlSignal(int,int,int,int,bool)), &ml, SLOT(midiSlot(int,int,int,int,bool)));
  ml.currentSettings(cs);
  if (ml.exec()) {
    cymidi->setListenMidi(false);
    if (!ml.zero) {
      switch (type) {
	case 0: /* table play */
	case 1: /* table volume  */
	  undo->push(new CommandMidiAssign(this, x, y, cymidi->mc, type));
	  break;
	case 2: /* global trig */
	case 3: /* global volume */
	  undo->push(new CommandGlobalMidiAssign(this, cymidi->mc, type, x));
	  break;
      }
    } else {
      switch (type) {
	case 0: /* table play */
	case 1: /* table volume  */
	  cymidi->mc.num = 255;
	  if (!rst)
	    undo->push(new CommandMidiAssign(this, x, y, cymidi->mc, type));
	  break;
	case 2: /* global trig */
	case 3: /* global volume */
	  cymidi->mc.num = 255;
	  if (!rst)
	    undo->push(new CommandGlobalMidiAssign(this, cymidi->mc, type, x));
	  break;
      }      
    }
  } else {
    cymidi->setListenMidi(false);
  }
  cymidi->localSearch = true;
}

void WinCycle::generalVolume(int vl) {
  ui->volLabel->setText(QString("%1") .arg(vl));
  if (!gvpress) {
    if (!volume->isSliderDown()) {
      bool yez = false;
      for (int u = 0; u < sessions->sessionsCount; u++) {
	if (sessions->musicTable[u]->rowCount() > 0)
	  yez = true;
      }
      if (yez) {
	undo->push(new CommandGlobalVolume(this, undo->oldvol, volume->value()));
      }
      undo->oldvol = volume->value();
    } else {
      gvpress = true;
    }
  }
  jax.setVolume(vl);
}

void WinCycle::generalVolumeUndo() {
  if (gvpress) {
    bool yez = false;
    for (int u = 0; u < sessions->sessionsCount; u++) {
      if (sessions->musicTable[u]->rowCount() > 0)
	yez = true;
    }
    if (yez) {
      undo->push(new CommandGlobalVolume(this, undo->oldvol, volume->value()), true);
    }
    gvpress = false;
  }
}
  
void WinCycle::stop(bool clrow) {
  if ((sessions->sessionsCount == 1) && (sessions->musicTable[sessions->currentSession]->rowCount() == 0))
    return;
  if (jax.state() == 0)
    jax.stopLooping();
  if (clrow) {
    for (int ss = 0; ss < sessions->sessionsCount; ss++) {
      for (int lp = 0; lp < sessions->musicTable[ss]->rowCount(); lp++) {
	if (sessions->loopz[lp][ss]->state != 0)
	  sessions->loopz[lp][ss]->state = 0;
	  QPushButton *pbtn = (QPushButton *) sessions->musicTable[ss]->cellWidget(lp, 5);
	  pbtn->setIcon(QIcon(":/icons/start.png"));
	  QWidget *dItem = new QWidget(sessions);
	  sessions->musicTable[ss]->setCellWidget(lp,6,dItem);
      }
    }
  }
  sessions->playD = false;
  playL->setIcon(QIcon(":/icons/start.png"));
  ui->sigMeter->resetPeaks();
  ui->sigMeter->startPeaks();
  ui->updateStatusSig(0,0);
  ui->updateStatusSig(0,1);  
  jax.setStandardCoord(0, sessions->currentSession);
}

void WinCycle::start() {
  sessions->playD = false;
  for (int ss = 0; ss < sessions->sessionsCount; ss++) {
    for (int lp = 0; lp < sessions->musicTable[ss]->rowCount(); lp++) {
      if (sessions->musicTable[ss]->cellWidget(lp,6) != NULL) {
	sessions->loopz[lp][ss]->state = 0;
	QWidget *dItem = new QWidget(sessions);
	sessions->musicTable[ss]->setCellWidget(lp,6,dItem);
	QPushButton *pbtn = (QPushButton *) sessions->musicTable[ss]->cellWidget(lp, 5);
	pbtn->setIcon(QIcon(":/icons/start.png"));
      }
    }
  }
}

void WinCycle::muteButton(bool mt) {
  if (mt) {
    mute->setIcon(QIcon(":icons/unmute.png"));
    volume->setEnabled(false);
    jax.setVolume(0);
  } else {
    mute->setIcon(QIcon(":icons/mute.png"));
    volume->setEnabled(true);
    jax.setVolume(volume->value());
  }
}

void WinCycle::newProject() {
  if (!aux->isPl()) {
    return;
  }
  if (closeProject()) {
    sessions->infinite_count = 1;
    sessions->addSession(sessions->infinite_count, sessions->currentSession);
    refreshTitle("Untitled");
    start();
    sessions->setCurrentIndex(0);
    aux->currentFile = "";
    undo->clean();
  }
}

bool WinCycle::closeProject() {
  if (okToContinue()) {
    if (!fStart) {
      stop(false);
    }
    for (int ss = 0; ss < sessions->sessionsCount; ss++) {
      if (ss > 0)
	sessions->removeTab(1);
      for (int lp = 0; lp < sessions->musicTable[ss]->rowCount(); lp++) {
	sessions->loopz[lp][ss]->closeLoop();
      }
      sessions->musicTable[ss]->clear();
    }    
    for (int a = 0; a < 13 /* Actual global functions */; a++) {
      globalmidifunctions[a].num = 255;
    }
    mix.reset();
    sessions->currentSession = 0;
    sessions->infinite_count = 1;
    sessions->sessionsCount = 0;
    return true;
  } else {
    return false;
  }
}

void WinCycle::preferences(bool forced) {
  if (!aux->isPl()) {
    return;
  } else {
    if (jax.state() == 0)
      stop(true);
  }
  Preferences pr(aux, this);
  if (pr.exec()) {
    if (pr.grsys->isChecked())
      aux->grsys = "opengl";
    else
      aux->grsys = "raster";
    aux->graphicsfxnext = (int)pr.grfx->isChecked();
    aux->restoreLastSession = (int)pr.rels->isChecked();
    aux->standardSL = aux->indexToSlmode(pr.ssl->currentIndex());
    aux->standardS = aux->indexToSmode(pr.ss->currentIndex());
    aux->standardV = pr.sv->value();
    int slvolume = pr.slv->value();
    if (aux->sooloovol != slvolume) {
      aux->sooloovol = slvolume;
      jax.slvol = slvolume;
      jax.sooperlooper_volume();
    }
    int bufx;
    switch (pr.buf->currentIndex()) {
      case 0:
	bufx = 128;
	break;
      case 1:
	bufx = 256;
	break;
      case 2:
	bufx = 512;
	break;
      case 3:
	bufx = 1024;
	break;
      case 4:
	bufx = 2048;
	break;
    }
    aux->standardQuickfade = pr.qmsec->value();
    aux->standardDebug = (int)pr.dbg->isChecked();
    jax.setDebug(windebug);
    if (aux->standardBuffer != bufx) {
      aux->standardBuffer = bufx;
      jax.setBufferFrames(bufx);
      if (!forced) {
	stopJ();
	startJ();
      }
    }
    jax.setQuickFade(aux->standardQuickfade);
  }
}

void WinCycle::refreshPaste() {
  if (aux->pasteL)
    pasteLoop->setDisabled(false);
  if (aux->pasteS)
    pasteSession->setDisabled(false);  
}

void WinCycle::winMod(bool mod) {
  saveProjectAction->setEnabled(mod);
  setWindowModified(mod);
  saveAsProjectAction->setEnabled(true);
}