/* bridge.h
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 */

#ifndef _CYBRIDGE_H_INCLUDED_
#define _CYBRIDGE_H_INCLUDED_ 

#include <sndfile.h>
#include <samplerate.h>
#include <jack/jack.h>
#include <jack/thread.h>
#include <jack/ringbuffer.h>
#include <alsa/asoundlib.h>

int debug;
int oldstate, oldonce, newstate, currentindex, oldindex, currentsession, oldsession, currentloopindex, currentloopsession, changestate, changeend, asyncLoopindex, asyncLoopsession;
int quicktrigger, quickvolume, quickavolume;
long quickmsec, currentposition, currentasyncposition, currentsamples;

struct loopdata {
  int index, session;
  char path[256];
  int buffer_bytes;
  int buffer_samples;
  float *e_buffer;
  float *d_buffer;
  float *j_buffer;
  float *k_buffer;
  SNDFILE *sound_file;
  int channels;
  int pipe[2];
  jack_ringbuffer_t *rb;
  pthread_t disk_thread;
  SRC_STATE *src;
  SF_INFO sfnfo;
  long position;
  int volume;
  int start;
  int end;
  int async;
};

struct loopdata asyncLoop;
int asyncEnabled;
struct loopdata currentLoop;
int currentEnabled;

struct loopdata loops[128][128];

int nextlooptrig, nexti, nexts, nextslm, nextsm, strig;
int slrec, sloverd, slmulti;
int tslrec, tsloverd, tslmulti;
jack_client_t *jack_client;
snd_seq_t *seq;
int midi_out;
long sample_rate;
int buffer_frames;
char *snderror;

int bridge(const char *path, struct loopdata *loop);
int bridgeopenbuffer(struct loopdata *loop, int li, int ls);
int bridgesetindex(struct loopdata *loop, int li, int ls);
void resetbridge(struct loopdata *loop);
void firststartbridge(struct loopdata *loop);
void deletebridge(struct loopdata *loop);
void modifyvolume(struct loopdata *loop, int vol);
void closefile(struct loopdata *loop);
long read_input_rb(void *PTR, float **buf);
int jack_ringbuffer_wait_for_write(int nbytes, struct loopdata *loop);
const char *errorstring(struct loopdata *loop);
float quick_volume(struct loopdata *loop);
float quick_async(struct loopdata *loop);
void endloop();
int endasync();
void setMidiSeq(snd_seq_t *seqcy, int port);
void process_sl_trigger();
void send_midi_sl(int type, int val);

void setCurrentLoop(struct loopdata loop);
void unsetCurrentLoop();
void disableLooping();
void totallyDisableLooping();
void enableLooping();

void setSamplerate(long sr);
void setJackClient(jack_client_t *cli);
void *disk_proc(void *PTR);


void setDebugBridge(int dbg);

#endif