/* loopin.c
 *
 *
 *    this file is part of:
 *
 *    Cyclone   ---   Loop Giant
 *
 * by Sergio Atzori, 2010
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "loopin.h"

int trigger(struct loopin lupin) {
  if (debug) {
    fprintf(stdout, "LOOPIN: Index: %i\t\t\tSession: %i\n",lupin.index,lupin.session);
    fprintf(stdout, "LOOPIN: SLmode: %i\t\t\tSmode: %i\n",lupin.slmode,lupin.smode);
    fprintf(stdout, "LOOPIN: state: %i\t\t\tstrig=%d\toldstate=%i\toldonce=%i\n",lupin.state, strig, oldstate, oldonce);
  }
  while (changestate || changeend) {
    usleep(5000);
    if (debug)
      fprintf(stdout, "LOOPIN: Waiting for state change\n");
  }
  int rstate = 0;
  if (lupin.smode == 3) {
    if ((((lupin.index != currentloopindex) || (lupin.session != currentloopsession)) && ((lupin.index != nexti) || (lupin.session != nexts))) || (!currentEnabled)) {
      if (lupin.state != 1) {
	loops[lupin.index][lupin.session].async = 1;
	if (lupin.state < 4)
	  lupin.state = 5;
	rstate = stateasync(lupin);
	return rstate;
      }
    } else {
      return -1;
    }
  } else {
    loops[lupin.index][lupin.session].async = 0;    
    if (lupin.state > 3)
      lupin.state = 0;
  }
  if ((lupin.index == asyncLoopindex) && (lupin.session == asyncLoopsession) && (asyncEnabled)) {
    loops[lupin.index][lupin.session].end = 1;
    endasync();
    loops[lupin.index][lupin.session].end = 0;
    return -1;
  }
  if (!currentEnabled)
      oldonce = 0;
  switch (lupin.state) {
    case 0:
      rstate = statezero(lupin);
      break;
    case 1:
      rstate = stateuno(lupin);
      break;
    case 2:
      rstate = statedue(lupin);
      break;
    case 3:
      rstate = statetre(/*lupin*/);
      break;
  }
  
  return rstate;
}

int statezero(struct loopin lupin) {
  int nstate;
  if (strig == 0 && (!oldonce)) {
    if (oldstate >= 0) {
      if ((((asyncLoopindex == currentloopindex) && (asyncLoopsession == currentloopsession)) || ((loops[asyncLoopindex][asyncLoopsession].index == currentindex) && (loops[asyncLoopindex][asyncLoopsession].session == currentsession))) && asyncEnabled) {
	oldstate = -1;
      } else {
	unsetCurrentLoop();
	oldstate = 0;
      }
    }
    if ((oldstate < 0) && (loops[currentloopindex][currentloopsession].start) && (!loops[currentloopindex][currentloopsession].async)) {
      unsetCurrentLoop();
      oldstate = 0;
    }
    nstate = 1;
    if (lupin.slmode > 0) {
      slrec = 1;
    }
    if (lupin.slmode > 2) {
      sloverd = 1;
    }
    if (((lupin.slmode == 1) || (lupin.slmode == 4)) && (lupin.smode != 2)) {
      nstate = 2;
      slmulti = 1;
    }
    if (!loops[lupin.index][lupin.session].start)
      firststartbridge(&loops[lupin.index][lupin.session]);
    setCurrentLoop(loops[lupin.index][lupin.session]);
    enableLooping();
    if (lupin.smode == 1) {
      strig = 1;
    }
    if (lupin.smode == 2) {
      oldonce = 1;
      nextlooptrig = 2;
      strig = 0;
    } else {
      oldonce = 0;
    }
  } else {
    if (nextlooptrig == 1) {
      oldstate = 0;
    } else {
      nextlooptrig = 1;
      oldstate = 1;
      oldindex = currentloopindex;
      oldsession = currentloopsession;   
    }
    nexti = lupin.index;
    nexts = lupin.session;
    nextslm = lupin.slmode;
    nextsm = lupin.smode;
    nstate = 3;
  }
  currentloopindex = lupin.index;
  currentloopsession = lupin.session;
  currentindex = loops[lupin.index][lupin.session].index;
  currentsession = loops[lupin.index][lupin.session].session;
  
  return nstate;
}

int stateuno(struct loopin lupin) {
  int nstate;
  if (nextlooptrig != 1) {
    if (lupin.smode == 2) {
      oldonce = 0;
    }
    if (strig == 0) {
      disableLooping();
      nstate = 0;
      oldstate = -1;
    } else {
      nextlooptrig = 2;
      nstate = 1;
      oldonce = 0;
    }
  } else {
    oldonce = 0;
    oldindex = nexti;
    oldsession = nexts;
    nextlooptrig = 2;
    while (changestate)
      usleep(1000);
    oldstate = 0;
    nexti = lupin.index;
    nexts = lupin.session;
    newstate = 1;
    nextlooptrig = 2;
    nstate = -1;
    changestate = 1;
  }
  process_sl_trigger();
  tslrec = 0;
  return nstate;
}

int statedue(struct loopin lupin) {
  if (nextlooptrig == 1) {
    nextlooptrig = 0;
  }
  currentloopindex = lupin.index;
  currentloopsession = lupin.session;
  currentindex = loops[lupin.index][lupin.session].index;
  currentsession = loops[lupin.index][lupin.session].session;
  oldstate = 0;
  slmulti = 0;
  return 1;
}

int statetre(/*struct loopin lupin*/) {
  currentloopindex = oldindex;
  currentloopsession = oldsession;
  currentindex = loops[oldindex][oldsession].index;
  currentsession = loops[oldindex][oldsession].session;
  oldstate = 0;
  nextlooptrig = 0;
  return 1;
}

int stateasync(struct loopin lupin) {
  if ((lupin.state == 5) && (asyncEnabled == 0)) {
    if (!loops[lupin.index][lupin.session].start)
      firststartbridge(&loops[lupin.index][lupin.session]);      
    asyncLoop = loops[lupin.index][lupin.session];
    asyncLoopindex = lupin.index;
    asyncLoopsession = lupin.session;
    asyncEnabled = 1;
    return 4;
  }
  
  if (lupin.state == 4) {
    asyncEnabled = 0;
    quickavolume = 0;
    resetbridge(&loops[lupin.index][lupin.session]);
    return 5;
  }
  
  return 5;
}